package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;

// for parsing multipart crud
import javax.mail.*;
import javax.mail.internet.*;

import java.io.*;

import com.arsdigita.acs.ACSException;

/**
 * Implements a multiset of name-value pairs.  Analogous to ns_set. 
 * Not really a Set implementation.... Implemented as Hashtable of Vectors.    
 */
public class AdSet implements DataSource {
   
    private Hashtable ht = new Hashtable();

    public AdSet() {
	super();
    }

    /**
     * Constructs an AdSet out of the parameter/value pairs in the servlet
     * request.
     * @param request The servlet request
     */
    public AdSet(ServletRequest request) {
	Enumeration e = request.getParameterNames();
	while (e.hasMoreElements()) {
	    String name = (String) e.nextElement();
	    String[] values = request.getParameterValues(name);
	    for (int i = 0; i < values.length; i++) {
		put(name, values[i]);
	    }
	}
    }

    /**
     * Constructs an AdSet out of the parameter/value pairs in the servlet
     * request.  Handles multipart requests for uploading files; adds
     * an element named <code>param_tmpfile</code> if a supplied form 
     * parameter named <code>param</code> is type=FILE.
     * <p>
     * <b>Warning</b>: this should only be called <b>once</b> for 
     * any given servlet request, because we cannot rewind the input
     * stream for a multipart request.
     *
     * @param request The servlet request
     */
    public AdSet(HttpServletRequest request) throws Exception {
	if (request.getContentType() != null && request.getContentType().toLowerCase().startsWith("multipart/form-data")) {
	    parseMultipartRequest(request);
	}
	else {
	    Enumeration e = request.getParameterNames();
	    while (e.hasMoreElements()) {
		String name = (String) e.nextElement();
		String[] values = request.getParameterValues(name);
		for (int i = 0; i < values.length; i++) {
		    put(name, values[i]);
		}
	    }
	}
    }

    /**
     * Constructs an AdSet out of an array of arrays of name-value pairs.
     *
     * @param objects an array of arrays of name value pairs. The first of the pair must be assignment compatible with String and the
     *                second with Object. ie, something of the form: <pre>
     * new Object[][] = { {"foo", new Foo()},
     *                    {"bar", new Bar()},
     *                    {"baz", new Baz()},
     *                    {"glorp", new Glorp()} }
     */
    public AdSet(Object[][] objects) throws Exception {
	for (int i = 0; i < objects.length; i += 1) {
	    if (objects[i].length != 2)
		throw new ACSException("Illegal argument format; should be an array of arrays of name-value pairs.");
	    put((String)objects[i][0], objects[i][1]);
	}
    }

    /**
     * Returns the values of an AdSet joined (in the Tcl sense) into one String.
     * Order of the list is dependent on (ie, the same as) the order returned by the keysAndValues() method.
     * @param delim what will be used to delimit the entries; null will default to ", ". Note however that "" will not default to anything.
     **/
    public String join(String delim) {
	if (delim == null)
	    delim = ", ";
	String retval = new String();
	Enumeration e = keysAndValues();
	if ( e.hasMoreElements() )
	    retval += ((String[])e.nextElement())[1];
	while ( e.hasMoreElements() )
	    retval += delim + ((String[])e.nextElement())[1];

	return retval;
    }

    /** 
     * Adds a key/value pair to this.  Ignores null values.
     * @param key The key
     * @param name The value
     */
    public void put(String key, String value) {
	if (value == null) 
	    // don't want null values inserted because it will
	    // cause iterator to not function properly
	    // since get(key) returns null if key is not found,
	    // this doesn't change the semantics of get().
	    return;

	Vector v = (Vector) ht.get(key);
	if (v == null) {
	    v = new Vector();
	    ht.put(key, v);
	}
	v.addElement(value);
    }
    
    /** 
     * Adds a key/value pair to this.  Ignores null values.
     * @param key The key
     * @param name The value
     */
    public void put(String key, Object value) {
	if (value != null) {
	    put(key, value.toString());
	}
    }

    /** 
     * Gets a value for a given key.
     * @param key The key
     * @return name The first value mapped from key, or <code>null</code>
     * if <code>key</code> is not mapped.
     */
    public String get(String key) {
	Vector v = (Vector) ht.get(key);
	if (v == null) {
	    return null;
	} else {
	    return (String) v.elementAt(0);
	}
    }

    /** 
     * Returns all values mapped from <code>key</code>.
     * @param key The key
     * @return An array of all values mapped from <code>key</code>
     */
    public String[] getAll(String key) {
	Vector v = (Vector) ht.get(key);
	if (v == null) {
	    return null;
	} else {
	    String[] s = new String[v.size()];
	    v.copyInto(s);
	    return s;
	}
    }

    /**
     * Returns true if AdSet contains <code>key</code>.
     */
    public boolean contains(String key) {
	return ht.contains(key);
    }

    /**
     * Returns <code>false</code>. For implementing DataSource.
     */
    public boolean next() {
	return false;
    }

    /**
     * Returns the current row number.
     */
    public int rownum() throws DataSourceException {
	return 1;
    }

    /**
     * Changes the first mapping from <code>key</code> to <code>value</code>.
     * @param key The key
     * @param value The value
     */
    public void update(String key, String value) {
	Vector v = (Vector) ht.get(key);
	
	if (v == null) {
	    v = new Vector();
	    ht.put(key, v);
	}
	
	if (v.size() == 0) {
	    v.addElement(value);
	} else {
	    v.setElementAt(value, 0);
	}
    }

    /**
     * Removes <code>key</code> from this.
     * @param key The key to remove
     */
    public void deleteKey(String key) {
	ht.remove(key);
    }

    /**
     * Removes all keys from this AdSet.
     */
    public void clear() {
        ht.clear();
    }

    /**
     * @return An array of keys in this AdSet.
     */
    public String[] keys() {
	Enumeration e = ht.keys();
	Vector v = new Vector();
	while (e.hasMoreElements()) {
	    v.addElement(e.nextElement());
	}
	String[] retval = new String[v.size()];
	v.copyInto(retval);
	return retval;	
    }

    /**
     * @return An enumeration of keys in this AdSet.
     */
    public Enumeration keysAsEnumeration() {
	return ht.keys();
    }

    /**
     * @return An iterator of String[2], where the first element is a key
     * in this and the second element is a value mapped from that key.
     */
    public Enumeration keysAndValues() {
	return new AdSetEnumeration(this);
    }

    /**
     * Returns number of keys in this AdSet.
     * @return the number of keys in this AdSet.
     */
    public int size() {
	return ht.size();
    }


    private void parseMultipartRequest(HttpServletRequest req) throws Exception
    // utility class for parsing multipart requests into AdSets
    // NOTE: This should only be called *once* per request, 
    // because the input stream may only be read a single time.
    {
	ServletRequestDataSource srds = new ServletRequestDataSource(req);
	MimeMultipart mm = new MimeMultipart(srds);
	int count = mm.getCount();	    
	
	for (int i=0; i<count; i++) {
	    BodyPart bp = mm.getBodyPart(i); 
	    String disp = bp.getHeader("Content-Disposition")[0];
	    Object content = bp.getContent();
	    int start = disp.indexOf("name=\"");
	    // param name starts at start + 6
	    // find the next quote
	    int end = disp.indexOf('"', start + 6);
	    String paramName = disp.substring(start+6,end);
	    // check to see if it's an uploaded file
	    int fnStart = disp.indexOf("filename=\"");
	    if (fnStart == -1) {
		// nope, so the content better be a string
		if (content instanceof String) {
		    this.put(paramName, content);
		} else {
		    // not a file, and not a string, so die
		    throw new IllegalArgumentException("Oops, no filename provided");
		}
	    } else {
		// we've got an uploaded file
		// file name starts at fnStart + 10
		int fnEnd = disp.indexOf('"', fnStart + 10);
		String fileName = disp.substring(fnStart+10, fnEnd);
		this.put(paramName, fileName);
		// create and dump to a temp file
		File tmpFile = File.createTempFile("acs", null, null);
		tmpFile.deleteOnExit();
		this.put(paramName+ "_tmpfile", tmpFile.getPath()); 
		FileOutputStream fos = new FileOutputStream(tmpFile);
		if (content instanceof InputStream) {
		    // we've got an input stream of some sort
		    byte [] buffer = new byte[4096];
		    InputStream is = (InputStream)content;
		    int ct = 0;
		    while ((ct = is.read(buffer, 0, 4096)) != -1) {
			fos.write(buffer, 0, ct);
		    }
		    fos.close();
		} else if (content instanceof String) {
		    // we've got a string literal so dump it to the file
		    PrintWriter pw = new PrintWriter(new OutputStreamWriter (fos));
		    pw.print(content); 
		    pw.close();
		}
	    }
	}
    }

    private class AdSetEnumeration implements Enumeration {
	private AdSet set;
	Enumeration keys;
	String currentKey;
	Enumeration values;
	String currentValue;

	public AdSetEnumeration(AdSet s) {
	    set = s;
	    keys = s.keysAsEnumeration();

	    if (keys.hasMoreElements()) {
		currentKey = (String) keys.nextElement();
		values = ((Vector) set.ht.get(currentKey)).elements();
		if (values.hasMoreElements()) {
		    currentValue = (String) values.nextElement();
		}
	    }
	}

	public boolean hasMoreElements() {
	    return currentValue != null;
	}

	public Object nextElement() {
	    if (currentValue == null) {
		throw new NoSuchElementException();
	    }
	    String[] retval = new String[2];
	    retval[0] = currentKey;
	    retval[1] = currentValue;

	    if (values.hasMoreElements()) {
		currentValue = (String) values.nextElement();
	    } else if (keys.hasMoreElements()) {
		currentKey = (String) keys.nextElement();
		values = ((Vector) set.ht.get(currentKey)).elements();
		currentValue = (String) values.nextElement();
	    } else {
		currentValue = null;
	    }
	    return retval;
	}
    }

}
