package com.arsdigita.acs;
import com.arsdigita.acs.db.Database;

import java.io.*;
import java.util.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.http.*;
import com.oroinc.text.perl.Perl5Util;
import org.apache.turbine.util.Log;

/**
 * Class for working with the client identification cookies
 * (browser_id, session_id, last_hit, etc.)
 *
 * Normally this object will be initialized by the Request Processor
 * and stored as the "client_identification" attribute for the
 * current request.
 * 
 * @author David Eison (eison@arsdigita.com)
 * @version $Version$
 * @see com.arsdigita.acs.RequestProcessor
 */
public class ClientIdentification {
    private Long browserId = null;
    private Long sessionId = null;
    private Integer userId = null;
    private String token = null;
    private Long lastIssue = null;
    private HttpServletRequest req = null;
    private HttpServletResponse resp = null;

    /**
     * Initializes the client identification information from the ad_browser_id and ad_session_id cookies.
     */
    public ClientIdentification (HttpServletRequest req, HttpServletResponse resp) {
        this.resp = resp;
        this.req = req;

        Perl5Util re = new Perl5Util();
        String ad_browser_id = null;
        String ad_session_id = null;
        try {
            ad_browser_id= Utilities.getCookieValue(req, "ad_browser_id");
            ad_session_id = Utilities.getCookieValue(req, "ad_session_id");
            if (ad_browser_id != null && re.match("/^([0-9]+)$/", ad_browser_id)) {
                browserId=new Long(re.group(0));
            }
            if (ad_session_id != null && re.match("/^([0-9]+),([0-9]*),([^,]*),([0-9]+)$/", Utilities.URLDecode(ad_session_id))) {
                sessionId = new Long(re.group(1));
                userId = new Integer(re.group(2));
                token = re.group(3);
                lastIssue = new Long (re.group(4));
            }
        } catch (Exception e) {
            // Fail quietly, these values will be regenerated.  This shouldn't happen unless the client has been mucking around with their cookies.
            Log.warn("ClientIdentification: ID Cookie check had an error, ID will be regenerated.  ad_browser_id cookie was " + ad_browser_id + ", ad_session_id cookie was " + ad_session_id + " (" + e.toString() + ")");
        }
    }

    /**
     * @return True if browser ID exists and is a number.
     */
    public boolean isBrowserIdValid() {
        return (browserId != null);
    }

    /**
     * @return True if SessionId exists, is a number, and lastIssue is within the timeout range.
     */
    public boolean isSessionIdValid() {
        long time = getTime();
        return (sessionId != null && lastIssue != null &&
                lastIssue.longValue() <= (time + getSessionTimeout()) && 
                lastIssue.longValue() + getSessionTimeout() >= time);
    }

    /**
     * Generates and assigns the session ID cookie.  Will also call the secure token function if
     * using SSL.
     */
    public void assignSessionId() throws SQLException, ACSException {
        Perl5Util re = new Perl5Util();
        Database db = new Database();
        Long time = new Long(getTime());

        String secureToken = null;
        String sessionIdString = db.queryToString("select sec_id_seq.nextval from dual");
        //Log.note("Assigning new session ID " + sessionIdString + "; session ID was " + sessionId + "; last_issue was " + lastIssue + "; current time is " + time + "; timeout is " + getSessionTimeout());
        this.sessionId = Long.valueOf(sessionIdString);
        this.token = Utilities.secRandomToken();
        if (req.isSecure()) {
            // Generate the secure token.
            secureToken = Utilities.secRandomToken();
            assignSecureTokenCookie(secureToken);
        }
        
	req.setAttribute("user_info", null);
        String loginToken;
        String login_cookie = Utilities.getCookieValue(req, "ad_user_login");
        if (login_cookie != null && re.match("/^([0-9]+),(.+)$/", Utilities.URLDecode(login_cookie))) {
            userId = Integer.valueOf(re.group(1));
            loginToken = re.group(2);
            String tokenCount = db.queryToString("select count(1) " + 
                                                 "from   users u, sec_login_tokens t " + 
                                                 "where  t.user_id = :user_id " +
                                                 "and    u.user_id = :user_id " +
                                                 "and    t.password = u.password " + 
                                                 "and    t.login_token = :login_token", new Object[] {userId, userId, loginToken});
            if (!tokenCount.equals("0")) {
                req.setAttribute("user_info", new UserInfo(userId.intValue()));
            }                                                 
        } else {
            // Assume 0 if no match.
            userId = new Integer(0);
        }

        Integer insertId;
        if ( req.getAttribute("user_info") == null) {
            insertId = null;
        } else {
            insertId = ((UserInfo)req.getAttribute("user_info")).getUserId();
            if (insertId.intValue() == 0) {
                // There is no user 0.
                insertId = null;
            }
        }

        // The TCL code has a nice ns_set interface for doing this.  We can't use an ad_set
        // because some of these might be null and it ignores null values.
        // We could use a Hashmap but it doesn't return its keys as a string or a vector,
        // so we'd need to write another join... we'll just write out the DML the long way instead.
        db.dml("insert into sec_sessions (session_id, user_id, token, secure_token, browser_id, last_ip, last_hit) " +
               "values " +
               "(:session_id, :insert_id, :token, :secure_token, :browser_id, :last_ip, :last_hit)",
               new Object[] {this.sessionId, insertId, this.token, secureToken, this.browserId, req.getRemoteAddr(), time});
        
        if (req.isSecure()) {
            req.setAttribute("sec_validated", "secure");
        } else {
            req.setAttribute("sec_validated", "insecure");
        }

        assignSessionIdCookie();

        // Update last_visit and second_to_last_visit
        db.dml("declare begin sec_rotate_last_visit(:browser_id, :current_time); end;", new Object[] {this.browserId, time});
        db.close();
    }

    /**
     * Generates and assigns the session ID cookie.
     */
    private void assignSessionIdCookie() {
        Cookie sessionIdCookie = new Cookie("ad_session_id", Utilities.URLEncode(sessionId + "," + userId + "," + token + "," + getTime()));
        sessionIdCookie.setMaxAge(getSessionTimeout());
        // The TCL code doesn't set the path, but it seems to be necessary for the Java.  Maybe it defaults to the current directory
        // in Java if not explicitly set?  Not sure.
        sessionIdCookie.setPath("/");
        //Log.note("Setting session ID Cookie " + sessionIdCookie.getName() + ":" + sessionIdCookie.getValue());
        resp.addCookie(sessionIdCookie);
    }

    /**
     * Assigns the secure token cookie based on the value passed in.
     * @param secureToken Value of cookie.
     */
    private void assignSecureTokenCookie(String secureToken) {
        // Sanity check - make sure we're using HTTPS.
        if (req.isSecure()) {
            Cookie secureTokenCookie = new Cookie("ad_secure_token", Utilities.URLEncode(secureToken));
            secureTokenCookie.setSecure(true);
            resp.addCookie(secureTokenCookie);
        } else {
            Log.error("Not setting secure token cookie becuase request is not secure.");
        }
    }

    /**
     * Generates and assigns browser ID cookie.
     */
    public void assignBrowserId() throws SQLException, ACSException {
        Database db = new Database();
        String browserIdString = db.queryToString("select sec_id_seq.nextval from dual");
        db.close();
        this.browserId = new Long(browserIdString);
	Cookie browserIdCookie = new Cookie("ad_browser_id", browserIdString);
        browserIdCookie.setPath("/");
        // This # came from Security.java's loginUser method.  I presume it was randomly selected as "really big number".
	browserIdCookie.setMaxAge(100000000);
	resp.addCookie(browserIdCookie);      
    }

    /**
     * # of seconds since Jan 1. 1970, same as the TCL version.
     */
    public static long getTime () {
        return Math.round(System.currentTimeMillis() / (double)1000);
    }

    /**
     * This updates the session ID cookie w/ a new last update value if it has been longer than the
     * timeout since the last time it was updated.
     */
    public void maybeUpdateLastHit() throws SQLException {
        // Called when the session already exists and is valid.
        long currentHit = getTime();
        if (lastIssue.longValue() + getCookieReissue() < currentHit) {
            //Log.note("Updating lastHit (" + (lastIssue.longValue() + getCookieReissue()) + " < " + currentHit + ")");
            Database db = new Database();
            // TODO: Add deferDML to db class?
            //db.deferDML("update sec_sessions set last_hit = :last_hit where session_id = :session_id", new Object[] {lastHit, sessionId});
            db.dml("update sec_sessions set last_hit = :last_hit where session_id = :session_id", new Object[] {new Long(currentHit), sessionId});
            db.close();
            MemoizeCache.flush("sec_get_session_info " + sessionId);
            assignSessionIdCookie(); // This just reissues with the current time.
        } 
    }

    /**
     * Returns the timeout, in seconds, for sessions.
     */
    public static int getSessionTimeout() {
        return Parameters.getInt("SessionTimeout", "", 86400);
    }

    /**
     * Returns the period, in seconds, after which we should reissue the session_id cookie and update last_hit in the sessions table.
     */
    public static int getCookieReissue() {
        return Parameters.getInt("SessionCookieReissue", "", 10);
    }

    /**
     * Get debugging string.
     */
    public String toString() {
        return "b=" + browserId.toString() + ", s=" + sessionId.toString() + ", u=" + userId + ", t=" + token + ", l=" + lastIssue;
    }

    /**
     * Gets unique identifier number for this browser (from cookie)
     * @return browser id.
     */
    public Long getBrowserId() {
        return browserId;
    }
    
    /**
     * Gets unique identifier number for this session.  Should check isSessionIdValid() to see if it has timed out.
     * @return session id.
     */
    public Long getSessionId() {
        return sessionId;
    }

    /**
     * Gets last time the session identification cookie was issued (this cookie is reissued periodically to keep
     * the session "alive".  It is not reissued every time the user views a page to reduce load on the server and
     * database.)
     *
     * @return Time of last cookie issue in <em>seconds</em> (not ms) 
     * since 1970
     */
    public Long getLastIssue() {
        return lastIssue;
    }

    /**
     * Returns the time of the last visit as a java.util.Date, to prevent
     * confusion about time units  (seconds vs. ms)
     * 
     * @see java.util.Date
     * @return last visit time
     */
    public java.util.Date getLastVisit() { 
	if (lastIssue == null) 
	    return null;
	else 
	    return new java.util.Date(lastIssue.longValue() * 1000);
    }

    /**
     * Changes the current user id.  This regenerates the session ID cookie and also 
     * updates the session info in the database.  Use 0 for no user.
     *
     * @param new User Id.
     */
    public void resetUserId(Integer userId) throws SQLException {
        this.userId = userId;
        assignSessionIdCookie();
        Database db = new Database();
        if (userId.intValue() == 0) {
            userId = null;
        }
        db.dml("update sec_sessions " + 
               "set user_id = :user_id " +
               "where session_id = :session_id", new Object[] {userId, sessionId});
    }

    /**
     * Changes the current user id.  Caller should regenerate session ID after calling this function.
     *
     * @param new User Id.
     */
    public void resetUserId(int userId) throws SQLException {
        resetUserId(new Integer(userId));
    }

}
