/* 
 * GeneralPermissions.java
 *  
 * ACS General Permissions module
 *
 * $Id: GeneralPermissions.java,v 1.5 2000/09/21 14:13:59 luke Exp $ 
 * 
 */

package com.arsdigita.acs;

import com.arsdigita.acs.db.*;
import java.sql.*;

public class GeneralPermissions {
    public static boolean userHasPermission(Integer userId,
					    String permission_type,
					    String onWhatId,
					    String onWhichTable) 
    throws ACSException {
	try {
	    Database db = new Database();
	    Boolean everyoneHasPermission = 
		new Boolean(db.queryToString("select decode(ad_general_permissions.all_users_permission_id(:permission_type, :on_what_id, :on_which_table), 0, 'false', 'true') from dual", new Object[] {permission_type, onWhatId, onWhichTable}));
	    if (everyoneHasPermission.booleanValue()) {
		// everyone can do this
		db.close();
		return true;
	    } 
	    if (userId == null) {
		db.close();
		return false;
	    }

	    Boolean result = new Boolean(db.queryToString("select decode(ad_general_permissions.user_has_row_permission_p(?, ?, ?, ?), 't', 'true', 'false') from dual", new Object[] {userId, permission_type, onWhatId, onWhichTable}));
	    db.close();
	    return result.booleanValue();
	}
	catch (SQLException sqle) {
	    throw new ACSException(sqle.getMessage());
	}
    }

    /**
     * Returns the number of permissions records there are on a certain
     * object, of a certain permission type.   Useful for knowing if you're
     * about to revoke the last admin permission for a record.
     */
    public static int permissionCount(Database db,
			       Integer onWhatId, 
			       String onWhichTable, 
			       String permissionType) 
	throws ACSException, SQLException  {
	String query = "select count(*) from general_permissions " + 
	    "where on_what_id = :on_what_id " + 
	    " and on_which_table = lower(:on_which_table) ";
	Object[] binds;
	if (!Utilities.emptyString(permissionType)) {
	    query += " and permission_type = ?";
	    binds = new Object[] {onWhatId, onWhichTable, permissionType};
	} else {
	    binds = new Object[] {onWhatId, onWhichTable};
	}
	String result = db.queryToString(query, binds);
	return Integer.parseInt(result);
    }

    /**
     * Grants a permission for a database object to the specified 
     * user or group/role, depending on what "scope" is.  Calls the 
     * appropriate pl/sql functions in the general_permissions
     * package.
     * 
     * @param scope The scope of the permission (user, group, 
     * registered_users, all_users, group_role)
     */
    public static void grantPermission(Database db,
				       String scope,
				       Integer userId,
				       Integer groupId,
				       String role,
				       String permissionType,
				       Integer onWhatId,
				       String onWhichTable) 
	throws ACSException, SQLException {

	if (scope.equals("user") && userId==null) {
	    throw new IllegalArgumentException("userid cannot be null if scope is user.");
	} 
	if (scope.startsWith("group") && groupId==null) {
	    throw new IllegalArgumentException("userid cannot be null if scope is group/group_role.");
	} 
	String namePiece = "";
	Object binds[];
	if (scope.equals("user") || scope.equals("group")) { 
	    namePiece = scope;
	    Integer id = (scope.equals("user"))? userId: groupId;
	    binds = new Object[] {id, permissionType, onWhatId, onWhichTable};
	} else if (scope.equals("registered_users")) {
	    namePiece = "reg_users";
	    binds = new Object[] {permissionType, onWhatId, onWhichTable};
	} else if (scope.equals("all_users")) {
	    namePiece = "all_users";
	    binds = new Object[] {permissionType, onWhatId, onWhichTable};
	} else if (scope.equals("group_role")) {
	    namePiece = "role";
	    binds = new Object[] {groupId, role, permissionType, onWhatId, onWhichTable};
	} else {
	    throw new IllegalArgumentException("cannot call grantPermission with scope " + scope);
	}

	final String procCall = "begin :1 := ad_general_permissions.grant_permission_to_" + namePiece + "(?, ?, ?" + ((binds.length>=4) ? ", ?" : "" )
	    + ((binds.length==5) ? ", ?" : "" )
	    + "); end;";
	db.plsql(procCall, binds);
    }

    /**
     * Returns the id in the general_permissions table for a 
     * database object, given the specified scope, user, group, etc.
     * Uses the pl/sql functions in the general_permissions
     * package.
     * 
     * @param scope The scope of the permission (user, group, 
     * registered_users, all_users, group_role)
     */
    public static Integer getPermissionId(Database db,
					  String scope,
					  Integer userId,
					  Integer groupId,
					  String role,
					  String permissionType,
					  Integer onWhatId,
					  String onWhichTable) 
	throws ACSException, SQLException {

	if (scope.equals("user") && userId==null) {
	    throw new IllegalArgumentException("userid cannot be null if scope is user.");
	} 
	if (scope.startsWith("group") && groupId==null) {
	    throw new IllegalArgumentException("userid cannot be null if scope is group/group_role.");
	} 
	String namePiece = "";
	Object binds[];
	if (scope.equals("user") || scope.equals("group")) { 
	    namePiece = scope;
	    Integer id = (scope.equals("user"))? userId: groupId;
	    binds = new Object[] {id, permissionType, onWhatId, onWhichTable};
	} else if (scope.equals("registered_users")) {
	    namePiece = "reg_users";
	    binds = new Object[] {permissionType, onWhatId, onWhichTable};
	} else if (scope.equals("all_users")) {
	    namePiece = "all_users";
	    binds = new Object[] {permissionType, onWhatId, onWhichTable};
	} else if (scope.equals("group_role")) {
	    namePiece = "group_role";
	    binds = new Object[] {groupId, role, permissionType, onWhatId, onWhichTable};
	} else {
	    throw new IllegalArgumentException("cannot call grantPermission with scope " + scope);
	}

	final String procCall = "select ad_general_permissions." + namePiece + "_permission_id (?, ?, ?" 
	    + ((binds.length>=4) ? ", ?" : "" )
	    + ((binds.length==5) ? ", ?" : "" )
	    + ") from dual";
	String result = db.queryToString(procCall, binds);
	return new Integer(result);
    }

    /**
     * Revokes the given permission row (the desired id can be obtained by calling getPermissionId)
     */
    public void revokePermission(Database db, Integer permissionId) 
    throws SQLException {
	db.dml("begin ad_general_permissions.revoke_permission(" + permissionId + "); end;");
    }
}
				     
					    
