package com.arsdigita.acs;

import java.util.*;

/**
 * Class for HTML-generating widgets.
 */

public final class HtmlWidgets {

    /**
     * Returns a list of HTML option tags.  <code>values</code>
     * is the list of option values, <code>text</code> is the
     * text to display after each option tag, and <code>selected</code>
     * is the default (selected) option.
     */
    public final static String htmlOptions(String[] values,
					   String[] text, 
					   String selected) {
	if (text.length != values.length) 
	    throw new IllegalArgumentException("values/text must be same length");
	
	StringBuffer result = new StringBuffer();
	for (int i = 0; i<values.length; i++) {
	    result.append("<option value=\"");
	    result.append(values[i]);
	    result.append("\"");
	    if (values[i].equals(selected)) {
		result.append(" SELECTED");
	    }
	    result.append(">");
	    result.append(text[i]);
	    result.append("\n");
	}
	return result.toString();
    }

    public final static String htmlOptions(Vector vals, Vector text, String selected) { 
	if (text.size() != vals.size()) 
	    throw new IllegalArgumentException("values/text must be same length");
	String[] v = new String[vals.size()];
	vals.copyInto(v);
	String[] t = new String[text.size()];
	text.copyInto(t);
	
	return htmlOptions(v, t, selected);
    }

    /**
     * Returns a list of HTML option tags from  an array of arrays of String pairs. The first of the pair is the
     * option value, the second is the text to be displayed.
     *
     * @param objects an array of arrays of name value pairs. ie, something of the form: <pre>
     *                new Object[][] = { {"true", "HTML"},
     *                                   {"false", "Plain text"} }
     * @param selected the value of the default (selected) option
     */
    public final static String htmlOptions(String[][] objects, String selected)
	throws ACSException {
	int pairCount = objects.length;
	String[] values = new String[pairCount];
	String[] text = new String[pairCount];
	for (int i = 0; i < pairCount; i += 1) {
	    if (objects[i].length != 2)
		throw new ACSException("Illegal argument format; should be an array of arrays of name-value pairs.");
	    values[i] = objects[i][0];
	    text[i] = objects[i][1];
	}

	return htmlOptions(values,text,selected);
    }

    /**
     * Assumes that in the given DataSource, the first column should
     * be used as the option text, and the second column should be
     * used as the option value.
     */
    public final static String htmlOptions(DataSource ds, String selected) 
    throws DataSourceException {
	String[] keys = ds.keys();
	Vector texts = new Vector();
	Vector values = new Vector();
	while (ds.next()) {
	    texts.addElement(ds.get(keys[0]));
	    values.addElement(ds.get(keys[1]));
	}
	return htmlOptions(values, texts, selected);
    }

    /** 
     * Creates a HTML form fragment of a type appropriate for the type of
     * data expected (eg radio buttons if the type is boolean).  The
     * column_type can be any of the following: integer, number, date, text
     * (up to 4000 characters), text_short (up to 200 characters), boolean,
     * and special (no form element will be provided).
     */
    public final static String fieldFormElement(String field_name,
						String column_type,
						String default_value) 
    {
	if (default_value == null) 
	    default_value = "";

	if (column_type.equals("integer") || column_type.equals("number")) {
	    return "<input type=text name=\"" + field_name + "\" value=\"" + Utilities.quoteHtml(default_value) + "\" size=5>";
	} else if (column_type.equals("date")) {
	    return HtmlWidgets.dateWidget(field_name);
	} else if (column_type.equals("text_short")) {
	    return "<input type=text name=\"" + field_name + "\" value=\"" + Utilities.quoteHtml(default_value) + "\" size=30 maxlength=200>";
	} else if (column_type.equals("text")) {
	    return "<textarea wrap name=\"" + field_name + "\" rows=8 cols=50>" + default_value + "</textarea>";
	} else if (column_type.equals("special")) {
	    return "Special field.";
	} else {
	    // it's boolean
	    boolean yesChecked = false;
	    boolean noChecked = false;
	    if (default_value.equalsIgnoreCase("t") || default_value.equalsIgnoreCase("y") || default_value.equalsIgnoreCase("yes")) {
		yesChecked = true;
	    }
	    if (default_value.equalsIgnoreCase("f") || default_value.equalsIgnoreCase("f") || default_value.equalsIgnoreCase("no")) {
		noChecked = true;
	    }
	    String toReturn = "<input type=radio name=\"" + field_name + 
		"\" value=t " + (yesChecked? "checked" : "") + ">Yes &nbsp;";
	    toReturn += "<input type=radio name=\"" + field_name + 
		"\" value=f " + (noChecked? "checked" : "") + ">No &nbsp;";
	    return toReturn;
	}
    }

    public final static String columnTypeWidget(String name) {
	return "<select name=\"" + name + "\">\n" + 
	    "<option value=boolean >Boolean (Yes or No)\n" + 
	    "<option value=integer >Integer (Whole Number)\n" + 
	    "<option value=number >Number (e.g., 8.35)\n" + 
	    "<option value=date >Date\n" + 
	    "<option value=text_short >Short Text (up to 200 characters)\n" + 
	    "<option value=text >Long Text (up to 4000 characters)\n" +
	    "<option value=special >Special (no form element will be provided)\n" + 
	    "</select>\n";
    }

    public final static String dateWidget(String fieldName) { 
	return (new org.apache.turbine.util.DateSelector(fieldName)).toString();
    }

    // silly method ported from Tcl proc of same name
    public final static String[] dayList() {
	return new String[] { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11",
			      "12", "13", "14", "15", "16", "17", "18", "19", "20", "21",
			      "22", "23", "24", "25", "26", "27", "28", "29", "30", "31"};
    }
}

