package com.arsdigita.acs;

import java.io.*;

/**
 * Utility class for getting the width and height of images for uploaded
 * files.  This code was derived from AOLServer 3.0, tclmisc.c.  The
 * terms of the GNU Public License apply.
 */

public class ImageSize {

    private int width;
    private int height;

    private RandomAccessFile raf;

/*
 * JPEG markers consist of one or more 0xFF bytes, followed by a marker
 * code byte (which is not an FF).  Here are the marker codes of interest
 * in this program.  (See jdmarker.c for a more complete list.)
 */

    static final int M_SOF0 = 0xC0;		/* Start Of Frame N */
    
    static final int M_SOF1  = 0xC1;	/* N indicates which compression process = */
    static final int M_SOF2 = 0xC2;		/* Only SOF0 and SOF1 are now in common use = */
    static final int M_SOF3  = 0xC3;
    static final int M_SOF5  = 0xC5;
    static final int M_SOF6  = 0xC6;
    static final int M_SOF7  = 0xC7;
    static final int M_SOF9  = 0xC9;
    static final int M_SOF10 = 0xCA;
    static final int M_SOF11 = 0xCB;
    static final int M_SOF13 = 0xCD;
    static final int M_SOF14 = 0xCE;
    static final int M_SOF15 = 0xCF;
    static final int M_SOI  = 0xD8;		/* Start Of Image (beginning of datastream) = */
    static final int M_EOI  = 0xD9;		/* End Of Image (end of datastream) = */
    static final int M_SOS  = 0xDA;		/* Start Of Scan (begins compressed data) = */
    static final int M_COM  = 0xFE;		/* COMment = */

    private int JpegRead2Bytes() throws IOException
    {
	int c1, c2;
	
	c1 = raf.read();
	c2 = raf.read();
	if (c1 == -1 || c2 == -1) {
	    return -1;
	}
	int retval = (c1 << 8) + c2;
	// System.out.println("c1: " + c1 + " c2: " + c2 + " 2bytes: " + retval);
	return retval;
    }

    /**
     * Initializes an ImageSize object with a JPEG image.
     * 
     */
    private void initJpeg(File f)  throws IOException
    {
	int i;
	raf = new RandomAccessFile(f, "r");
	int c = 0;
	if (raf.read() == 0xFF && raf.read() == M_SOI) {
	    while (true) {
		// look for next JPEG marker
		do { 
		    i = raf.read();
		} while (i != 0xFF);
		do { 
		    i = raf.read();
		} while (i == 0xFF);
		if (i == -1 || i == M_SOS || i == M_EOI) {
		    break;
		}
		// System.out.println("Got marker: " + i);
		if (i >= 0xC0 && i <= 0xC3) {
		    // System.out.println("About to return:");
		    if (JpegRead2Bytes() != -1 && raf.read() != -1
			&& (this.height = JpegRead2Bytes()) != -1
			&& (this.width = JpegRead2Bytes()) != -1) {
			// w and h are set
			return;
		    }
		    break;
		}
		i = JpegRead2Bytes();
		if (i < 2 && i > 0) {
		    break;
		}
		long nextPos = raf.getFilePointer() + i - 2;
		// System.out.println("about to seek to nextPos: " + nextPos + " ; current pos = " + raf.getFilePointer());
		if (nextPos > raf.length()) {
		    break;
		}
		raf.seek(nextPos); 
	    }
	}
	raf.close();
	return;
    }

    public void initGif(File f) throws IOException { 
	/*
	 * Read the GIF version number
	 */
	
	raf = new RandomAccessFile(f, "r");
	byte buf[] = new byte[0x300];
	if (raf.read(buf, 0, 6) == -1) {
	    raf.close();
	    throw new IOException("Bad GIF file format: not enough room for header");
	}
	
	for (int i = 0; i < 6; i++) { 
	    if (buf[i] != "GIF87a".charAt(i) && buf[i] != "GIF89a".charAt(i)) {
		throw new IOException("Bad GIF file format: not GIF87a or GIF89a");
	    }
	}
	
	if (raf.read(buf, 0, 7) == -1) {
	    throw new IOException("Bad file format");
	}
	
	int depth = 1 << ((buf[4] & 0x7) + 1);
	int colormap = ((buf[4] & 0x80) != 0 ? 1 : 0);

	if (colormap != 0) {
	    if (raf.read(buf, 0, 3*depth) == -1) {
		throw new IOException("Bad file format");
	    }
	}
    outerloop:
	do {
	    if (raf.read(buf, 0, 1) == -1) {
		throw new IOException("Bad file format");
	    }
	    
	    if (buf[0] == '!') {
		int count;
		
		if (raf.read(buf, 0, 1) == -1) {
		    throw new IOException("Bad file format");
		}
		do {
		    if ((count = raf.read()) == -1) {
			throw new IOException("Bad file format");
		    }
		    if (count == 0) { 
			continue outerloop;
		    }
		    if (raf.read(buf, 0, count) == -1) {
			throw new IOException("Bad file format");
		    }
		} while (true);
	    } else if (buf[0] != ',') {
		throw new IOException("Bad file format");
	    }
	    
	    if (raf.read(buf, 0, 9) == -1) {
		throw new IOException("Bad file format");
	    }

	    raf.close();
	    
	    this.width = 0x100 * buf[5] + buf[4];
	    this.height = 0x100 * buf[7] + buf[6];
	    return;
	} while (true);
    }
    
    public ImageSize(String fileName) throws IOException {
	if (fileName.endsWith("jpeg") || fileName.endsWith("jpg")) { 
	    initJpeg(new File(fileName));
	} else if (fileName.endsWith("gif")) { 
	    initGif(new File(fileName));
	} else {
	    throw new IllegalArgumentException ("Cannot handle file: " + fileName);
	}
    }
    
    public int getWidth() { 
	return this.width;
    }

    public int getHeight() { 
	return this.height;
    }

    public static void main(String[] args) throws Exception { 
	ImageSize is = new ImageSize(args[0]);
	System.out.println(is.getWidth());
	System.out.println(is.getHeight());
    }
			   
	
}



