// $Id: LastVisit.java,v 1.4 2000/10/27 21:41:43 deison Exp $

package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;

import java.io.*;
import java.util.*;

import com.arsdigita.acs.db.*;
import org.apache.turbine.util.Log;
import java.sql.SQLException;
import com.oroinc.text.perl.Perl5Util;


/**
 * This utility class handles the maintenance of last_visit and 
 * second_to_last_visit cookies.
 * Each of these is the number of seconds since January 1, 1970.
 *
 * TODO: Move mostly into ClientIdentification.
 */
public class LastVisit extends Filter {

    private static HashMap ipAddressTimesCache;
    /*private final static int DEFAULT_LAST_VISIT_UPDATE_INTERVAL = 600;
    private final static int DEFAULT_LAST_VISIT_CACHE_UPDATE_INTERVAL = 600;
    private final static int DEFAULT_LAST_VISIT_EXPIRATION = 86400;*/

    // TODO: Temp for debugging
    private final static int DEFAULT_LAST_VISIT_UPDATE_INTERVAL = 10;
    private final static int DEFAULT_LAST_VISIT_CACHE_UPDATE_INTERVAL = 10;
    private final static int DEFAULT_LAST_VISIT_EXPIRATION = 20;

    static {
        ipAddressTimesCache = new HashMap();
        // we only establish these filters if the system is enabled in ad.ini
        if (Parameters.getBoolean("LastVisitCookiesEnabledP", "", true)) {
            LastVisit filter = new LastVisit();
            // will maintain the last_visit and second_to_last_visits users columns
            // we could have done just "/*" but that puts cookies on all the images
            // and so forth; it is unfriendly to people who've linked to images
            // in-line from other pages and it is unfriendly to those who have
            // "warn about cookies" enabled
            // 
            // TODO: Determine if these concerns are obsolete due to our session IDs.  -DEE 10/22/2000
            // We would like to register on just HTML/HTM/JSP, but unfortunately our filtering
            // doesn't support that yet.
            //RequestProcessor.registerFilter("/*\\.html", filter);
            //RequestProcessor.registerFilter("/*\\.htm", filter);
            //RequestProcessor.registerFilter("/*\\.jsp", filter);
            RequestProcessor.registerFilter("/", filter);
        }
    }

    /**
     * Updates the last visit and second_to_last_visit values in the database.
     */
    private static void updateLastVisits (Integer userId) throws SQLException {
        Database db = new Database();
        db.dml("update users " +
               "set last_visit = sysdate, " +
               "second_to_last_visit = last_visit, " +
               "n_sessions = n_sessions + 1 " +
               "where user_id = :user_id", new Object[] {userId});
    }

    /**
     * Updates the last_visit value in the database.
     */
    private static void updateLastVisit (Integer userId) throws SQLException {
        Database db = new Database();
        db.dml("update users " +
               "set last_visit = sysdate " +
               "where user_id = :user_id", new Object[] {userId});
    }

    /**
     * Updates session statistics in database.
     * Same as updateSessionStatistics(repeat, true).
     */
    private static void updateSessionStatistics (boolean repeat) throws SQLException {
        updateSessionStatistics(repeat, true);
    }

    /**
     * Updates session statistics in database.
     */
    private static void updateSessionStatistics (boolean repeat, boolean session) throws SQLException {
        String updateSql;
        String insertSql;
        if (repeat) {
            if (session) {
                updateSql = "update session_statistics " +
                             "set session_count = session_count + 1, " +
                             "repeat_count = repeat_count + 1 " +
                             "where entry_date = trunc(sysdate)";
                insertSql = "insert into session_statistics (session_count, repeat_count, entry_date) " +
                    " values " +
                    "(1,1,trunc(sysdate))";
            } else {
                // The user came to the site with no cookies.
                // We recorded a session, but no repeat session
                // at this point.

                // The user subsequently logged in.  We now
                // know that this is a repeat visit.
                updateSql = "update session_statistics " +
                    "set repeat_count = repeat_count + 1 " +
                    "where entry_date = trunc(sysdate)";
                insertSql = "insert into session_statistics (session_count, repeat_count, entry_date) " +
                    "values " +
                    "(0,1,trunc(sysdate))";
            }
        } else {
            // not a repeat user
            updateSql = "update session_statistics " +
                "set session_count = session_count + 1 " +
                "where entry_date = trunc(sysdate)";
            insertSql = "insert into session_statistics (session_count, repeat_count, entry_date) " +
                "values " +
                "(1,0,trunc(sysdate))";
        }
        Database db = new Database();
        if (db.dml(updateSql) == 0) {
            db.dml(insertSql);
        }
    }

    /**
     * returns the seconds since January 1, 1970 of the second_to_last_visit
     * (from the cookie).  If there is no cookie (or cookie is not a number), returns null.
     * 
     * @return Seconds since Jan 1. 1970 to second_to_last_visit or null.
     */
    private static Integer getSecondToLastVisitUT(HttpServletRequest req) {
         String second_to_last_visit = Utilities.getCookieValue(req, "second_to_last_visit");
         Integer secs = null;
         if (!Utilities.emptyString(second_to_last_visit)) {
             try {
                 secs = Integer.valueOf(second_to_last_visit);
             } catch (NumberFormatException e) {
                 Log.warn("Invalid number '" + second_to_last_visit + "' found for second_to_last_visit cookie, ignoring.");
             }
         }
         return secs;
    }

    /**
     * Just what it sounds like.  Based on Calendar.java
     * Forexample, this will be -5 for EST outside of daylight savings time, and -4 for EST during daylight savings time.
     */
    private static int getCurrentHoursDifferenceFromGMT() {
        java.util.Calendar c = new java.util.GregorianCalendar();
        int offset = (c.get(Calendar.ZONE_OFFSET) / (60*60*1000) + c.get(Calendar.DST_OFFSET) / (60*60*1000));
        return offset;
    }

    /**
     * The actual work happens here.
     *
     * Think of last_visit as representing the actual time of the last
     * visit (with some delay controlled by LastVisitExpiration, for
     * efficiency) and second_to_last_visit as representing the last visit
     * time for the previous "session".
     */
    private static void maintainLastVisitsInternal(HttpServletRequest req, HttpServletResponse resp, boolean readSecondToLastVisit) throws SQLException, ACSException {
        Integer userId = Security.getUnverifiedUserID(req);
        String lastVisitCookie = Utilities.getCookieValue(req, "last_visit");
        String secondToLastVisitCookie = null;
        long now = ClientIdentification.getTime();
        if (readSecondToLastVisit) {
            secondToLastVisitCookie = Utilities.getCookieValue(req, "second_to_last_visit");
        }
        if (!Utilities.emptyString(lastVisitCookie)) {
            // we have a last visit cookie; maybe it is old and needs refreshing.
            int expiration_seconds = Parameters.getInt("LastVisitExpiration", "", DEFAULT_LAST_VISIT_EXPIRATION);
            long lastVisit;
            try {
                lastVisit = Long.parseLong(lastVisitCookie);
            } catch (NumberFormatException e) {
                lastVisit = 0; // This will force expiration
            }
            if (now - lastVisit > expiration_seconds) {
                // consider this a new visit & update the cookie.
                String nowString = String.valueOf(now);
                assignLastVisitCookie(resp, nowString);
                assignSecondToLastVisitCookie(resp, lastVisitCookie);
                updateSessionStatistics(true);
                // If this is a registered user, update the users table.
                if (userId != null && userId.intValue() != 0) {
                    updateLastVisits(userId);
                }
            } else if (readSecondToLastVisit && (now - lastVisit > Parameters.getInt("LastVisitUpdateInterval", "", DEFAULT_LAST_VISIT_UPDATE_INTERVAL))) {
                // This counts as the same session; just update the last_visit cookie and database field.
                assignLastVisitCookie(resp, String.valueOf(now));
                // If this is a registered user, update the users table.
                if (userId != null && userId.intValue() != 0) {
                    updateLastVisits(userId);
                }
            }
            // last visit was too recent to do anything about, don't do anything.
        } else {
            // no last visit cookie
            String ipAddress = req.getRemoteAddr();
            Long time = (Long)ipAddressTimesCache.get(ipAddress);
            long currentTime = ClientIdentification.getTime();
            if (time == null || (currentTime - time.longValue() > Parameters.getInt("LastVisitCacheUpdateInterval", "", DEFAULT_LAST_VISIT_CACHE_UPDATE_INTERVAL))) {
                // We haven't seen this IP, let's consider this a new session
                assignLastVisitCookie(resp);
                ipAddressTimesCache.put(ipAddress, time);
            } else {
                // We've already seen this IP.
                // Since there is no last visit cookie, we
                // assume the attempt to cookie was unsuccessful 
                // on a previous hit and thus do not set the cookie again.
                return;
            }

            // Let's record this as a new visit.
            if (userId == null || userId.intValue() == 0) {
                updateSessionStatistics(false);
            } else {
                // This is the very rare case where the user has
                // a user_id cookie, but not a last visit cookie.
                // If the person is a user, update the
                // last_visit dates in the database.
                Database db = new Database();
                updateSessionStatistics(true);
                updateLastVisits(userId);
                Integer timeDiff = new Integer(getCurrentHoursDifferenceFromGMT());
                String secondToLastVisitUT = (db.queryToString("select 86400*(second_to_last_visit - to_date('1970-01-01') - (:time_diff/24)) " +
                                                        "from users " +
                                                        "where user_id = :user_id", new Object[] {timeDiff, userId}, null));
                //Log.note("LastVisit: found secondToLastVisit value " + secondToLastVisitUT);
                if (secondToLastVisitUT != null) {
                    assignSecondToLastVisitCookie(resp, secondToLastVisitUT);
                }
                db.close();
            }
        }
    }

    private static void assignLastVisitCookie(HttpServletResponse resp) {
        assignLastVisitCookie(resp, String.valueOf(ClientIdentification.getTime()));
    }

    private static void assignLastVisitCookie(HttpServletResponse resp, String value) {
        Cookie lastVisitCookie = new Cookie("last_visit", value);
        // This # came from Security.java's loginUser method.  I presume it was randomly selected as "really big number".
	lastVisitCookie.setMaxAge(100000000);
        lastVisitCookie.setPath("/");
        resp.addCookie(lastVisitCookie);
    }

    private static void assignSecondToLastVisitCookie(HttpServletResponse resp, String value) {
        //Log.note("LastVisit: setting secondToLastVisit cookie to " + value);
        Cookie secondToLastVisitCookie = new Cookie("second_to_last_visit", value);
	secondToLastVisitCookie.setMaxAge(100000000);
        secondToLastVisitCookie.setPath("/");
        resp.addCookie(secondToLastVisitCookie);        
    }

    public int filter(HttpServletRequest req, 
		      HttpServletResponse resp,
		      ServletContext ctx) throws ACSException {
        try {
            String userAgent = req.getHeader("User-Agent");
            if (userAgent != null && userAgent.toLowerCase().indexOf("aolserver") > 0) {
                return FILTER_OK;
            }
            // If WhosOnlineP is set, we use secondToLastVisit to get more accurate lastVisit times.
            maintainLastVisitsInternal(req, resp, Parameters.getBoolean("WhosOnlineP", "", false));
        } catch (Exception e) {
            throw new ACSException(e.getMessage());
        }
        return FILTER_OK;
    }
}


