package com.arsdigita.acs;

import java.util.Hashtable;

/**
 * Cache for memoized values.  A value can be memoized with a time limit,
 * after which it expires.  
 *
 * <p>
 * 
 * All the public methods and private members of MemoizeCache are static,
 * since there should be only one instance of the cache for each web 
 * service instance (i.e., one per webapp context).
 */

public class MemoizeCache { 
    /** 
     * map keys to values
     */
    private static Hashtable values;

    /**
     * map keys to expiration times
     */
    private static Hashtable expirations;

    static { 
	values = new Hashtable();
	expirations = new Hashtable();
    } 

    /** 
     * Returns the cached value for <code>key</code>; returns null if 
     * key is unmapped or if the mapping has expired.
     */
    public static Object lookup(String key) { 
	// check to see if expired first
	Long expireTime = (Long)expirations.get(key);
	if (expireTime == null || expireTime.longValue() > System.currentTimeMillis()) {
	    // no expiration time, value still valid, or unmapped, 
	    // so return whatever value we have mapped if any
	    return values.get(key);
	} else {
	    // expired
	    return null;
	}
    }

    /**
     * Flush a value from the cache.
     */
    public static void flush(String key) { 
	values.remove(key);
	expirations.remove(key);
    }

    /** 
     * Cache a value with no expiration time--i.e., forever
     */
    public static void cache(String key, Object value) { 
	values.put(key, value);
	// System.out.println("Cache called!");
    }

    /** 
     * Cache a value with a duration, in seconds.
     */
    public static void cache(String key, Object value, int dur) { 
	values.put(key, value);
	expirations.put(key, new Long(System.currentTimeMillis() + dur*1000));
    }

    public static void main(String[] args) { 
	cache("forever", "live4ever");
	cache("awhile", "5sec", 3);
	cache("toflush", "nowUSeeIt");

	System.out.println(lookup("forever"));
	System.out.println(lookup("awhile"));
	System.out.println(lookup("toflush"));
	System.out.println(lookup("unmapped"));

	try{Thread.sleep(4000);} catch(Exception e) {}

	System.out.println(lookup("forever"));
	System.out.println(lookup("awhile"));
	System.out.println(lookup("toflush"));
	
	flush("toflush");
	
	System.out.println(lookup("forever"));
	System.out.println(lookup("awhile"));
	System.out.println(lookup("toflush"));
    }
}



