package com.arsdigita.acs;

import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Class to handle ACS parameters, loaded on server startup.
 * Parameters are stored and accessed via static methods
 * <p>
 * EXPECTS: IniFileName servlet init parameter: points to an .xml file 
 * with ACS parameters
 * <p>
 * We chose to make the ACS parameters a separate XML file, rather than
 * putting them in web.xml, because it allows us to make a more convenient
 * syntax for having parameters with multiple (i.e., lists of) values.
 * It also maintains some separation between things that are web-server
 * specific, and the things that are specific to ACS and server-independent.
 * 
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @version $Version$
 */
public class Parameters extends HttpServlet {
    // this is where we store properties
    // static so we can use static methods to access them transparently
    // through JSP pages

    // there should only be one instance of init parameters for each 
    // service anyway
    private static Hashtable props;

    // list of classes that need to be loaded by the classloader
    // to invoke their static initializers
    private static Vector classesToLoad;

    // accessors

    /** 
     * Gets a parameter as an Object.
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @param def Default if not found
     * @return The value of the parameter, or def if not found
     */
    public static Object getObject(String param, 
				   String module, 
				   Object def) {
	if (module != null && module.length() > 0) 
	    param = module + "." + param;
	Object retval = props.get(param.toLowerCase());
	if (retval == null) 
	    return def;
	else return retval;
    }

    /** 
     * Gets a parameter as an Object.
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @return The value of the parameter, or null
     */
    public static Object getObject(String param, 
				   String module) {
	return getObject(param, module, null);
    }

    /** 
     * Gets a parameter as a String.
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @param def Default value
     * @return The value of the parameter, or def if not found
     */
    public static String get(String param, String module, String def) {
	Object val = getObject(param, module, def);
	if (val != null) 
	    return val.toString();

	// for strings we want to default to the empty string
	// to avoid printing the word "null" in JSP output, and
	// to free programmers from having to check for nulls before
	// every string operation on a parameter (or run the risk of
	// throwing NullPointerException)
	else return "";
    } 

    /** 
     * Gets a parameter as a String.
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @return The value of the parameter, or empty string (not null) 
     * if not found.
     */
    public static String get(String param, String module) {
	return get(param, module, "");
    } 

    /**
     * Gets a list of  parameter values as a Vector.
     * @param param The parameter name
     * @module The module name (may be empty or null)
     * @return The list of parameter values, or null if not found.
     */
    public static Vector getVector(String param, String module) {
	return (Vector)getObject(param, module, null);
    } 

    /** 
     * Gets a parameter as a boolean primitive
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @param def Default value
     * @return The value of the parameter, or def if not found.
     */
    public static boolean getBoolean(String param, 
				     String module, 
				     boolean def) {
	Object val = getObject(param, module, new Boolean(def));
	if (val != null) 
	    return ((Boolean)val).booleanValue();
	else return def;
    }

    /** 
     * Gets a parameter as a boolean primitive
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @return The value of the parameter, or false if not found.
     */
    public static boolean getBoolean(String param, String module) {
	return getBoolean(param, module, false);
    }

    /** 
     * Gets a parameter as an int primitive
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @param def Default value
     * @return The value of the parameter, or def if not found.
     */
    public static int getInt(String param, 
			     String module, 
			     int def) {
	Object val = getObject(param, module, new Integer(def));
	if (val != null) 
	    return ((Integer)val).intValue();
	else return def;
    }

    /** 
     * Gets a parameter as a boolean primitive
     * @param param The parameter
     * @param module The module name (may be empty or null) 
     * @return The value of the parameter, or 0 if not found.
     */
    public static int getInt(String param, String module) {
	return getInt(param, module, 0);
    }

    /** 
     * Calls readParameters at start-up.
     */
    public void init () throws ServletException {
	props = new Hashtable();
	classesToLoad = new Vector();

	readParameters();
    }

    /**
     * Returns an enumeration of classes that need to be loaded at 
     * server start-up to invoke static initializers.
     */
    public final static Enumeration getInitClasses() { 
	return classesToLoad.elements();
    }

    /**
     * Read init parameter IniFileName, parses the file named by that
     * parameter with an XML parser (we use the SAX parser), and sets 
     * up the static <props> object.
     */
    private void readParameters() throws ServletException {
	try {
	    com.sun.xml.parser.ValidatingParser p = 
		new com.sun.xml.parser.ValidatingParser(true);
	    String iniFile = getInitParameter("IniFileName");
	    DocumentHandler dh = new InitFileHandler(this);
	    p.setDocumentHandler(dh);
	    p.parse(iniFile);
	} 
	catch (Exception e) {
	    System.out.println("Error initializing parameters: " + e.getMessage());
	    throw new ServletException(e);
	}
    }	

    /**
     * Utility class used to parse the XML parameters file.
     */
    private class InitFileHandler extends HandlerBase {
	Hashtable props;
	Vector classesToLoad;
	
	StringBuffer sb;
	String currentModule;
	String currentName;
	String currentValue;
	String currentType = "string";
	Vector valueList;
	
	
	public InitFileHandler(Parameters p) {
	    this.props = p.props;
	    this.classesToLoad = p.classesToLoad;
	    
	    valueList = new Vector();
	    sb = new StringBuffer();
	}

	public void startElement(String tag, AttributeList al) {
	    if (tag.equalsIgnoreCase("module")) {
		currentModule = al.getValue("name");
	    }
	    else if (tag.equalsIgnoreCase("parameter")) {
		currentName = al.getValue("name");
		currentType = al.getValue("type");
		if (currentType == null) 
		    currentType = "string";
	    }
	}

	public void characters(char [] chars, int start, int len) {
	    for (int i = start; i < start + len; i++)
		sb.append(chars[i]);
	}
    
	public void endElement(String tag) {
	    if (tag.equalsIgnoreCase("value")) {
		currentValue = sb.toString().trim();
		valueList.addElement(currentValue);
	    }

	    else if (tag.equalsIgnoreCase("type")) 
		currentType = sb.toString();

	    else if (tag.equalsIgnoreCase("module")) 
		currentModule = null;

	    else if (tag.equalsIgnoreCase("loadclass")) 
		classesToLoad.addElement(sb.toString().trim());

	    else if (tag.equalsIgnoreCase("parameter")) {
		if (currentModule != null && currentModule.length() > 0) {
		    currentName = currentModule + "." + currentName;
		}
		currentName = currentName.toLowerCase();
		// stash it in the props list
		if (valueList.size()>1) {
		    if (currentType.equalsIgnoreCase("int")) {
			for (int i = 0; i < valueList.size(); i++)
			    valueList.setElementAt(new Integer((String)valueList.elementAt(i)), i);
		    } else if (currentType.equalsIgnoreCase("boolean")) {
			for (int i = 0; i < valueList.size(); i++)
			    valueList.setElementAt(new Boolean((String)valueList.elementAt(i)), i);
		    }
		    props.put(currentName, valueList);

		}
		else if (currentType.equalsIgnoreCase("string"))
		    props.put(currentName, currentValue);
		else if (currentType.equalsIgnoreCase("boolean"))
		    props.put(currentName, new Boolean(currentValue));
		else if (currentType.equalsIgnoreCase("int"))
		    props.put(currentName, new Integer(currentValue));
		valueList = new Vector();
	    }
	    sb = new StringBuffer();
	}
    } 

}


