package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import com.oroinc.text.perl.Perl5Util;
import org.apache.turbine.util.Log;
import com.arsdigita.acs.db.*;
import java.sql.*;



/**
 * Checks if a user was referred here by another site, and logs it to the database.
 *
 * @creation-date 4 Jul 1998
 * @author Philip Greenspun (philg@mit.edu)
 * @author Tracy Adams (teadams@mit.edu)
 * @author David Eison (eison@arsdigita.com, Java port)
 * @cvs-id $Id: RefererFilter.java,v 1.1 2000/10/27 21:18:14 deison Exp $
 * 
 * we've misspelled "referrer" because it is misspelled in the HTTP
 * standard
 * modified by philg@mit.edu on November 24, 1999 to handle concurrency problem
 * (on first insert of the day)
 *
 * checks to see if there is an external referer header
 * then tries to get db conn
 * then increments counter
 */
public class RefererFilter extends Filter {

    static {
        // This should be a trace filter if we support alternate types of filters at some point.
        //Log.note("RefererFilter: Registering referer filter");
        RequestProcessor.registerFilter("/", new RefererFilter());
    }

    // NOTE that we do not use /string/ for re.match.  We are using the \ character
    // because it's on the list of "unsafe" characters that are not allowed in URLs.


    public static boolean isRefererExternal (HttpServletRequest req) {
        return isRefererExternal(req.getHeader("Referer"));
    }

    /**
     * Check if the user was referred from an external site.
     *
     * @param req The servlet request that we will get the headers from.
     * @return false if the user was referred from the current site, and true if they came
     *         from an external site.
     */
    public static boolean isRefererExternal (String referer) {
        Vector localHostnames = Parameters.getVector("LocalDomain", "referer");
        if (localHostnames != null && referer != null) {
            Perl5Util re = new Perl5Util();
            Enumeration e = localHostnames.elements();
            while (e.hasMoreElements()) {
                String localHost = (String)e.nextElement();
                if (re.match(("\\.*//" + localHost + ".*/.*\\").toLowerCase(), referer.toLowerCase())) {
                    // We found a match in between // and /
                    // case insensitive, including wildcard in case the port number was included, e.t.g,
                    // "photo.net:80" "photo.net:443"
                    return false;
                }
            }
        }
        // No match on localhost
        return true;
    }

    public static boolean isRefererIncludePatternPresent (HttpServletRequest req) {
        return isRefererIncludePatternPresent(req.getHeader("Referer"));
    }

    /**
     *     Check referer header for patterns that must be included.
     * @param req The servlet request that we will get the headers from.
     * @return false if there are no required patterns defined or if none match, and true if there are any matches.
     */
    public static boolean isRefererIncludePatternPresent (String referer) {
        Vector inclusionGlobs = Parameters.getVector("InclusionGLOB", "referer");
        if (inclusionGlobs != null && referer != null) {
            Perl5Util re = new Perl5Util();
            Enumeration e = inclusionGlobs.elements();
            while (e.hasMoreElements()) {
                String inclusionPattern = (String)e.nextElement();
                if (re.match(("\\" + inclusionPattern + "\\").toLowerCase(), referer.toLowerCase())) {
                    return true;
                }
            }
            // nothing matched
            return false;
        } else {
            return true;
        }
    }

    /**
     * Check if the URL points to a GIF or JPEG, since we want to ignore these.
     *
     * @return false if the URL points to a GIF or JPEG, true if it does not.
     */
    public static boolean shouldTrackURL (HttpServletRequest req) {
        if (req.getRequestURI().endsWith(".jpg") || req.getRequestURI().endsWith(".gif")) {
            // We don't want to track referers to this URL
            return false;
        } else {
            return true;
        }
    }

    /**
     * The filter that checks the referer header and logs referrers that we want to keep track of.
     *
     * @return always returns filter_ok
     */
    public int filter(HttpServletRequest req, HttpServletResponse resp, ServletContext ctx) throws ACSException {
        try {
            String referer = req.getHeader("Referer");
            // <= 250 is to cut back on Oracle errors due to too-long strings
            if (!Utilities.emptyString(referer) && isRefererExternal(referer) && isRefererIncludePatternPresent(referer) && shouldTrackURL(req) && referer.length() <= 250) {
                // We have an external header, and the local URL we wish to track.
                // Let's try to hit the DB
                
                // let's try to figure out if we should log it straight
                // or reduced down with a GLOB pattern
                boolean globbed = false;
                boolean regexp = false;
                Database db = new Database();
                ResultSet rs = db.query("select * from referer_log_glob_patterns");
                Perl5Util re = new Perl5Util();
                String canonicalForeignUrl = null;
                String searchEngineName = null;
                String queryString = null;
                Integer completeUserId = null;
                String foreignUrl = null;
                while (rs.next()) {
                    if (Utilities.stringGlobMatch(rs.getString("glob_pattern"), referer)) {
                        // We don't want to log under the raw string
                        globbed = true;
                        canonicalForeignUrl = rs.getString("canonical_foreign_url");
                        searchEngineName = rs.getString("search_engine_name");
                        if (! Utilities.emptyString(searchEngineName) && re.match("\\" + rs.getString("search_engine_regexp") + "\\", referer)) {
                            // We successfuly regexped for the query string
                            queryString = re.group(1);
                            completeUserId=Security.getUnverifiedUserID(req);
                            regexp = true;
                            break;
                        } else {
                            // we globbed but weren't a search engine
                            // or couldn't find the query-string, bust out of the loop
                            break;
                        }
                    }
                }
                rs.close();
                if (globbed) {
                    foreignUrl = canonicalForeignUrl;
                } else {
                    foreignUrl = referer;
                }
                // we can match and not have anything in query string if the user is dumb.
                if (regexp && !Utilities.emptyString(queryString)) {
                    String decodedQueryString = Utilities.URLDecode(queryString);
                    db.dml("insert into query_strings (query_date, query_string, search_engine_name, user_id)" + 
                           "            values " +
                           "          (sysdate, :decoded_query_string, :search_engine_name, :complete_user_id)", new Object[] {decodedQueryString, searchEngineName, completeUserId});
                }
                String localUrl = req.getRequestURI();
                if (db.dml("update referer_log set click_count = click_count + 1" + 
                           "       where local_url = :local_url " + 
                           "        and foreign_url = :foreign_url " +
                           " and trunc(entry_date) = trunc(sysdate)", new Object[] {localUrl, foreignUrl}) == 0) {
                    // there wasn't already a row there; we want to insert a new row
                    // but we have to be careful because we're not inside a database
                    // transaction.  It is possible another thread is simultaneously
                    // executing this logic and inserting an extra row.
                    db.dml("insert into referer_log (local_url, foreign_url, entry_date, click_count) " +
                           "       select :local_url, :foreign_url, trunc(sysdate), 1 " +
                           "         from dual " +
                           "        where 0 = (select count(*)  " +
                           "                     from referer_log " +
                           "                    where local_url = :local_url " +
                           "                     and foreign_url = :foreign_url " +
                           "                     and trunc(entry_date) = trunc(sysdate))", new Object[] {localUrl, foreignUrl, localUrl, foreignUrl});
                }
                db.close();
            }
        } catch (SQLException e) {
            throw new ACSException (e.getMessage());
        }
        // regardless of what happened above, return OK
        return FILTER_OK;
    }
}

// TODO: Finish below?
/*
##################################################################
#
# interface to the ad-user-contributions-summary.tcl system
#
# (to report user searches to the site administrator only)
#

ns_share ad_user_contributions_summary_proc_list

if { ![info exists ad_user_contributions_summary_proc_list] || [util_search_list_of_lists $ad_user_contributions_summary_proc_list "Searches" 0] == -1 } {
    lappend ad_user_contributions_summary_proc_list [list "Searches" searches_user_contributions 0]
}

#this proc used to take a db handle as an arg
#now it is handled through the DB API

ad_proc searches_user_contributions {

    user_id purpose

} {

    Lists all searches performed by any user while a given user was logged in or referred in to the site.

    @param user_id id of the selected user.
    @param purpose must be "site_admin" for procedure to return the list of searches.
    @return empty list if purpose does not equal "site_admin", otherwise a triplet including all the searches typed on this site while this user was logged in or was referred in.

} {

    if { $purpose != "site_admin" } {
	return [list]
    }
    
    set items ""

    db_foreach querys_by_user "select query_date, 
      decode(subsection, null, search_engine_name, subsection) location, 
      decode(n_results, null, '', ' - ' || n_results || ' result(s)') n_results_string, query_string
     from query_strings
       where user_id = :user_id
      order by query_date asc
    " {

	append items "<li>$query_date:</a>
<a href=\"/admin/searches/by-word?query_string=[ns_urlencode $query_string]\"><b>$query_string</b></a>	<a href=\"/admin/searches/by-location?location=[ns_urlencode $location]\">($location)</a>
$n_results_string
"
    }
    
    db_release_unused_handles

    if [empty_string_p $items] {
	return [list]
    } else {
	return [list 0 "Searches" "<ul>\n\n$items\n\n</ul>"]
    }
}
*/
