// $Id: RequestProcessor.java,v 1.14 2000/10/25 19:39:42 jbank Exp $

package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;

import java.io.*;
import java.util.*;

import com.arsdigita.acs.db.*;
import com.arsdigita.acs.ClientIdentification;
import org.apache.turbine.util.Log;
import java.sql.SQLException;
import com.oroinc.text.perl.Perl5Util;

/** 
 * Request procesor servlet for ACS -- handles abstract URLs and 
 * filters.
 * <p> 
 * When this servlet is mapped to "/" in the acs-java context, it will
 * handle all requests in the context that do not have a specific servlet
 * associated with them.  In other words, it can receive and handle requests
 * for paths corresponding to non-existent files.  
 * <p>
 * Its service() method looks at the requested URI, and if the file doesn't 
 * exist, it tries adding different extensions (.jsp, .html, etc.); on 
 * success, it forwards to the appropriate servlet.  On failure (no matching
 * file), it forwards to the default servlet to display the 404 message. 
 * <p>
 * Also handles registration of filters.  
 * <p>
 * NOTE: you MUST override the default mapping of *.jsp URLs in the
 * ACS servlet context for JSP requests to be filtered.  Otherwise,
 * explicitly requesting a *.jsp URL will go directly to the
 * JspServlet and not through the this servlet.  In the context's 
 * web.xml file you should have a servlet mapping from the URL pattern 
 * "*.jsp" to RequestProcessor.
 * <p>
 * Assumes that there are servlets in the same servlet context named
 * "jsp" (to handle JSPs) and "default" to handle everything else
 * (true in Tomcat, Sun JWS, not sure about Oracle 8.1.7, but should
 * be similar if their web server is based on Tomcat)
 * <p>
 * Note: if the request URL is a directory, then we look for dir/index.* 
 * instead of dir.*.
 *
 * @author Bill Schneider (bschneid@arsdigita.com)
 */
public class RequestProcessor extends HttpServlet {

    private static RequestProcessor instance;
    private static Vector filters;
    private static Vector ext_handlers;

    /**
     * URL that ForceHostP will redirect to, based on URL in Parameters file.
     */
    private static String desiredURL = null;

    /**
     * URL that ForceHostP will match against, based on URL in Parameters file.
     */
    private static String desiredHostWithoutPort = null;
    static {
	filters = new Vector();
	ext_handlers = new Vector();
        registerHandler("jsp", new JspHandler());
        if (Parameters.getBoolean("ForceHostP", "", true)) {
            // We need to do some one-time setup for ForceHostP

            // We make a new RE processor since we can't get to the non-static one from here.
            // We don't make the non-static one static because that could cause threading
            // problems.
            Perl5Util re_temp = new Perl5Util();
            // Strip trailing slash
            re_temp.match("/(^.*)\\/$/", Parameters.get("URL", ""));
            desiredURL = re_temp.group(1);
            re_temp.match("/^https?:\\/\\/([^:]+)\\/?/", desiredURL);
            desiredHostWithoutPort = re_temp.group(1);
            Log.note("ForceHostP initialized: will be checking for host " + desiredHostWithoutPort + " and redirecting to " + desiredURL);
        }
    }

    /**
     * Used for ForceHostP.  
     * Can't be a static variable because that would cause threading problems.
     */
    private Perl5Util re = null;

    /**
     * Returns the currently-loaded instance of the request
     * processor servlet.
     * @return The current JVM's RequestProcessor instance.
     */
    public static RequestProcessor getInstance() {
	return instance;
    }

    /** 
     * Called when servlet is loaded, and sets static instance.
     */
    public void init() throws ServletException {
	super.init(); 
	instance = this;
        if (re == null && Parameters.getBoolean("ForceHostP", "", true)) {
            re = new Perl5Util();
        }
    }
    
    /** 
     * Handles an abstract URL in the request req, and executes filters 
     * on the request.  Forwards the request as appropriate.
     *
     * @param req The servlet request.
     * @param resp The servlet response.
     */
    public void service(HttpServletRequest req, HttpServletResponse resp) 
    throws ServletException, IOException {
	// handle an abstract URL, run filters, etc.

	String url = req.getRequestURI();
	// tear the context off the url
	int i = req.getContextPath().length();
	url = url.substring(i);

	// set the ORIGINAL request URI in the request attributes
	// if not already 
	if (req.getAttribute("request_url")==null) { 
	    req.setAttribute("request_url", url);
	}

	// System.out.println("servlet called: URI = " + url);
	
	// this flag in the request indicates that we've already
	// run registered procs on the filter, and that we shouldn't 
	// filter again
	if (req.getAttribute("already_filtered") == null) { 

            // Based on rp_handler_preauth in ACS Classic's request processor,
            // we need:
            // 1) Scoping?
            // 2) Force reload of changed files?  -Skipped, we should improve Tomcat's handling
            //    of this if we need it.
            // 3) Developer Support logging?
            // 4) Skip cookies for "system" directories (like SYSTEM)?
            // 5) Force Host
            // 6) Generate security/session cookies
            // 7) Initialize document processor?  (Not part of ACS-Java)
            //
            // Note that from the above list, we presently only do #5 and #6.

            // Forcing host.  This must be before security so that cookies will be right.
            if (Parameters.getBoolean("ForceHostP", "", true)) {
                // req.getServerName doesn't include the port, so we ask for the Host header directly.  This is
                // more consistent with the TCL and the Cookies file anyway
                String hostHeader = req.getHeader("Host");
                if (! Utilities.emptyString(hostHeader) && re.match("/^([^:]*)/", hostHeader)) {
                    String hostWithoutPort=re.group(1);
                    if (! hostWithoutPort.equals(desiredHostWithoutPort)) {
                        // It may be more correct to call something else besides getRequestURI (i.e. getServletPath),
                        // but this is what HttpUtils getRequestURL uses so I assume it's right (although the
                        // docs would seem to indicate otherwise).  -DEE 10/22/2000
                        StringBuffer target = new StringBuffer(desiredURL).append(req.getRequestURI());
                        Log.note("Host header is set to " + hostHeader + "; forcing to " + target.toString());
                        try {
                            AdSet query = new AdSet(req);
                            String queryString = Utilities.exportURLVars(query);
                            if (!Utilities.emptyString(queryString)) {
                                target.append("?").append(queryString);
                            }
                        } catch (Exception e) {
                            // This should have been thrown by "new AdSet(req)"
                            Log.error("Error getting query parameters, forceHostP will not use query string (Error=" + e);
                        }
                        Utilities.sendRedirect(resp, target.toString());
                        // Abort early, we've handed off to another server.
                        return;
                    }
                }
            }            

            // Handling security.
            ClientIdentification id = null;
            try {
                id = new ClientIdentification(req, resp);
                
                // The above object already got these cookie values, but it did some processing on them so we're going to get them again directly for our debug log
                // Log.note("ad_browser_id=<<" + Utilities.getCookieValue(req, "ad_browser_id") + ">>; ad_session_id=<<" + Utilities.getCookieValue(req, "ad_session_id") + ">>");

                if (! id.isBrowserIdValid()) {
                    id.assignBrowserId();
                }
                if (! id.isSessionIdValid()) {
                    id.assignSessionId();
                } else {
                    // This will update the last hit and recompute the session id cookie if needed.
                    id.maybeUpdateLastHit();
                }
            } catch (SQLException e) {
                Log.error("SQL Error in client identification; proceeding without updating session_id, last_hit, etc.  Error=" + e.toString());
            } catch (ACSException e) {
                Log.error("ACS Error in client identification; proceeding without updating session_id, last_hit, etc.  Error=" + e.toString());
            }
            req.setAttribute("client_identification", id);
            // All of our per-request initialization should be done by now, so that we can run filters.

	    // go through registered filters/procedures
	    // (handle registered procedures as well)
	filter_loop:
	    for (i = 0; i<filters.size(); i++) {
		FilterInfo f = ((FilterInfo)filters.elementAt(i));
		// TO DO: path-filter mapping needs to be a bit more elaborate
		// to deal with subcommunities
		// System.out.println("Checking filters: url = " + url + " filter path = " + f.getPath());
		if (url.startsWith(f.getPath())) {
		    try {
			int result = f.filter(req,resp,getServletContext());
			switch (result) {
			case Filter.FILTER_OK:
			    break;
			case Filter.FILTER_RETURN:
			    // stop processing cold
			    return;
			case Filter.FILTER_BREAK:
			    break filter_loop;
			}
		    } 
		    catch (ACSException e) {
			throw new ServletException(e.getMessage());
		    }
		}
	    }
	}

	req.setAttribute("already_filtered", "yes");

	String realFileName = getServletContext().getRealPath(url);
	File file = new File(realFileName);
	if (file.exists() && file.isDirectory()) {
	    if (!url.endsWith("/")) url = url + "/";
	    url = url + "index";
	}
	
	if (!file.exists()) {
	    String testURL;
            for (i=0; i<ext_handlers.size(); i++) {
                String ext = "." + ((HandlerInfo)ext_handlers.elementAt(i)).getExt();
		testURL = url + ext;
		file = new File(getServletContext().getRealPath(testURL));
		// System.out.println(file);
		if (file.exists()) {
		    // we have the file
		    // we have to forward here so that we get a new request
		    // with the proper filename
		    RequestDispatcher rd = req.getRequestDispatcher(testURL);
		    rd.forward(req,resp);
                    return;
		}
	    }
            // file doesn't exist, so call default servlet to
            // display 404 error or file directory if enabled
            RequestDispatcher rd = getServletContext().getNamedDispatcher("default");
            rd.forward(req,resp);
	} else {
	    // file already exists and no other servlet exists 
	    // to handle it, so call default servlet
        handler_loop:
            for (i=0; i<ext_handlers.size(); i++) {
                HandlerInfo h = ((HandlerInfo)ext_handlers.elementAt(i));
                String ext = h.getExt();
                if (url.endsWith("."+ext)) {
                    int result = h.handle(req, resp, getServletContext());
                    switch (result) {
                    case Handler.HANDLER_OK:
                        break;
                    case Handler.HANDLER_RETURN:
                        // stop processing cold
                        return;
                    case Handler.HANDLER_BREAK:
                        break handler_loop;
                    }
                }
            }
            /* Try the default handler */
	    RequestDispatcher rd = getServletContext().getNamedDispatcher("default");
	    rd.forward(req,resp);
	}
    }

    /** 
     * Registers a request filter with a priority.  Lower numbers 
     * are higher priority.
     *
     * @see com.arsdigita.acs.Filter
     * @param path The relative URL path to filter
     * @param f A Filter adapter
     * @param priority The priority (lower is higher priority)
     */
    public static void registerFilter(String path, Filter f, int priority) {
        FilterInfo fi = new FilterInfo(f, path, priority);
	int i = 0;
	while (i < filters.size() && 
	       ((FilterInfo)filters.elementAt(i)).getPriority() <= priority)
	    i++;
	filters.insertElementAt(fi,i);
    }

    public static void registerHandler(String ext, Handler h) {
        HandlerInfo fi = new HandlerInfo(h, ext);
	ext_handlers.add(fi);
    }


    /** 
     * Registers a request filter with no particular priority. 
     *
     * @see com.arsdigita.acs.Filter
     * @param path The relative URL path to filter
     * @param f A Filter adapter
     */
    public static void registerFilter(String path, Filter f) {
	registerFilter(path,f,10000);
    }
}

class FilterInfo extends Filter {
    Filter f;
    String path;
    int priority;
    public FilterInfo(Filter f, String path, int priority) {
        this.f = f;
        this.path = path;
        this.priority = priority;
    }

    public void setPath(String s) { path = s; }

    public String getPath() { return path; }

    public void setPriority(int i) { priority = i; }
    
    public int getPriority() { return priority; }
    public int filter(HttpServletRequest req, HttpServletResponse resp, ServletContext ctx) throws ACSException {
        return f.filter(req, resp, ctx);
    }
}

class HandlerInfo implements Handler {
    Handler h;
    String ext;
    public HandlerInfo(Handler h, String ext) {
        this.h = h;
        this.ext = ext;
    }

    public void setExt(String s) { ext = s; }
    public String getExt() { return ext; }
    public int handle(HttpServletRequest req, HttpServletResponse resp, ServletContext ctx) throws ServletException, IOException {
        return h.handle(req, resp, ctx);
    }
}

class JspHandler implements Handler {
    public int handle(HttpServletRequest req, HttpServletResponse resp, ServletContext ctx) throws ServletException, IOException {
        RequestDispatcher rd = ctx.getNamedDispatcher("jsp");
        rd.forward(req, resp);
        return HANDLER_RETURN;
    }
}
