package com.arsdigita.acs;

import java.util.Vector;
import java.util.Date;
import java.util.Calendar;

/**
 * Utility class for running scheduled procedures, modelled after the
 * AOLserver ns_schedule_proc interface.  Has an interface for
 * daily/weekly procs.
 * <P>
 * Since procedures are not first-class objects in Java, we have to pass
 * in the procedure to be scheduled wrapped inside an object that implements
 * the java.lang.Runnable interface.  We define an abstract class, 
 * ScheduledProcedure, that serves as a record for maintaining 
 * data about a stored procedure, when it will run next, whether or not 
 * it is periodic, etc.
 * <P>
 * We keep the list of scheduled procedures in a priority queue, so that
 * the iteration over the list to figure out whose turn is next is done
 * when procedures are scheduled rather than on a regular basis.  Each
 * task is run in its own thread, so that other scheduled procedures are
 * not delayed due to one procedure's execution time.
 * <P>
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @see java.util.Timer
 * @see java.lang.Thread
 * @see com.arsdigita.acs.ScheduledProcedure
 * @version $Version$
 */
public class Scheduler {
    private static Scheduler instance;
    private Vector taskList;
    private Thread loopThread;
    private int counter = 0;

    /** 
     * Utility function to get a static Scheduler instance.
     * @return This JVM's Scheduler instance.
     */
    public static Scheduler getInstance() {
	if (instance==null) 
	    instance = new Scheduler();
	return instance;
    }

    /** 
     * Creates a new Scheduler interface and starts it running.
     */
    public Scheduler() {
	taskList = new Vector();
	loopThread = new LoopThread(taskList);
	loopThread.start();
    }
    
    private int scheduleInternal(ScheduledProcedure sp, long initialDelay, long period) { 

	// internal scheduling proc, returns the id of the
	// scheduled procedure

	if (period < 0) 
	    throw new IllegalArgumentException("Period must be non-negative.");
	if (period == 0) 
	    sp.recurring = false;
	else sp.recurring = true;
	sp.id = ++counter;
	sp.period = period;
	sp.nextTime = initialDelay + System.currentTimeMillis();
	
	// keep the list in sorted order (a priority queue!)
	synchronized(taskList) {
	    int i = 0;
	    while(i<taskList.size() && ((ScheduledProcedure)taskList.elementAt(i)).nextTime < sp.nextTime)
		i++;
	    taskList.insertElementAt(sp, i);
	    if (i==0) 
		// the task we scheduled just became the first, so
		// drop what we were going to do and do sp instead
		taskList.notify();
	}
	return sp.id;
    }
    
    private int scheduleInternal(ScheduledProcedure sp, long initialDelay) {
	return scheduleInternal(sp, initialDelay, 0);
    }
    
    /** 
     * Schedules a procedure to run every <code>interval</code> seconds.
     * @param sp Proc to schedule.
     * @param interval Time between execution in seconds.
     * @return The procedure's ID, for later unscheduling
     */
    public int schedule(ScheduledProcedure sp, long interval) { 
	return scheduleInternal(sp, interval*1000, interval*1000);
    }

    /** 
     * Schedules a procedure to run once in <code>interval</code> seconds.
     * @param sp Proc to schedule.
     * @param interval Time until execution, in seconds.
     * @return The procedure's ID, for later unscheduling
     */
    public int scheduleOnce(ScheduledProcedure sp, long interval) {
	return scheduleInternal(sp, interval*1000);
    }

    /** 
     * Schedules a procedure to run every day at <code>hour:minute</code>.
     * @param sp Proc to schedule.
     * @param hour Hour to run proc (in 24-hour time)
     * @param minute Minute to run proc
     * @return The procedure's ID, for later unscheduling
     */
    public int scheduleDaily(ScheduledProcedure sp,
			      int hour, 
			      int minute) { 
	Calendar now = Calendar.getInstance();
	Calendar c = (Calendar)now.clone();
	c.set(Calendar.HOUR_OF_DAY, hour);
	c.set(Calendar.MINUTE, minute);
	c.set(Calendar.SECOND,0);
	// make sure the first scheduled time is in the future
	if (c.before(now)) c.add(Calendar.DAY_OF_MONTH, 1);
	long initialDelay = c.getTime().getTime()-now.getTime().getTime();
	
	// TO DO: LOG
	return scheduleInternal(sp, initialDelay, 24*60*60*1000L);
    }

    /** 
     * Schedules a procedure to run once at <code>hour:minute</code>.
     * Waits until tomorrow if that time has already passed today.
     * 
     * @param sp Proc to schedule.
     * @param hour Hour to run proc (in 24-hour time)
     * @param minute Minute to run proc
     * @return The procedure's ID, for later unscheduling
     */
    public int scheduleDailyOnce(ScheduledProcedure sp,
				  int hour, 
				  int minute) { 
	Calendar now = Calendar.getInstance();
	Calendar c = (Calendar)now.clone();
	c.set(Calendar.HOUR_OF_DAY, hour);
	c.set(Calendar.MINUTE, minute);
	c.set(Calendar.SECOND, 0);
	// make sure the first scheduled time is in the future
	if (c.before(now)) c.roll(Calendar.DAY_OF_MONTH, true);
	long initialDelay = c.getTime().getTime()-now.getTime().getTime();
	// TO DO: LOG
	return scheduleInternal(sp, initialDelay);
    }
    

    /** 
     * Schedules a procedure to run each week at <code>hour:minute</code>, 
     * on <code>day</code>.
     * 
     * @see java.util.Calendar#DAY_OF_WEEK
     * @param sp Proc to schedule.
     * @param day Day of week to run proc.  Use the day-of-week constants
     * in {@link java.util.Calendar}. 
     * @param hour Hour to run proc (in 24-hour time)
     * @param minute Minute to run proc
     * @return The procedure's ID, for later unscheduling
     */
    public int scheduleWeekly(ScheduledProcedure sp,
				      int day, 
				      int hour,
				      int minute) { 
	Calendar now = Calendar.getInstance();
	Calendar c = (Calendar)now.clone();
	c.set(Calendar.HOUR_OF_DAY, hour);
	c.set(Calendar.MINUTE, minute);
	c.set(Calendar.DAY_OF_WEEK, day);
	c.set(Calendar.SECOND,0);
	if (c.before(now)) 
	    for (int i=0; i<7; i++) 
		c.roll(Calendar.DAY_OF_MONTH, true);
	// TO DO: LOG
	long initialDelay = c.getTime().getTime()-now.getTime().getTime();
	return scheduleInternal(sp, initialDelay, 7*24*60*60*1000L);
    }

    /** 
     * Schedules a procedure to run once at <code>hour:minute</code>, 
     * on <code>day</code>.  Waits for next week if that time has 
     * already passed this week.
     * 
     * @see java.util.Calendar#DAY_OF_WEEK
     * @param sp Proc to schedule.
     * @param day Day of week to run proc.  Use the day-of-week constants
     * in {@link java.util.Calendar}. 
     * @param hour Hour to run proc (in 24-hour time)
     * @param minute Minute to run proc
     * @return The procedure's ID, for later unscheduling
     */
    public int scheduleWeeklyOnce(ScheduledProcedure sp,
				  int day, 
				  int hour,
				  int minute) { 
	Calendar now = Calendar.getInstance();
	Calendar c = (Calendar)now.clone();
	c.set(Calendar.HOUR_OF_DAY, hour);
	c.set(Calendar.MINUTE, minute);
	c.set(Calendar.DAY_OF_WEEK, day);
	c.set(Calendar.SECOND,0);
	if (c.before(now)) 
	    for (int i=0; i<7; i++) 
		c.roll(Calendar.DAY_OF_MONTH, true);
	long initialDelay = c.getTime().getTime()-now.getTime().getTime();
	return scheduleInternal(sp, initialDelay);

    }

    /** 
     * Unschedules a scheduled procedure.
     * @param spId The scheduled proc ID to unschedule.
     */
    public void unschedule(int spId) {
	
	synchronized (taskList) {
	    int i=0;
	    while (i<taskList.size()) {
		ScheduledProcedure sp = (ScheduledProcedure)taskList.elementAt(i);
		if (sp.id == spId) {
		    taskList.removeElementAt(i);
		    sp.recurring = false; 
		    // be safe and make sure that at the very least, 
		    // we don't reschedule this procedure
		    break;
		}
		i++;
	    }
	    // deleted the proc about to run--wake up the loop and don't
	    // run it
	    if (i==0) taskList.notify(); 
	}
    }

    public static void main(String [] args) {
	Scheduler s = Scheduler.getInstance();
	ScheduledProcedure sp = new ScheduledProcedure() {
	    public void run() {
		System.out.println("HERE " + new Date());
	    }
	};

	ScheduledProcedure sp2 = new ScheduledProcedure() {
	    public void run() {
		System.out.println("THERE " + new Date());
	    }
	};

	System.out.println(s.schedule(sp, 3));
	try { 
	    Thread.sleep(5000); 
	    System.out.println(s.schedule(sp2, 2));
	    Thread.sleep(7324);
	    System.out.println("UNSCHEDULING #2 " + new Date());
	    s.unschedule(2);
	    System.out.println("UNSCHEDULED #2 " + new Date());
	} catch(Exception e) {} 
    }

    private class LoopThread extends Thread {
	private Vector taskList;

	public LoopThread(Vector taskList) {
	    this.taskList = taskList;
	}
	
	public void run() {
	    while (true) {
		try {
		    synchronized(taskList) {
			while (taskList.isEmpty())
			    taskList.wait();
			
			// get the first thing to do out of the queue
			final ScheduledProcedure sp = (ScheduledProcedure)taskList.elementAt(0);
			long timeToWait = sp.nextTime - System.currentTimeMillis();
			if (timeToWait > 0) 
			    taskList.wait(timeToWait);
			
			// if something else beats us to the punch,
			// it will notify us.

			// first, make sure that the procedure whose 
			// turn is next wasn't changed under our noses
			if ((Object)sp!=taskList.elementAt(0))
			    // if it changed, it must've been cancelled; 
			    // go back to top of loop
			    continue;
			
			// otherwise, run task in a new thread
			Thread taskThread = new Thread() {
			    public void run() {
				sp.run();
			    }
			};
			taskThread.start();
			// take the task out of the list
			taskList.removeElementAt(0);
			
			// if task is recurring, reinsert it at the 
			// appropriate place 
			if (sp.recurring) {
			    sp.nextTime = sp.nextTime + sp.period;
			    int i = 0;
			    while(i<taskList.size() && ((ScheduledProcedure)taskList.elementAt(i)).nextTime < sp.nextTime) i++;
			    taskList.insertElementAt(sp, i);
			}				
		    } // synchronized(taskList)
		} catch(InterruptedException ie) {}
	    } // while true()
	} // public void run()
    } 
}


