package com.arsdigita.acs;

import com.arsdigita.acs.db.*;

import java.io.*;
import java.util.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.http.*;
import com.oroinc.text.perl.Perl5Util;
import org.apache.turbine.util.Log;

/**
 * Utility procedures for verifying user log-in information.
 *
 * @author Jin Choi (jsc@photonet.arsdigita.com)
 * @version $Version$
 * @see com.arsdigita.acs.UserInfo
 */
public class Security {

    /** 
     * Helper function to checks the current HttpSession for a set
     * <code>UserInfo</code> object.  If the session is not set, it
     * uses the ad_user_login cookie to try to get and verify the user
     * info.
     *
     * @param req The current servlet request
     * @return A <code>UserInfo</code> object for the logged-in user,
     * or null if the user is not yet logged in.  */
    private static UserInfo getUserInfoHelper(HttpServletRequest req) {
	HttpSession session = req.getSession();
	UserInfo user = (UserInfo) session.getAttribute("user_info");

	if (user != null) {
	    return user;
	}

	// Check to see if the user has a permanent login cookie.
	String adUserLoginCookie = Utilities.getCookieValue(req, "ad_user_login");
	if (!Utilities.emptyString(adUserLoginCookie)) {
	    Perl5Util re = new Perl5Util();
	    if (re.match("/^(\\d+),(\\S+)/", Utilities.URLDecode(adUserLoginCookie))) {
		String userId = re.group(1);
		String permanentLoginToken = re.group(2);
		
		try {
		    Database db = new Database();
		    Object[] bind = new Object[2];	
		    bind[0] = userId;
		    bind[1] = permanentLoginToken;
		    
		    ResultSet rs = db.query("select u.user_id, first_names, last_name, email, user_state from users u, sec_login_tokens s where u.user_id = ? and user_state = 'authorized' and u.user_id = s.user_id and u.password = s.password and login_token = ?", bind);
		    user = new UserInfo(rs);
		    if (user != null) {
			return user;
		    }
		} catch (Exception e) {
 		    // Ignore errors, just bounce them to the login.  At least log it.
		    Log.warn("Security.getUserInfoHelper hit error: " + e.toString());
		}
	    }
	}
        return user;
    }

    /** 
     * Checks the current HttpSession for a set <code>UserInfo</code> object.
     * This object will be non-null if and only if the user has not already
     * logged in.  Redirects to /acs/register if the user is not yet
     * logged in and redirect is true.  Otherwise returns null. 
     * Must be called from a JSP tag class. 
     *
     * @param pc The current page context
     * @param redirect If true, redirects to the registration page if
     * the user is not yet logged in.
     * @return A <code>UserInfo</code> object for the logged-in user,
     * or null if the user is not yet logged in.
     */
    public static UserInfo getVerifiedUserInfo(PageContext pc, 
					       boolean redirect)
	throws IOException {

        UserInfo user = getUserInfoHelper((HttpServletRequest)pc.getRequest());
        if (user != null) {
            return user;
        }

	// if user is not logged in, return null if !redirect
	// otherwise redirect to registration page
	if (!redirect) 
	    return null;
	
	redirectForRegistration(pc);
	return null;
    }

    /**
     * Same as getVerifiedUserInfo(pc, true).
     */
    public static UserInfo getVerifiedUserInfo(PageContext pc) throws java.io.IOException { 
	return getVerifiedUserInfo(pc, true);
    }

    /**
     * A quicker/lighter weight way to get a user ID without ever hitting the database.
     * Thus, this ID is not verified and should not be relied on; it's reported
     * back straight from the user's cookie.
     *
     * @return User ID (from cookie) or null if cookie does not exist or is invalid.
     */
    public static Integer getUnverifiedUserID(HttpServletRequest req) {
        Integer retval = null;
	String adUserLoginCookie = Utilities.getCookieValue(req, "ad_user_login");
	if (!Utilities.emptyString(adUserLoginCookie)) {
	    Perl5Util re = new Perl5Util();
	    if (re.match("/^(\\d+),(\\S+)/", Utilities.URLDecode(adUserLoginCookie))) {
		String userId = re.group(1);
                try {
                    retval = Integer.valueOf(userId);
                    // To be consistent, always return null if user ID is unknown (i.e. not set or set to 0)
                    if (retval.intValue() == 0) {
                        retval = null;
                    }
                } catch (NumberFormatException e) {
                    // Ignore
                    Log.warn("Invalid number '" + userId + "' found for user Id in ad_user_login cookie, ignoring.");
                }
            }
        }
        return retval;
    }


    /**
     * Redirects to /register. 
     */
    public final static void redirectForRegistration(PageContext pc) 
	throws java.io.IOException { 
        redirectForRegistration((HttpServletRequest) pc.getRequest(), (HttpServletResponse) pc.getResponse());
    }

    /**
     * Redirects to /register. 
     */
    public final static void redirectForRegistration(HttpServletRequest request, HttpServletResponse response) 
        throws java.io.IOException { 

        // Redirect to /register/
        AdSet varsToForward = new AdSet();

        String returnURL = request.getRequestURI();

	String query = "";

	Enumeration enum = request.getParameterNames();
	int j = 0;
	while (enum.hasMoreElements()) {
	    j++;
	    if (j > 1) 
		query += "&" ;

	    String name = (String)enum.nextElement();
	    String[] vals = request.getParameterValues(name);
	    for (int i=0; i<vals.length; i++) {
		if (i > 0) {
		    query += "&";
		}
		query += (name + "=" + Utilities.URLEncode(vals[i]));
	    }
	    // System.out.println(query);
	}

        if (query.length() > 0) {
            returnURL += "?" + query;
        }
        varsToForward.put("return_url", returnURL);
	
        // we do this ugliness because for some strange reason Tomcat
        // puts in some ";jsessionID" crud when we have a question mark
        // in the redirect URL with no query 
        
        String redirectExports = Utilities.exportURLVars(varsToForward);
	redirectExports = "?" + redirectExports;
        
        String redirectURL = response.encodeRedirectURL("/register/index" + redirectExports);
        
        // before sending redirect we should free up the database handle
        // on this page context
        Database db = (Database)request.getAttribute("db");
        try {
            db.close();
        } catch(Exception e) { 
            
        }
        response.sendRedirect(redirectURL);
    }

    /**
     * Logs out the user.  Expires the ad_user_login cookie and takes the
     * user_info attribute out of the current HttpSession.
     */
    public static void logoutUser(PageContext pc) throws SQLException, ACSException { 
        loginUser(pc, new Integer(0), false);
        // We used to have another method here to do this, but that was bad becuase
        // it didn't maintain the session ID cookie... switching back to the TCL
        // method of just calling login w/ user ID 0 so that we're more likely
        // to work right and stay consistent with TCL's cookies.
	/*HttpSession session = pc.getSession();
	session.removeAttribute("user_info");
	Cookie loginCookie = new Cookie("ad_user_login", "");
	loginCookie.setPath("/");
	loginCookie.setMaxAge(100000000);
	HttpServletResponse response =  (HttpServletResponse) pc.getResponse();
	response.addCookie(loginCookie);*/
    }

    public static void loginUser(PageContext pc, Integer userId, boolean forever) throws SQLException, ACSException {
	Cookie loginCookie = new Cookie("ad_user_login", "");
	loginCookie.setPath("/");
        HttpServletRequest request = (HttpServletRequest) pc.getRequest();

        // Setup new session ID
        ClientIdentification id = (ClientIdentification)request.getAttribute("client_identification");
        id.resetUserId(userId);

	if (userId.intValue() == 0) {
	    // Hose the user's permanent login token.
	    loginCookie.setMaxAge(0);
            pc.getSession().removeAttribute("user_info");
	} else if (forever) {
	    if (request.isSecure()) {
		loginCookie.setSecure(true);
	    }
	    String loginToken = Utilities.secRandomToken();
	    // To avoid having more than one token inserted, do an INSERT WHERE
	    // and then select the token back.
	    Database db = new Database();
	    Object[] bind = new Object[2];
	    bind[0] = loginToken;
	    bind[1] = userId;
	    db.dml("insert into sec_login_tokens(user_id, password, login_token) " +
		   "select user_id, password, ? " + 
		   "from   users " +
		   "where  user_id = ? " + 
		   "and    (select count(*) from sec_login_tokens where user_id = users.user_id) = 0", bind);

	    bind[0] = userId;
	    loginToken = db.queryToString("select login_token " +
					  "from users u, sec_login_tokens t " +
					  "where  u.user_id = ? " +
					  "and t.user_id = ? " +
					  "and u.password = t.password", bind);
	    loginCookie.setMaxAge(100000000);
	    loginCookie.setValue(userId + "," + loginToken);
	}
	HttpServletResponse response = (HttpServletResponse) pc.getResponse();
	response.addCookie(loginCookie);
    }

    /** 
     * Checks the current HttpSession for a set <code>UserInfo</code> object.
     * This object will be non-null if and only if the user has not already
     * logged in.  Redirects to /register if the user is not yet
     * logged in and redirect is true.
     *
     * @param req The current servlet request
     * @param resp The curent servlet response
     * @param redirect If false, return null for not-logged-in user; 
     * if true, redirect for registration.
     * @return A <code>UserInfo</code> object for the logged-in user,
     * or null if the user is not yet logged in.
     */
    public static UserInfo getVerifiedUserInfo(HttpServletRequest req, 
					       HttpServletResponse resp,
					       boolean redirect)
	throws IOException {

        UserInfo user = getUserInfoHelper(req);
        if (user != null) {
            return user;
        }

	if (!redirect) 
	    return null;

	String returnURL = req.getRequestURI();
	String query = req.getQueryString();

	if (query != null) {
	    returnURL += "?" + query;
	}
	AdSet varsToForward = new AdSet();
	Enumeration enum = req.getParameterNames();
	while (enum.hasMoreElements()) {
	    String name = (String)enum.nextElement();
	    String[] vals = req.getParameterValues(name);
	    for (int i=0; i<vals.length; i++) {
		varsToForward.put(name, vals[i]);
	    }
	}
	varsToForward.put("return_url", returnURL);

	String redirectURL = resp.encodeRedirectURL("/register/?" + Utilities.exportURLVars(varsToForward));
	
	resp.sendRedirect(redirectURL);
	return null;
    }

    /**
     * Same as <code>getVerifiedUserInfo(req, resp, true)</code>
     */
    public static UserInfo getVerifiedUserInfo(HttpServletRequest req, 
					       HttpServletResponse resp) 
	throws IOException {
	return getVerifiedUserInfo(req, resp, true);
    }


    public static int getSessionTimeout() {
        return Parameters.getInt("SessionTimeout", "", 86400);
    }


    /* Do the actual database lookup for a property. */
    private static Property lookupProperty(boolean browser, Long id, String module, String name) {
        Property prop = null;
        String query;
        if (browser) {
            query = "select property_value, secure_p from sec_browser_properties " +
                "where browser_id = :id and module = :module and property_name = :name";
        } else {
            query = "select property_value, secure_p from sec_session_properties " +
                "where session_id = :id and module = :module and property_name = :name";
        }
        try {
            Database db = new Database();
            ResultSet rs = db.query(query, new Object[] { id, module, name} );

            if (rs.next()) {
                SingleSelection ss = new SingleSelection(rs);
                String value = ss.get("property_value");
                String secure_p = ss.get("secure_p");
                prop = new Property(value, secure_p.equals("t"));
            }
            db.close();
        } catch (Exception e) {
            Log.note("Got exception in lookupProperty" + e);
        }
        return prop;
    }

    /**
     * Looks up a property for the current session, or for the
     * browser. If cache is true, will use the cached value if
     * available. If cache_only is true, will never incur a database
     * hit (i.e., will only return a value if cached). If the property
     * is secure, we must be on a validated session over SSL.
     *
     * @param pc The pageContext
     * @param module The module
     * @param name The name
     * @param deflt The default value
     * @param browser If true use browser property, otherwise session property
     * @param cache If true, use the cached value if available
     * @param cache_only If true, only return a value if cached
     */
    public static String getClientProperty(PageContext pc, String module, String name,
                                           String deflt, boolean browser, boolean cache, boolean cache_only) {
        HttpServletRequest request = (HttpServletRequest) pc.getRequest();
        ClientIdentification client_id = (ClientIdentification)request.getAttribute("client_identification");
        
        Long id = browser ? client_id.getBrowserId() : client_id.getSessionId();

        String cache_cmd = "sec_lookup_property:" + browser + ":" + id + ":" + module + ":" + name;
        Property prop = (Property)MemoizeCache.lookup(cache_cmd);

        if (cache_only && prop == null) {
            return deflt;
        }

        if (!cache) {
            MemoizeCache.flush(cache_cmd);
        }

        if (prop == null) {
            /* Need to execute actual cmd and cache the value */
            prop = lookupProperty(browser, id, module, name);
            if (prop != null) MemoizeCache.cache(cache_cmd, prop, getSessionTimeout());
        }
        if (prop == null) {
            return deflt;
        }
        
        if (!prop.secure && request.isSecure()) {
            return deflt;
        }
        return prop.value;
    }

    /**
     * Looks up a property for the current session, or for the
     * browser. If the property is secure, we must be on a validated
     * session over SSL.
     *
     * @param pc The pageContext
     * @param module The module
     * @param name The name
     */
    public static String getClientProperty(PageContext pc, String module, String name) {
        return getClientProperty(pc, module, name, "", /*browser*/false, /* cache */true,  /*cache_only*/false);
    }

    /**
     * Looks up a property for the current session, or for the
     * browser. If the property is secure, we must be on a validated
     * session over SSL.
     *
     * @param pc The pageContext
     * @param module The module
     * @param name The name
     * @param deflt The default value
     */
    public static String getClientProperty(PageContext pc, String module, String name, String deflt) {
        return getClientProperty(pc, module, name, deflt, /*browser*/false, /* cache */true,  /*cache_only*/false);
    }

    /**
     * Looks up a property for the current session, or for the
     * browser. If the property is secure, we must be on a validated
     * session over SSL.
     *
     * @param pc The pageContext
     * @param module The module
     * @param name The name
     * @param deflt The default value
     * @param browser If true use browser property, otherwise session property
     */
    public static String getClientProperty(PageContext pc, String module, String name, String deflt, boolean browser) {
        return getClientProperty(pc, module, name, deflt, browser, /* cache */true,  /*cache_only*/false);
    }


    /**
     *
     * Sets a client (session- or browser-level) property. If
     * persistent is true, the new value will be written through to
     * the database. If secure is true, the property will not be
     * retrievable except via a validated, secure (HTTPS) connection.
     *
     * @param pc The pageContext
     * @param module The module
     * @param name The name
     * @param value The value to store
     * @param browser If true use browser property, otherwise session property
     * @param persistent Store the value in the database
     * @param secure Only access through a secure connection.
     */
    public static void setClientProperty(PageContext pc, String module, String name, String value,
                                         boolean browser, boolean persistent, boolean secure) throws ACSException {
        Log.note("setClientProperty: " + module + ":"+name+":"+value);
        HttpServletRequest request = (HttpServletRequest) pc.getRequest();
        ClientIdentification client_id = (ClientIdentification)request.getAttribute("client_identification");
        
        if (secure && !request.isSecure()) {
            throw new ACSException("Unable to set secure property in insecure or invalid session");
        }

        Long id = browser ? client_id.getBrowserId() : client_id.getSessionId();
        String kind = browser ? "browser" : "session";

        String cache_cmd = "sec_lookup_property:" + browser + ":" + id + ":" + module + ":" + name;
        
        if (persistent) {
            /* Setup the queries */
            Log.note("setClientProperty: doing queries");
            String prop_delete_dml = "	delete from sec_"+kind+"_properties "+
                "where  "+kind+"_id    = :id "+
                "and    module        = :module "+
                "and    property_name = :name";

            SqlStatement prop_insert_dml = new SqlStatement
                (" begin \n"+
                 "insert into sec_"+kind+"_properties "+
                 "   ("+kind+"_id, "+
                 "   module, "+
                 "   property_name, "+
                 "   secure_p, "+
                 "   property_value) "+
                 " values "+
                 "   ( :id, "+
                 "     :module, "+
                 "     :name, "+
                 "     :secure, "+
                 "     empty_clob())"+
                 " returning property_value into :1; " +
                 " \nend;");
            prop_insert_dml.bind(id);
            prop_insert_dml.bind(module);
            prop_insert_dml.bind(name);
            prop_insert_dml.bind(secure ? "t" : "f");
            try {
                Database db = new Database();
                db.beginTransaction();
                db.dml(prop_delete_dml, new Object[] { id, module, name });
                db.clobDml(prop_insert_dml, value);

                db.endTransaction();
                db.close();
            } catch (Exception e) {
                Log.note("Got exception in setClientProperty" + e);
            }
        }

        Property prop = new Property(value, secure);
        MemoizeCache.cache(cache_cmd, prop, getSessionTimeout());
    }

    public static void setClientProperty(PageContext pc, String module, String name, String value) throws ACSException {
        setClientProperty(pc, module, name, value, /* browser */false, /*persistent*/true, /*secure*/false);
    }

    public static void setClientProperty(PageContext pc, String module, String name, String value, boolean browser) throws ACSException {
        setClientProperty(pc, module, name, value, browser, /*persistent*/true, /*secure*/false);
    }
}

class Property {
    String value;
    boolean secure;
    public Property(String value, boolean secure) {
        this.value = value;
        this.secure = secure;
    }
}

