package com.arsdigita.acs;

import java.sql.*;
import java.util.*;
import java.io.*;
import com.arsdigita.acs.db.*;

/**
 * Represents a generic multiple-row data source pulled in from the 
 * database.  Provides a generic interface for getting properties and
 * filling the datasource from either a ResultSet or a SqlStatement 
 * and a database handle. 
 * 
 * <p>
 * Entity objects for doing logical operations on more specific 
 * database objects should extend Selection, adding additional accessors
 * for derived properties as needed.
 * <p>
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @version $Id: Selection.java,v 1.14 2000/10/11 20:18:16 bschneid Exp $
 */

public class Selection implements DataSource {
    /** 
     * The result set that we're managing
     */
    protected ResultSet rs;
    protected ResultSetMetaData rsmd;
    protected Database db;
    protected int ctr = 0;

    public Selection() { } 

    /**
     * Creates a new Selection, initialized from a ResultSet.
     */
    public Selection(ResultSet rs) throws SQLException {
	this.rs = rs;
	rsmd = rs.getMetaData();
    }

    /**
     * Utility constructor to initialize from a result set created
     * from a SqlStatement and a database handle.  
     */
    public Selection(SqlStatement sql, Database db) throws SQLException {
	this.db = db;
	rs = db.query(sql);
	rsmd = rs.getMetaData();
    }

    /** 
     * Returns <code>true</code> if this contains a column named 
     * <code>column</code>.
     */
    public boolean contains(String column) throws DataSourceException {
	try {
	    for (int i = 1; i <= rsmd.getColumnCount(); i++) {
		if (column.equalsIgnoreCase(rsmd.getColumnLabel(i))) {
		    // see if the value is not null
		    return true;
		}
	    }
	    return false;
	} catch(SQLException e) {
	    throw new DataSourceException(e.getMessage());
	}
    }

    /**
     * Returns a mapping from column names to objects, to stash
     * a current set of values for a row.  Serves a similar purpose as 
     * set_variables_after_query.
     */
    protected Hashtable freeze() throws SQLException,
    IOException {
	Hashtable ht = new Hashtable();
	for (int i = 1; i <= rsmd.getColumnCount(); i++) {
	    String columnLabel = rsmd.getColumnLabel(i).toLowerCase();
	    // see if this column is a CLOB or BLOB;
	    // if it's a CLOB, just get the string out so a template
	    // can just dump it out as a string.  If it's a BLOB,
	    // same deal, but store it as a byte[].
	    int type = rsmd.getColumnType(i);
	    if (type == Types.CLOB) {
		ht.put(columnLabel, Database.getCLOB(rs, columnLabel));
	    } else if (type == Types.BLOB) {
		ht.put(columnLabel, Database.getBLOB(rs, columnLabel));
	    } else if (type == Types.DATE || type == Types.TIMESTAMP) {
		// by default, only return ANSI date YYYY-MM-DD;
		// use to_char if you want anything else.
		// behavior of ResultSet.getString() on date 
		// may be non-portable and db-dependent;
		// java.sql.Date.toDate() should be more consistent.
		java.sql.Date date = rs.getDate(columnLabel);
		if (date == null && columnLabel != null) {
		    ht.put(columnLabel, "");
		} else if (date != null) {
		    ht.put(columnLabel, date.toString());
		}
	    } else {
		if (columnLabel != null) {
		    if (rs.getObject(i) == null) {
			ht.put(columnLabel, ""); 
		    } else {
			ht.put(columnLabel, rs.getObject(i));
		    }
		}
	    }
	}
	return ht;
    }

    /**
     * Generic get-property accessor.  Returns the value of the column
     * named by <code>key</code>; if it's a CLOB column, still returns the
     * text from the CLOB.  If it's a DATE column, returns an ANSI date
     * string (yyyy-mm-dd).  Returns an empty string if the column is null
     * (a perhaps unnecessary evil to prevent the string "null" from popping 
     * up in JSP pages--probably should get rid of this with more consistent
     * use of ad:sub tag!).
     * 
     * @exception com.arsdigita.acs.InvalidKeyException if key is not found.
     */
    
    public String get(String key) throws DataSourceException {
	
	try {
	    if (this.contains(key)) {
		int type = -1;
		for (int i = 1; i <= rsmd.getColumnCount(); i++) {
		    if (rsmd.getColumnName(i).toLowerCase().equals(key)) {
			type = rsmd.getColumnType(i);
		    }
		}
		if (type == Types.CLOB) {
		    return Database.getCLOB(rs, key);
		} else if (type == Types.DATE || type == Types.TIMESTAMP) {
		    // by default, only return ANSI date YYYY-MM-DD;
		    // use to_char if you want anything else.
		    // behavior of ResultSet.getString() on date 
		    // may be non-portable and db-dependent;
		    // java.sql.Date.toDate() should be more consistent.
		    java.sql.Date d = rs.getDate(key);
		    return (d == null) ? "" : d.toString();
		} else {
		    String s = rs.getString(key);
		    return (s == null)? "" : s;
		}
	    }
	    else
		throw new InvalidKeyException("DataSource does not countain field '" + key + "'");
	} catch (SQLException e) {
	    throw new DataSourceException(e.getMessage());
	} catch (java.io.IOException e) {
	    throw new DataSourceException(e.getMessage());
	}
    }

    /** 
     * Advances to the next row in the data source, if there is one.
     * @return true; or false if there are no more elements.
     */

    public boolean next() throws DataSourceException {
	try {
	    ctr++;
	    return rs.next();
	} catch (SQLException e) {
	    throw new DataSourceException(e.getMessage());
	}
    }

    /** 
     * Returns a list of available keys (column names) in the Selection.
     */

    public String[] keys() throws DataSourceException {
	try { 
	    Vector v = new Vector();
	    for (int i = 1; i <= rsmd.getColumnCount(); i++) {
		v.addElement(rsmd.getColumnLabel(i).toLowerCase());
	    }
	    String[] retval = new String[v.size()];
	    v.copyInto(retval);
	    return retval;
	} catch (SQLException e) {
	    throw new DataSourceException(e.getMessage());
	}
    }

    /**
     * Returns the contents of a BLOB column in this Selection.
     * (note that, since CLOBs are handled transparently in get(), 
     * we don't need a getCLOB() method!)
     * @param key The column name to retrieve 
     * @return A byte array with the BLOB contents.
     */
    public byte[] getBLOB(String key) throws DataSourceException { 
	try {
	    return Database.getBLOB(rs, key);
	} catch (Exception e) { 
	    throw new DataSourceException(e.getMessage());
	}
    }

    /**
     * Dumps the contents of a BLOB column to an OutputStream.
     * @param key The column name
     * @param os The output stream
     */
    public void dumpBLOB(String key, OutputStream os) 
	throws DataSourceException { 
	try {
	    Database.dumpBLOB(rs, key, os);
	} catch (Exception e) { 
	    throw new DataSourceException(e.getMessage());
	}
    }

    /**
     * Returns the current row number in the datasource.
     */
    public int rownum() throws DataSourceException { 
	return ctr;
    }
}

    

