package com.arsdigita.acs;

import java.sql.*;
import com.arsdigita.acs.db.Database;
import com.arsdigita.acs.ug.*;

/**
 * 
 * Class to abstract information about a user of the web service.  
 * 
 * @version $Version$ 
 * @author Jin Choi (jsc@photonet.arsdigita.com)
 */
public class UserInfo {
    private Integer userId;
    private String firstNames;
    private String lastName;
    private String email;
    private String userState;
    
    /**
     * Validates a user login attempt. If successful, returns a new 
     * <code>UserInfo</code> instance.  Otherwise, throws 
     * <code>ACSException</code> with a detail message, either 
     * "password incorrect" or "user not found".
     * 
     * @param email The user's email address
     * @param passwd The user's supplied password, to validate
     * @exception com.arsdigita.acs.ACSException
     */
    public UserInfo(String email, String passwd) throws SQLException, ACSException {
	Database db = new Database();
	Object[] bind = new Object[1];
	bind[0] = email;
	ResultSet rs = db.query("select user_id, first_names, last_name, email, password, user_state from users where email = ? ", bind);
	//and user_state = 'authorized'
	if (!rs.next()) {
	    throw new ACSException("user not found");
	}

	if (! passwd.equals(rs.getString("password"))) {
	    throw new ACSException("password incorrect");
	}

	initFromResultSet(rs);
	db.close();
    }

    public UserInfo(String userId) throws SQLException, ACSException {
	this(Integer.parseInt(userId));
    }

    public UserInfo(int userId) throws SQLException, ACSException {
	Database db = new Database();
	Object[] bind = new Object[1];
	bind[0] = new Integer(userId);
	ResultSet rs = db.query("select user_id, first_names, last_name, email, password, user_state from users where user_id = ? ", bind);
	//and user_state = 'authorized'
	if (!rs.next()) {
	    throw new ACSException("user not found");
	}

	initFromResultSet(rs);
	db.close();
    }

    UserInfo(ResultSet rs) throws SQLException, ACSException {
	if (!rs.next()) {
	    throw new ACSException("user not found");
	}

	initFromResultSet(rs);
    }

    private void initFromResultSet(ResultSet rs)
	throws SQLException, ACSException {
	
	userId = new Integer(rs.getInt("user_id"));
	firstNames = rs.getString("first_names");
	lastName = rs.getString("last_name");
	email = rs.getString("email");
	userState = rs.getString("user_state");
    }

    /**
     * @return The user's user ID.
     */
    public Integer getUserId() {
	return userId;
    }

    /**
     * @return The user's first name.
     */
    public String getFirstNames() {
	return firstNames;
    }

    /**
     * @return The user's last name.
     */
    public String getLastName() {
	return lastName;
    }

    /**
     * @return The user's full (first + last) name.
     */
    public String getFullName() {
	return firstNames + " " + lastName;
    }

    /**
     * @return The user's email address.
     */
    public String getEmail() {
	return email;
    }

    /**
     * @return The user's registration state
     */
    public String getUserState() {
	return userState;
    }

    public String toString() {
	return userId + ": " + firstNames + " " + lastName + "<" + email + ">";
    }

    /**
     * Verifies that this user is a member of <code>groupId</code>.
     *
     * @return <code>true</code> if the user is a member of 
     * <code>groupId</code>; otherwise <code>false</code>
     */
    public boolean isGroupMember(Object groupId) throws SQLException, ACSException {
	if (groupId == null) 
	    return false;
	Database db = new Database();
	Integer count = new Integer(db.queryToString("select count(*) from user_group_map where group_id = ? and user_id = ?", new Object[] {groupId, this.userId}));
	db.close();
	return (count.intValue() > 0);
    }

    /**
     * Verifies that this user is an administrator of <code>groupId</code>.
     *
     * @return <code>true</code> if the user is a member of 
     * <code>groupId</code>; otherwise <code>false</code>
     */
    public boolean isGroupAdmin(Object groupId) throws SQLException, ACSException {
	if (groupId == null) 
	    return false;
	Database db = new Database();
	Integer count = new Integer
	    (db.queryToString
	     ("select count(*) from user_group_map where group_id = ? and user_id = ? and lower(role)='administrator'", 
	      new Object[] {groupId, this.userId}));
	db.close();
	return (count.intValue() > 0);
    }

    /**
     * Verifies that this user is a member of administration group for
     * the given module and submodule.
     *
     * @param module The module
     * @param submodule The submodule (or null/empty if none)
     * @return <code>true</code> if the user is a member of the 
     * <code>module</code> admin group; otherwise <code>false</code>
     */
    public boolean isAdministrationGroupMember(String module, 
					       String sub) 
	throws SQLException, ACSException {
	try {
	    SingleGroupDataSource g = new SingleGroupDataSource(module,sub);
	    return isGroupMember(g.get("group_id"));
	}
	catch (ACSException e) { 
	    return false;
	}
	catch (DataSourceException e) { 
	    return false;
	}
    }

    /** 
     * returns true if a user is a site-wide admin.
     */
    public boolean isSiteWideAdmin() {
	Database db = null;
	try {
	    db = new Database();
	    String s = db.queryToString("select ad_group_member_p(?, system_administrator_group_id) from dual", new Object[] {userId});
	    return ("t".equals(s));
	}
	catch (Exception e) { 
	    return false;
	}
	finally {
	    try {
		db.close();
	    }
	    catch(Exception e) {}
	}
    }	    
}


