package com.arsdigita.acs;

import java.util.Enumeration;
import java.util.Vector;
import java.io.*;
import java.sql.*;
import com.oroinc.text.perl.Perl5Util;
import javax.servlet.http.*;
import org.apache.turbine.util.Log;
import java.net.*;

/**
 * Utility procedures for ACS.
 * 
 * @author Jin Choi (jsc@photonet.arsdigita.com)
 * @author Bill Schneider (bschneid@arsdigita.com)
 */

public class Utilities {
    private static java.util.Random rng = new java.util.Random();

    /**
     * Note: You must NOT use group() on re, or you may get weird results
     * if you have multiple requests using re at the same time.  You must
     * allocate a fresh Perl5Util object if you want to do grouping.
     */
    private static Perl5Util re = new Perl5Util();
    

    /**
     * Quotes "special" characters in HTML text (ampersand, angle brackets,
     * quote).
     * @param s The string to quote
     * @return The quoted string
     */
    public final static String quoteHtml(String s) {
	s = re.substitute("s/&/&amp;/g", s);
	s = re.substitute("s/\"/&quot;/g", s);
	s = re.substitute("s/</&lt;/g", s);
	s = re.substitute("s/>/&gt;/g", s);
	return s;
    }

    /** 
     * Takes a plaintext string, and returns an HTML string that, when 
     * rendered by a web browser, will appear as the original input string
     *
     * @param s The input plaintext string
     * @return A HTML string with blank lines coverted to <pre><p></pre>
     * and ampersands/angle brackets escaped.
     */
    public final static String textToHtml(String s) {
	s = quoteHtml(s);
	s = re.substitute("s/\\015\\012\\015\\012/<p>/g", s);
	s = re.substitute("s/\\012\\012/<p>/g", s);
	s = re.substitute("s/\\015\\015/<p>/g", s);
	return s;
    }
    
    /** 
     * Removes tags and substitutes P tags with newlines.
     */
    public final static String htmlToText(String s) {
	// first take out new-lines
	s = re.substitute("s/\\015//g", s);
	s = re.substitute("s/\\012//g", s);
	s = re.substitute("s/<[Pp]>/\\012\\012/g", s);
	s = re.substitute("s/<br>/\\012/ig", s);
	// take out other tags
	s = re.substitute("s/<[^>]>//g", s);
	return s;
    }

    public final static String maybeHtmlToText(String s, String html_p) {
	if (html_p.equals("t")) 
	    return htmlToText(s);
	else return s;
    }
    
    /** 
     * Exports variables for use in a form, as input type=hidden.
     * @param vars An <code>AdSet</code> of variable names and values.
     * @return An HTML string for inclusion within HTML &lt;FORM&gt; tags.
     */
    public final static String exportFormVars(AdSet vars) {
	return exportFormVars(vars, null);
    }

    /** 
     * Exports variables for use in a form, as input type=hidden.
     * @param vars An <code>AdSet</code> of variable names and values.
     * @param exclusionList A space-separated list of fields in the AdSet to ignore (or null to export all of them)
     * @return An HTML string for inclusion within HTML &lt;FORM&gt; tags.
     */
    public final static String exportFormVars(AdSet vars, String exclusionList) {
	StringBuffer buf = new StringBuffer();

	Vector exclude = null;
	int excludeLen = 0;
	if (exclusionList != null) {
	    exclude = re.split("/\\s+/", exclusionList);
	    excludeLen = exclude.size();
	}

	Enumeration e = vars.keysAndValues();
	nextVar: while (e.hasMoreElements()) {
	    String[] pair = (String[]) e.nextElement();
	    String key = pair[0];
	    String value = pair[1];
	    
	    for (int i = 0; i < excludeLen; i++) {
		if (((String)exclude.elementAt(i)).equals(key)) {
		    continue nextVar;
		}
	    }
	    buf.append("<input type=hidden name=\"");
	    buf.append(key);
	    buf.append("\" value=\"");
	    buf.append(quoteHtml(value));
	    buf.append("\">\n");
	}
	return buf.toString();
    }

    /** 
     * Exports variables for use in a URL, as name=value&amp;name=value...
     * @param vars An <code>AdSet</code> of variable names and values.
     * @return An HTML string usable as the query string after a URL.
     * (Does NOT include the initial "?" character.)
     */
    public final static String exportURLVars(AdSet vars) {
	return exportURLVars(vars, null);
    }

    /** 
     * Exports variables for use in a URL, as name=value&amp;name=value...
     * @param vars An <code>AdSet</code> of variable names and values.
     * @param exclusionList A space-separated list of fields in the AdSet to ignore (or null to export all of them)
     * @return An HTML string usable as the query string after a URL.
     * (Does NOT include the initial "?" character.)
     */
    public final static String exportURLVars(AdSet vars, String exclusionList) {
	StringBuffer buf = new StringBuffer();

	Vector exclude = null;
	int excludeLen = 0;
	if (exclusionList != null) {
	    exclude = re.split("/\\s+/", exclusionList);
	    excludeLen = exclude.size();
	}

	boolean firstTime = true;
	Enumeration e = vars.keysAndValues();

	nextVar: while (e.hasMoreElements()) {
	    String[] pair = (String[]) e.nextElement();
	    String key = pair[0];
	    String value = pair[1];

	    for (int i = 0; i < excludeLen; i++) {
		if (((String)exclude.elementAt(i)).equals(key)) {
		    continue nextVar;
		}
	    }
	    if (! firstTime) {
		buf.append('&');
	    }
	    buf.append(key);
	    buf.append('=');
	    buf.append(URLEncode(value));
	    firstTime = false;
	}
	return buf.toString();
    }

    /**
     * Exports variables for use in a URL, as name=value&amp;name=value...
     * @param rs A <code>ResultSet</code>.
     * @param vars A space-separated list of fields from the result set
     * to export. If null, all fields are exported.
     */
    public final static String exportURLVars(ResultSet rs, String vars) {
	StringBuffer buf = new StringBuffer();

	boolean firstTime = true;

	try {
	    if (vars != null) {
		Vector v = re.split("/\\s+/", vars);
		int len = v.size();
	    
		for (int i = 0; i < len; i++) {
		    String key = (String) v.elementAt(i);
		    String value = rs.getString(key);
		    
		    if (value != null) {
			if (! firstTime) {
			    buf.append('&');
			}
			buf.append(key);
			buf.append('=');
			buf.append(URLEncode(value));
			firstTime = false;
		    }
		}
	    } else {
		ResultSetMetaData rsmd = rs.getMetaData();
		int nCols = rsmd.getColumnCount();
		for (int i = 1; i <= nCols; i++) {
		    String key = rsmd.getColumnName(i).toLowerCase();
		    String value = rs.getString(key);
		    
		    if (value != null) {
			if (! firstTime) {
			    buf.append('&');
			}
			buf.append(key);
			buf.append('=');
			buf.append(URLEncode(value));
			firstTime = false;
		    }
		}
	    }
	} catch (SQLException e) {
	}

	return buf.toString();
    }

    /**
     * Exports variables for use in a form, as hidden input fields.
     * @param rs A <code>ResultSet</code>.
     * @param vars A space-separated list of fields from the result set
     * to export. If null, all fields are exported.
     */
    public final static String exportFormVars(ResultSet rs, String vars) {
	StringBuffer buf = new StringBuffer();

	try {
	    if (vars != null) {
		Vector v = re.split("/\\s+/", vars);
		int len = v.size();
	    
		for (int i = 0; i < len; i++) {
		    String key = (String) v.elementAt(i);
		    String value = rs.getString(key);
		    
		    if (value != null) {
			buf.append("<input type=hidden name=\"" + key + "\" value = \"" + quoteHtml(value) + "\">");
		    }
		}
	    } else {
		ResultSetMetaData rsmd = rs.getMetaData();
		int nCols = rsmd.getColumnCount();
		for (int i = 1; i <= nCols; i++) {
		    String key = rsmd.getColumnName(i).toLowerCase();
		    String value = rs.getString(key);
		    
		    if (value != null) {
			buf.append("<input type=hidden name=\"" + key + "\" value = \"" + quoteHtml(value) + "\">");
		    }
		}
	    }
	} catch (SQLException e) {
	}

	return buf.toString();
    }

    /**
     * Exports variables for use in a URL, as name=value&amp;name=value...,
     * or in a form (input type=hidden name=... value=...)
     * @param ds A <code>DataSource</code>. 
     * @param vars A space-separated list of fields from the result set
     * to export. If null, all fields are exported.
     * @param form if <code>true</code>, export as form vars, else export
     * as URL vars
     */
    public final static String exportVars(DataSource ds, 
					  String vars, 
					  boolean form) {
	StringBuffer buf = new StringBuffer();

	boolean firstTime = true;

	try {
	    String[] keys = null;
	    if (vars != null) {
		Vector v = re.split("/\\s+/", vars);
		keys = new String[v.size()];
		v.copyInto(keys);
	    } else {
		keys = ds.keys();
	    }
	    int len = keys.length;
	    for (int i = 0; i < len; i++) {
		String key = keys[i];
		String value = ds.get(key);
		if (value != null) {
		    if (!form) {
			if (! firstTime) {
			    buf.append('&');
			}
			buf.append(key);
			buf.append('=');
			buf.append(URLEncode(value));
			firstTime = false;
		    } else {
			buf.append("<input type=hidden name=\"");
			buf.append(key);
			buf.append("\" value=\"");
			buf.append(quoteHtml(value));
			buf.append("\">\n");
		    }
		}
	    }
	} catch (DataSourceException e) {
	}
	return buf.toString();
    }

    /**
     * Exports a single variable for use in a form, as input type=hidden.
     * @param key Variable name.
     * @param value Variable value.
     */
    public final static String exportFormVar(String key, String value) {
        return ("<input type=hidden name=\"" + key + "\" value=\"" + quoteHtml(value) + "\">\n");
    }

    /**
     * Exports variables for use in a URL, as name=value&amp;name=value...,
     * or in a form (input type=hidden name=... value=...) from a servlet
     * requests.  Pulls values from the attributes first, then from 
     * the form variables
     * @param req A servlet request
     * @param vars A space-separated list of fields from the result set
     * to export. If null, all form variables are exported.
     * @param form if <code>true</code>, export as form vars, else export
     * as URL vars
     */
    public final static String exportVars(HttpServletRequest req,
					  String vars, 
					  boolean form) {
	StringBuffer buf = new StringBuffer();

	boolean firstTime = true;

	try {
	    Vector v = null;
	    if (vars != null) {
		v = re.split("/\\s+/", vars);
	    } else {
		Enumeration enum = req.getParameterNames();
		while (enum.hasMoreElements()) {
		    v.addElement(enum.nextElement());
		}
	    }
	    int len = v.size();
	    for (int i = 0; i < len; i++) {
		String key = (String)v.elementAt(i);
		// try attributes first
		Object o = req.getAttribute(key);
		String value = (o == null)? null : o.toString();
		if (value == null) 
		    value = req.getParameter(key);
		if (value != null) {
		    if (!form) {
			if (! firstTime) {
			    buf.append('&');
			}
			buf.append(key);
			buf.append('=');
			buf.append(URLEncode(value));
			firstTime = false;
		    } else {
			buf.append("<input type=hidden name=\"");
			buf.append(key);
			buf.append("\" value=\"");
			buf.append(quoteHtml(value));
			buf.append("\">\n");
		    }
		}
	    }
	} catch (Exception e) {
	}
	return buf.toString();
    }

    /**
     * Exports variables for use in a URL, as name=value&amp;name=value...,
     * @param ds A <code>DataSource</code>. 
     * @param vars A space-separated list of fields from the result set
     * to export. If null, all fields are exported.
     */
    public final static String exportURLVars(DataSource ds, String vars) {
	return exportVars(ds, vars, false);
    }

    /**
     * Exports variables for use in a form, as input type=hidden
     * @param ds A <code>DataSource</code>. 
     * @param vars A space-separated list of fields from the result set
     * to export. If null, all fields are exported.
     */
    public final static String exportFormVars(DataSource ds, String vars) {
	return exportVars(ds, vars, true);
    }

    /**
     * Exports variables for use in a URL, as name=value&amp;name=value...,
     * @param req A servlet request
     * @param vars A space-separated list of fields from the result set
     * to export. If null, all fields are exported.
     */
    public final static String exportURLVars(HttpServletRequest req, 
					     String vars) {
	return exportVars(req, vars, false);
    }

    public final static String exportFormVars(HttpServletRequest req, 
					     String vars) {
	return exportVars(req, vars, true);
    }

    /** 
     * URL-encodes a string.  
     * NOTE: is there a routine for this in javax.servlet.http?
     * @param s The string to encode
     * @return The encoded string.
     */
    public final static String URLEncode(String s) {
	StringBuffer buf = new StringBuffer();

	int strlen = s.length();
	for (int i = 0; i < strlen; i++) {
	    char c = s.charAt(i);
	    if (! ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')
		   || (c >= '0' && c <= '9')
		   || c == '_' || c == '.' || c == '-')) {
		buf.append('%');
		buf.append(Integer.toHexString(c));
	    } else {
		buf.append(c);
	    }
	}
	return buf.toString();
    }

    /**
     * Returns "value="form_value"", URL-encoded appropriately for inclusion
     * in an HTML form.
     * @param val The value to encode.
     */
    public final static String exportFormValue(String val) { 
	return "value=\"" + URLEncode(val) + "\"";
    }

    /**
     * This method decodes the given urlencoded string.
     *
     * @param  str the url-encoded string
     * @return the decoded string
     * @exception IllegalArgumentException If a '%' is not
     * followed by a valid 2-digit hex number.
     *
     * @author: cut & paste from JServ, much faster that previous tomcat impl 
     */
    public final static String URLDecode(String str)
	throws NumberFormatException, StringIndexOutOfBoundsException
    {
        if (str == null)  return  null;

        StringBuffer dec = new StringBuffer();    // decoded string output
        int strPos = 0;
        int strLen = str.length();

        dec.ensureCapacity(str.length());
        while (strPos < strLen) {
            int laPos;        // lookahead position

            // look ahead to next URLencoded metacharacter, if any
            for (laPos = strPos; laPos < strLen; laPos++) {
                char laChar = str.charAt(laPos);
                if ((laChar == '+') || (laChar == '%')) {
                    break;
                }
            }

            // if there were non-metacharacters, copy them all as a block
            if (laPos > strPos) {
                dec.append(str.substring(strPos,laPos));
                strPos = laPos;
            }

            // shortcut out of here if we're at the end of the string
            if (strPos >= strLen) {
                break;
            }

            // process next metacharacter
            char metaChar = str.charAt(strPos);
            if (metaChar == '+') {
                dec.append(' ');
                strPos++;
                continue;
            } else if (metaChar == '%') {
		// We throw the original exception - the super will deal with it
		//                try {
		dec.append((char) Integer.parseInt(
						   str.substring(strPos + 1, strPos + 3), 16));
		//                } catch (NumberFormatException e) {
		//                    throw new IllegalArgumentException("invalid hexadecimal "
		//                    + str.substring(strPos + 1, strPos + 3)
		//                    + " in URLencoded string (illegal unescaped '%'?)" );
		//                } catch (StringIndexOutOfBoundsException e) {
		//                    throw new IllegalArgumentException("illegal unescaped '%' "
		//                    + " in URLencoded string" );
		//                }
                strPos += 3;
            }
        }

        return dec.toString();
    }

    /** 
     * Returns a cookie value as a String, given a cookie name.
     * @param request The servlet request
     * @param withName The cookie name
     * @return The cookie value
     * @see javax.servlet.http.Cookie
     * @see javax.servlet.http.HttpServletRequest#getCookies()
     */
    public final static String getCookieValue(HttpServletRequest request, String withName) {
	Cookie[] cookies = request.getCookies();

	if (cookies == null) {
	    return null;
	}

	for (int i = 0; i < cookies.length; i++) {
	    Cookie c = cookies[i];
	    if (c.getName().equals(withName)) {
		return c.getValue();
	    }
	}
	return null;
    }

    /** 
     * Converts a <code>java.util.Vector</code> to a String joined on 
     * another String. Analogous to the <code>join</code> procedure in 
     * Tcl or Perl.  
     * @param v The <code>Vector</code> to join.
     * @param s The string inserted between each element of <code>v</code>
     * @return The list of elements in <code>v</code>, separated by 
     * <code>s</code>
     */
    public final static String join(Vector v, String s) {
	StringBuffer buf = new StringBuffer();
	Enumeration e = v.elements();

	if (e.hasMoreElements()) {
	    buf.append(e.nextElement().toString());
	}
	while (e.hasMoreElements()) {
	    buf.append(s);
	    buf.append(e.nextElement().toString());
	}
	return buf.toString();
    }

    /** 
     * Converts an array to a String joined on 
     * another String. Analogous to the <code>join</code> procedure in 
     * Tcl or Perl.  
     * @param v The <code>Vector</code> to join.
     * @param s The string inserted between each element of <code>v</code>
     * @return The list of elements in <code>v</code>, separated by 
     * <code>s</code>
     */
    public final static String join(Object[] o, String s) {
	StringBuffer buf = new StringBuffer();
	if (o.length > 0) { 
	    buf.append(o[0].toString());
	}
	for (int i = 1; i < o.length; i++) {
	    buf.append(s);
	    buf.append(o[i].toString());
	}
	return buf.toString();
    }

    /**
     * Generates a single random character.
     * @return A random alphanumeric character, a period, or slash.
     */
    public final static char secRandomChar() {
	final String legalChars = 
	    "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz./";
	return(legalChars.charAt(rng.nextInt(64)));
    }

    /** 
     * Digests a string into eight ASCII characters 
     * (horribly insecurely). 
     * @param str The string input
     * @return The digested string
     */
    public final static String secDigestString (String str) {
	
	// This was fed to ns_crypt to help generate random data. 
	int chars[] = new int[] {32,32,32,32,32,32,32,32};
	for (int i = 0; i<str.length(); i++) {
	    int chr = str.charAt(i);
	    int index = i % 8;
	    chars[index] = chars[index] + chr;
	}

	StringBuffer sb = new StringBuffer(); 
	for (int i = 0; i<8; i++) {
	    sb.append((char)(chars[i]%93+33));
	}
	return sb.toString();
    }
    
    /**
     * Generates a random token, using the TokenLength as the token 
     * length.  Reads <code>TokenLength</code> and <code>TokenSecret</code>
     * parameters from <code>Parameters</code>, in no module.
     * @return The random token
     */
    public final static String secRandomToken() {
	
	StringBuffer token = new StringBuffer();
	int length = Parameters.getInt("TokenLength", "", 32); 
	String secret = Parameters.get("TokenSecret", "", "tkpwtfzp!");
	while (token.length() < length) {
	    //  Digest a funky random string and ns_crypt it.
	    StringBuffer str = new StringBuffer();
	    for (int i = 0; i<8; i++) 
		str.append(secRandomChar());
	    str.append(secret);
	    for (int i = 0; i<8; i++) 
		str.append(secRandomChar());
	    String digested = secDigestString(str.toString());
	    StringBuffer salt = new StringBuffer();
	    salt.append(secRandomChar());
	    salt.append(secRandomChar());
	    String crypted = Crypt.crypt(salt.toString(),digested);
	    // # Pull the non-salt characters out.
	    // append token [string range $crypted 2 12]
	    token.append(crypted.substring(2,12));
	}
	token.setLength(length);
	return token.toString();
    }

    /** 
     * @param s A string to test
     * @return <code>true</code> if <code>s</code> is null or empty; 
     * otherwise <code>false</code>
     */
    public final static boolean emptyString(String s) {
	if (s == null || s.length() == 0) 
	    return true;
	else return false;
    }


    /**
     * Analogous to HttpServletResponse.sendRedirect(), but necessary
     * for redirecting if there have been any headers set (such as
     * cookie headers), since (in at least the Tomcat implementation)
     * sendRedirect() ignores any header objects attached to the
     * response.
     * @param response Response object to send redirect with.
     * @param location URL to redirect to.
     */
    public final static void sendRedirect(HttpServletResponse response, String location) throws IOException {
	response.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
	response.addHeader("Location", location);
	response.getOutputStream().close();
    }

    /** 
     * Removes duplicate entries from a Vector.
     * @param v The Vector to modify
     */
    public final static void removeDups(Vector v) {
	for (int i = 0; i<v.size(); i++) {
	    int j = i + 1; 
	    while (j < v.size()) {
		if (v.elementAt(i).equals(v.elementAt(j))) {
		    v.removeElementAt(j);
		    if (j >= v.size()) 
			break;
		} else {
		    j++;
		}
	    }
	}
    }
    
    public final static String prettyBoolean(Object in) {
	if (in != null && (in.equals("t") || in.equals(new Integer(1)))) 
	    return "Yes";
	else 
	    return "No";
    }

    public final static void copyFile(File src, File dest) throws IOException { 
	FileInputStream fis = new FileInputStream(src);
	FileOutputStream fos = new FileOutputStream(dest);
	try {
	    byte[] buffer = new byte[8196];
	    int sz = 0;
	    while ((sz = fis.read(buffer, 0, 8196)) != -1) { 
		fos.write(buffer, 0, sz);
	    }
	} 
	finally {
	    fis.close();
	    fos.close();
	}
    }

    /**
     * Sends a single e-mail message; analogous to ns_sendmail.
     */
    public final static void sendMail(String to, 
				      String from, 
				      String subject,
				      String body) throws Exception { 
	org.apache.turbine.util.mail.Email e = 
	    new org.apache.turbine.util.mail.SimpleEmail();
	e.addTo(to, null);
	e.setFrom(from, null);
	e.setSubject(subject);
	e.setMsg(body);
	e.send();
    }

    /**
     * @return true if the given string looks like a valid email address.
     */
    public final static boolean emailValidP(String email) {
	return re.match("/^[^@<>\"\t ]+@[^@<>\".\t]+([.][^@<>\".\n ]+)+$/", email);
    }
    public final static boolean URLValidP(String queryURL) {
	return re.match ("/https?:\\/\\/./", queryURL);
    }

    public static String generateRandomString() {
	StringBuffer randomStringBuffer = new StringBuffer();
	for (int i=0 ; i<8 ; i+=1) {
	    randomStringBuffer.append(secRandomChar());
	}
	return randomStringBuffer.toString();

    }

    public static String generateRandomString(int u) {
	StringBuffer randomStringBuffer = new StringBuffer(u);
	for (int i=0 ; i<u ; i+=1) {
	    randomStringBuffer.append(secRandomChar());
	}
	return randomStringBuffer.toString();

    }

    /**
     * Return true if the input string looks like a number.
     */
    public final static boolean isNumber(String s) { 
	return re.match("/^-?[0-9]+(\\.[0-9]+)?([Ee][\\+-]?[0-9]+)?$/", s);
    }

    /**
     * This is a utility proc for performing a TCL-style glob
     * (i.e. the "string match" command)
     * via a mostly equivalent regexp command.
     *
     * TCL glob syntax specifies match anything on * and
     * match nothing or one character on ?, unless they are
     * preceeded with a \.
     *
     * There are some other differences between string match and regexps, but this
     * should be enough to account for the most common ones.
     */
    public final static boolean stringGlobMatch(String pattern, String s) {
        pattern = re.substitute("s/[^\\\\]\\*/.*/g", pattern);
        pattern = re.substitute("s/[^\\\\]\\?/.?/g", pattern);
        return re.match("/^"+pattern+"$/", s);
    }

    public final static String exceptionToString(Throwable e) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        e.printStackTrace(pw);
        return sw.toString();
    }

    public final static String fileToString(File f) throws ACSException {
        return fileToString(f, "");
    }

    public final static String fileToString(File f, String exc_message) throws ACSException {
        try {
            FileInputStream fis = new FileInputStream(f);
            StringWriter sw = new StringWriter();
            int a_char = fis.read();
            while ( a_char != -1 ) {
                sw.write(a_char);
                a_char = fis.read();
            }
            fis.close();
            return sw.toString();
        } catch (IOException e) { 
            throw new ACSException(exc_message + e.getMessage());
        }
    }

    public final static void writeStringToResponse(HttpServletResponse resp, String s) throws ACSException {
        writeStringToResponse(resp, s, resp.SC_OK, "text/html");
    }

    public final static void writeStringToResponse(HttpServletResponse resp, String s, int status, String content_type) throws ACSException {
        try {
            resp.setStatus(status);
            resp.setContentType(content_type);
            OutputStream out = resp.getOutputStream();
            PrintWriter pw = new PrintWriter(out);
            pw.print(s);
            pw.flush();
            out.flush();
        } catch (IOException e) { 
            throw new ACSException(e.getMessage());
        }
    }

    public final static boolean linkResponding(String url) {
        return linkResponding(url, new int[] { 404 });
    }

    public final static boolean linkResponding(String url, int bad_codes[]) {
        try {
            URL u = new URL(url);
            URLConnection uc = u.openConnection();
            uc.connect();
            if (uc instanceof HttpURLConnection) {
                HttpURLConnection huc = (HttpURLConnection)(uc);
                int response = huc.getResponseCode();
                huc.disconnect();
                for (int i = 0; i<bad_codes.length; i++) {
                    if (response == bad_codes[i]) {
                        return false;
                    }
                }
                return true;
            }
            return false;
        } catch (Exception e) {
            Log.note("linkResponding exception: " + e.getMessage());
            return false;
        }
        
    }

    /**
     * Returns the contents of a web page as a string.
     * This can be used instead of ns_geturl (although the name was changed
     * slightly to more clearly indicate the purpose).
     *
     * @param url String, including protocol (i.e. http://www.arsdigita.com/doc/index.html)
     * @return String containing the text read.
     * @throws java.io.IOException in the event of an error.
     */
    public final static String getFromURL(String url) throws java.io.IOException {
        URL u = new URL(url);
        URLConnection uc = u.openConnection();
        uc.connect();
        StringBuffer sb = new StringBuffer();
        BufferedReader in = new BufferedReader(new InputStreamReader(uc.getInputStream()));
        // TODO: Figure out if there is a stock way to do this already somewhere,
        // this looks to me like something Tomcat or Turbine should take care of
        // but reading docs and grepping didn't turn anything up. 
        // It may arguably be better to take care of this by defining and
        // registering a content handler, but I'm not sure how that might
        // affect other things in the system.  -DEE 10/31/00
        String line = null;
        do {
            line = in.readLine();
            if ( line == null ) {
                break;
            }
            sb.append(line + System.getProperty("line.separator"));
        } while (line != null);
        in.close();
        return sb.toString();
    }
}


