package com.arsdigita.acs.bboard;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;

public class TopicsDataSource extends Selection {

    private String oldModerationPolicy = "";

    public TopicsDataSource(SqlStatement sql, Database db) throws SQLException {
	super(sql, db);
    }

    public TopicsDataSource(ResultSet rs) throws SQLException {
	super(rs);
    }

    public TopicsDataSource() { } 

    public String getModerationPolicy() throws DataSourceException {
	String moderationPolicy = get("pretty_moderation_policy");
	if (moderationPolicy != null && 
	    !moderationPolicy.equals(oldModerationPolicy)) {
	    oldModerationPolicy = moderationPolicy;
	    return moderationPolicy;
	} else {
	    return null;
	}
    }

    /**
     * Verifies that a user can read messages for this topic.
     * Redirects to registration page if topic is public and user
     * not logged in.  Displays error page if private and user
     * not part of group.
     * @return true if user is authorized, false if not
     */
    public boolean authorizeUser(UserInfo user, PageContext pc) throws Exception {
	try {
	    HttpServletResponse resp = (HttpServletResponse)pc.getResponse();
	    String ra = get("read_access");
	    if (ra == null) {
		// default to private
		ra = "group";
	    }
	    if (ra.equals("any")) {
		return true;
	    } else if (ra.equals("public")) {
		// "public" means user must be logged in to read this topic
		if (user == null) {
		    String returnUrl = "/register?"; 
		    AdSet set = new AdSet();
		    set.put("return_url", 
			    BBoard.urlStub() + "q-and-a?topic_id=" 
			    + get("topic_id"));
		    returnUrl += Utilities.exportURLVars(set);
		    resp.sendRedirect(returnUrl);
		    return false;
		} else {
		    return true;
		}
	    } else if (ra.equals("group") && user != null) {
		// means the user must belong to one of the topic's groups.
		// branimir 2000-02-04
		
		// lars 2000-04-27: first we check the group associated with the topic
		if (user.isGroupMember(get("group_id"))) {
		    return true;
		} 
	    }
	    
	    // Well, the user isn't in any of the topic's groups. But.. 
	    // If they are site admin group member, let's let them in anyway. 
	    // default to group is private (read_access = group)
	    if (user != null && user.isAdministrationGroupMember("site_wide", "")) {
		// user is site admin, let them look at the bboard
		return true;
	    } else {
		pc.getRequest().setAttribute("topic", this);
		pc.forward("/templates/bboard/not-authorized.tpl");
		return false;
	    }
	}
	catch (Exception e) { 
	    throw e;
	}
    } 

    /**
     * Verifies that a user can administer messages for this topic.
     * Redirects to registration page if topic is public and user
     * not logged in.  Displays error page if user is not an authorized
     * administrator.
     * @return true if user is authorized, false if not
     */
    public boolean adminAuthorize(UserInfo user, PageContext pc) {
	try {
	    if (user == null) {
		String returnUrl = "/register?"; 
		AdSet set = new AdSet();
		set.put("return_url", 
			BBoard.urlStub() + "admin-home?topic_id=" 
			+ get("topic_id"));
		returnUrl += Utilities.exportURLVars(set);
		((HttpServletResponse)pc.getResponse()).sendRedirect(returnUrl);
		return false;	    
	}
	    // user is logged in; see if they're a member of this topic
	    // true if user is primary maintainer, or if user_id is
	    boolean result = get("primary_maintainer_id").equals(user.getUserId())
		|| user.isAdministrationGroupMember("bboard", get("topic_id"))
		|| user.isSiteWideAdmin();
	    if (!result) { 
		pc.forward("/templates/bboard/not-authorized-admin.tpl");
		return false;
	    } else
		return true;
	}
	catch (Exception e) {
	    return false;
	}
    } 

    /** 
     * Returns back-link URL for this topic id.  For now, just 
     * q-and-a?topic_id=&topic=
     */
    public String getBackLink() throws DataSourceException { 
	return "q-and-a?" + 
	    Utilities.exportURLVars(this, "topic topic_id");
    }
}

