package com.arsdigita.acs.chat;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;

public class Chat {

    /**
     * @return properties for the given chat room
     * @exception WarningException (with a friendly error message for the user) 
     *            if the room does not exist
     */
    public static ChatRoom getRoomProperties(Database db, Integer roomId) 
        throws ACSException {

        String cacheKey = "chat room " + roomId;
        ChatRoom cr = (ChatRoom)MemoizeCache.lookup(cacheKey);
        if (cr == null) {
            String sql = "select * \n" +
                "from chat_rooms \n" +
                "where chat_room_id = ?";
        
            try {
                ResultSet rs = db.query(sql, new Object[] { roomId });
                if (!rs.next()) {
                    throw new WarningException("Room deleted", "We couldn't find the chat room you tried to enter. It was probably deleted by the site administrator."); 
                }	
                cr = new ChatRoom(rs);
            } catch (Exception e) {
                throw new ACSException(e.getMessage());
            }

            MemoizeCache.cache(cacheKey, cr, Parameters.getInt("RoomPropertiesCacheTimeout", "chat", 600));
        }
        return cr;
    }
    
    /**
     * @return the SystemName parameter
     */
    public static String getSystemName() {
        return Parameters.get("SystemName", "chat", "Chat");
    }

    /**
     * Post message to the chat room marked as a system note.
     */
    public static void postSystemNote(Database db, String msg, Integer userId, 
                                      Integer roomId, String remoteAddr) 
        throws ACSException {

        post(db, msg, userId, roomId, remoteAddr, "t", "t", null);
    }

    /**
     * Post a normal message to the chat room.
     */
    public static void postMessage(Database db, String msg, Integer userId, 
                                   Integer roomId, String remoteAddr) 
        throws ACSException {

        SingleSelection roomInfo = getRoomProperties(db, roomId);
        boolean approved;
        try {
            if (roomInfo.get("moderated_p").equals("t")) {
                // If one of the moderators posts a message, we immediately approve it.
                approved = GeneralPermissions.userHasPermission(userId, "admin", roomId.toString(), "chat_rooms");
            } else {
                approved = true;
            }
        } catch (DataSourceException dse) {
            throw new ACSException(dse.getMessage());
        }
        post(db, msg, userId, roomId, remoteAddr, approved ? "t" : "f", "f", null);
    }

    /**
     * Post a normal message to the chat room.
     */
    public static void postPersonalMessage(Database db, String msg, Integer userId, 
                                           Integer recipientId, String remoteAddr) 
        throws ACSException {

        post(db, msg, userId, null, remoteAddr, null, null, recipientId);
    }

    private static void post(Database db, String msg, Integer userId, Integer roomId, 
                             String remoteAddr, String approved, String system, Integer recipientId) 
        throws ACSException {

        // NYI: check msg for naughty html
        // NYI: content tagging (msg_bowdlerized and content_tag fields in chat_msgs table)

        if (Utilities.emptyString(msg)) {
            return;
        }

        String sql = "insert into chat_msgs \n" +
            "(chat_msg_id, msg, creation_date, creation_user, \n" +
            " creation_ip_address, chat_room_id, approved_p, \n" +
            " system_note_p, recipient_user) \n" +
            "values \n" +
            "(chat_msg_id_sequence.nextval, ?, sysdate, ?, ?, ?, ?, ?, ?)";

        SqlStatement sqlInsert = new SqlStatement();
        sqlInsert.setSql(sql);
        sqlInsert.bind(msg);
        sqlInsert.bind(userId);
        sqlInsert.bind(remoteAddr);
        sqlInsert.bind(roomId);
        sqlInsert.bind(approved);
        sqlInsert.bind(system);
        sqlInsert.bind(recipientId);

        try {
            db.dml(sqlInsert);
        } catch (SQLException sqlex) {
            throw new ACSException(sqlex.getMessage());
        }

        // invalidate cached pages

        MemoizeCache.flush("chat_entire_page " + roomId + " short");
        MemoizeCache.flush("chat_entire_page " + roomId + " medium");
        MemoizeCache.flush("chat_entire_page " + roomId + " long");
        MemoizeCache.flush("chat_entire_page " + roomId + " short moderator");
        MemoizeCache.flush("chat_entire_page " + roomId + " medium moderator");
        MemoizeCache.flush("chat_entire_page " + roomId + " long moderator");
        MemoizeCache.flush("chat_js_entire_page " + roomId);
    }


    /**
     * Returns chat_msg_id of most recent post in a room; used by JavaScript client to 
     * figure out whether an update to the main window is needed
     *
     * 5/28/2000
     * mbryzek added a unique index chat_msgs_room_approved_id_idx 
     * on chat_msgs(chat_room_id, approved_p, chat_msg_id)
     * to avoid hitting the chat_msgs table at all when we're calling
     * chat_last_post. (this is important because the JavaScript 
     * client calls this like crazy)
     *
     *  Execution Plan
     *  ----------------------------------------------------------
     *     0	  SELECT STATEMENT Optimizer=CHOOSE
     *     1	0   SORT (AGGREGATE)
     *     2	1     INDEX (RANGE SCAN) OF 'CHAT_MSGS_ROOM_APPROVED_ID_IDX' (
     *  	  UNIQUE)
     */
    public static Integer lastPost(Database db, Integer roomId) 
        throws ACSException, SQLException {
        // NYI: this should really use a prebuilt PreparedStatement.
        String query = "select max(chat_msg_id) from chat_msgs where chat_room_id = :roomId and approved_p = 't'";
        return new Integer(db.queryToString(query, new Object[] { roomId }, ""));
    }

    /**
     * @return Integer array: first element is the time of the last post between
     * currently connected user and argument-specified user.  second element
     * is the user id that wrote the post.
     */
    public static Integer[] lastPersonalPost(Database db, Integer userId, Integer chatterId) 
        throws ACSException, SQLException {
        Integer time=null, creationUser=null;
        String sql = "select to_char(creation_date, 'HH24MISS') as time, creation_user " +
            "from chat_msgs " +
            "where (creation_user = ? and recipient_user = ?) " +
            "or (creation_user = ? and recipient_user = ?) " +
            "order by creation_date desc";

        ResultSet rs = db.query(sql, new Object[] {userId, chatterId, chatterId, userId});
        if (rs.next()) {
            time = new Integer(rs.getInt(1));
            creationUser = new Integer(rs.getInt(2));
            rs.close();
        }
        return new Integer[] { time, creationUser };
    }
}
