package com.arsdigita.acs.db;

import java.sql.*;
import javax.sql.*;
import oracle.jdbc.driver.*;
import oracle.jdbc.pool.*;
import oracle.sql.CLOB;
import oracle.sql.BLOB;
import com.arsdigita.acs.ACSException;
import com.arsdigita.acs.Parameters;
import java.io.*;
import java.util.Vector;
import java.util.Hashtable;
import javax.servlet.*;

import com.oroinc.text.perl.*;

import org.apache.turbine.util.Log;

/**
 * Wrapper class for Oracle JDBC.  Opens database connections, manages
 * connection pools, handles queries, and provides utility methods for
 * large-object (BLOB/CLOB) operations.  
 *
 * @author Jin Choi (jsc@photonet.arsdigita.com)
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @version $Version$
 */

public class Database {
    static private OracleConnectionCache ods;
    
    static {
	try {
	    OracleConnectionPoolDataSource ocpds = new OracleConnectionPoolDataSource();
	    ocpds.setURL(Parameters.get("ConnectionURL", "db",
					"jdbc:oracle:oci8:@"));
	    ocpds.setUser(Parameters.get("dbuser", "db"));
	    ocpds.setPassword(Parameters.get("dbpassword", "db"));
	    
	    ods = new OracleConnectionCacheImpl(ocpds);
	} catch (SQLException e) {
	    System.err.println("SQLException caught while attempting to open connection: " + e.getMessage());
	    System.exit(1);
	}
    }

    private Connection conn;

    // keep a count of how deep we are so we can have nested
    // transactions.
    //    private int transactionDepth = 0;
    
    //    private boolean transactionAborted = false;
    private boolean inTransaction = false;

    /** 
     * Returns a database handle for an open database connection from
     * the connection pool.
     */
    public Database() throws SQLException {
	conn = ods.getConnection();
    }

    /** 
     * Returns a database handle for an open database connection from
     * the connection pool.  Stashes away a servlet context for logging.
     */
    public Database(ServletContext sc) throws SQLException {
	conn = ods.getConnection();
    }
    
    /**
     * Returns this database handle to the connection pool.
     */
    public void close() throws SQLException {
        if (inTransaction()) {
            Log.error("Database object is being closed while in transaction; this probably shouldn't happen.  Committing anyway.");
	    conn.commit();
	}

	conn.close();
	conn = null;
    }

    /**
     * Returns true if in a transaction.
     */
    public boolean inTransaction() throws SQLException {
	return inTransaction;
    }
	

    /**
     * Starts a transaction by turning off auto-commit mode.
     */
    public void beginTransaction() throws SQLException {
	if (!inTransaction) {
	    // top-level transaction
	    conn.setAutoCommit(false);
	    // transactionDepth = 1;
	    // transactionAborted = false;
	    inTransaction = true;
	}
	// otherwise we're already in a transaction; just increment
	// the counter
	else {
	    // transactionDepth++;
	    throw new SQLException("Nested transactions not allowed.");
	}
    }

    /**
     * Completes a transaction by turning on auto-commit mode.  Commits
     * the transaction.
     */
    public void endTransaction() throws SQLException {
	//	transactionDepth--;
	//	if (transactionDepth <= 0) {
	//	    if (!transactionAborted) { 
	//		conn.commit();
	//	    } else {
	//		conn.rollback();
	//}
	// conn.setAutoCommit(true);
	//} 

	if (inTransaction) { 
	    conn.commit();
	    conn.setAutoCommit(true);
	    inTransaction  = false;
	} else {
	    throw new SQLException("Can't end a transaction unless we're in one!");
	}
    }

    /**
     * Rolls back and aborts a transaction.  turns auto-commit back on.
     */
    public void abortTransaction() throws SQLException {
	//	transactionDepth--;
	//	transactionAborted = true;
	// it's ok if we rollback more than once..

	conn.rollback();
	conn.setAutoCommit(true);
	inTransaction = false;
    }

    /**
     * Called by the garbage collector at some point after <code>this</code>
     * goes out of scope.  Makes sure that the connection is returned
     * to the pool.
     */
    protected void finalize() throws Throwable {
	if (conn != null) {
	    conn.close();
	    conn = null;
	}
    }

    /** 
     * Returns the result of a query.
     * @param sql An object consisting of the SQL query and the 
     * bind values
     * @return A <code>ResultSet</code> with the query results.
     */
    public ResultSet query(SqlStatement sql) throws SQLException {
	return query(sql.getSql(), sql.getBindValues());
    }

    /** 
     * Returns the result of a query.
     * @param sql A SQL query
     * @param bind An array of bind values
     * @return A <code>ResultSet</code> with the query results.
     */
    public ResultSet query(String sql, Object[] bind) throws SQLException {
        try {
            if (bind.length == 0) {
                return query(sql);
            }
            PreparedStatement stmt = conn.prepareStatement(sql);
            bindParameters(stmt, bind);
            Log.note("db: query: " + sql);
            Log.note("db: bind values: " + bindsToString(bind));
            return stmt.executeQuery();
        } catch (SQLException sqle) {
            throw new SQLExceptionWrapper(sqle, sql, bind);
        }
    }

    /**
     * Returns the result of a query with no bind variables.
     * @param sql A SQL query
     * @return A <code>ResultSet</code> with the query results.
     */
    public ResultSet query(String sql) throws SQLException {
        try {
            Statement stmt = conn.createStatement();
            Log.note("db: query: " + sql);
            return stmt.executeQuery(sql);
        } catch (SQLException sqle) {
            throw new SQLExceptionWrapper(sqle, sql);
        }
    }

    /**
     * Executes a DML statement (insert, update, delete) with no 
     * bind variables.
     * @param sql A SQL query
     * @return The number of rows affected
     */
    public int dml(String sql) throws SQLException {
        try {
            Statement stmt = conn.createStatement();
            Log.note("db: dml: " + sql);
            return stmt.executeUpdate(sql);
        } catch (SQLException sqle) {
            throw new SQLExceptionWrapper(sqle, sql);
        }
    }

    /**
     * Executes a DML statement (insert, update, delete). 
     * @param sql A SQL query
     * @param bind A list of bind values
     * @return The number of rows affected
     */
    public int dml(String sql, Object[] bind) throws SQLException {
        try {
            if (bind.length == 0) {
                return dml(sql);
            }
            PreparedStatement stmt = conn.prepareStatement(sql);
            bindParameters(stmt, bind);
            Log.note("db: dml: " + sql);
            Log.note("db: bind values: " + bindsToString(bind));
            return stmt.executeUpdate();
        } catch (SQLException sqle) {
            throw new SQLExceptionWrapper(sqle, sql, bind);
        }
    }

    /**
     * Executes a DML statement (insert, update, delete). 
     * @param sql An object containing a SQL query and a list of bind
     * values
     * @return The number of rows affected
     */
    public int dml(SqlStatement sql) throws SQLException {
	return dml(sql.getSql(), sql.getBindValues());
    }


    /**
     * Calls a SQL stored function, or other PL/SQL code, whose first bind
     * variable may be is an OUT parameter returning the value of the
     * stored procedure/function.
     * 
     * <p>
     * 
     * The SQL statement will generally be of the form 
     * <code>begin ? := stored_function(?, ?, ?); end;</code> 
     * or <code>begin stored_proc(?, ?) </code>.
     * The value returned from <code>stored_function</code> will be
     * returned, or null will be returned for procedures.
     *
     * <p> 
     * 
     * Note that a stored function with no side effects
     * (inserts/updates/deletes) may be <code>SELECT</code>ed from
     * <code>dual</code> and called as a normal query, and a stored
     * procedure with no side effects may be executed by the dml()
     * method.  Also, DDL can often be executed by the dml() method as
     * well.
     * 
     * @param sql An object containing the SQL code
     * @param bind A list of bind values
     * @param outType The type of the OUT parameter (or Types.NULL if none)
     * @return The object returned by the PL/SQL code, or null if none
     * 
     * @see java.sql.CallableStatement
     * @see java.sql.Types */
    public Object plsql(String sql, Object[] bind, int outType) throws SQLException {
        try {

            CallableStatement stmt;
            stmt = conn.prepareCall(stripNewLines(sql));
            if (outType != Types.NULL) {
                // there's an OUT parameter, so register it 
                // in the first position
                stmt.registerOutParameter(1, outType);
                bindParameters(stmt, bind, 1);
            } else {
                bindParameters(stmt, bind, 0);
            }
	
            Log.note("db: plsql: " + sql);
            Log.note("db: bind values: " + bindsToString(bind));

            stmt.executeUpdate();
            if (outType != Types.NULL) 
                return stmt.getObject(1);
            else 
                return null;
        } catch (SQLException sqle) {
            throw new SQLExceptionWrapper(sqle, sql, bind);
        }
    }

    /**
     * Calls a SQL stored procedure, stored function, or other PL/SQL
     * code with IN parameters but no OUT parameters.
     *
     * <p> 
     * 
     * Note that a stored function with no side effects
     * (inserts/updates/deletes) may be <code>SELECT</code>ed from
     * <code>dual</code> and called as a normal query, and a stored
     * procedure with no side effects may be executed as DML.  
     * 
     * @param sql An object containing the SQL code
     * @param bind A list of bind values
     * @return The number of rows affected
     * @see java.sql.CallableStatement
     * @see java.sql.Types */
    public Object plsql(String sql, Object[] bind) throws SQLException {
	return plsql(sql, bind, java.sql.Types.NULL);
    }

    /**
     * Alternate form; calls plsql(stmt.getSql(), stmt.getBindValues()).
     */
    public Object plsql(SqlStatement stmt) throws SQLException { 
	return plsql(stmt.getSql(), stmt.getBindValues());
    }

    /**
     * Alternate form; calls plsql(stmt.getSql(), stmt.getBindValues(), outType).
     */
    public Object plsql(SqlStatement stmt, int outType) throws SQLException { 
	return plsql(stmt.getSql(), stmt.getBindValues(), outType);
    }

    /** 
     * Utility function for writing files to large objects in the database.
     * 
     * @param sql SQL insert or update, with the BLOB locator
     * returned as an OUT parameter, as the final bind variable (e.g.,
     * <code> ... returning blob_column into ?</code>) 
     * 
     * <p>
     * 
     * <b>Note:</b> Note that the SQL statement must be wrapped in a 
     * begin ... end block.  This is a kludge; issuing a bare insert/update
     * results in a "not enough bind values" error from Oracle, and 
     * trying to make the locator an IN/OUT parameter with setNull or
     * setBLOB results in weird errors in the database driver.
     * 
     * @param bind List of bind vals
     * @param f The file to write
     * @param blob_p If <code>true</code>, treat the returned locator as a
     * BLOB; otherwise treat it as a CLOB.
     * @return the number of rows affected
     */
    private int lobDmlFile(String sql, Object[] bind, File f, boolean blob_p) throws SQLException, IOException {
        try {
            // for some reason, Oracle doesn't like PL/SQL blocks in callable
            // statements to contain newlines.. probably a bug in the db 
            // driver.  For now, work around

            CallableStatement stmt = conn.prepareCall(stripNewLines(sql));
            bindParameters(stmt, bind);
            int outType; 
            if (blob_p) {
                outType = java.sql.Types.BLOB;
            }
            else {
                outType = java.sql.Types.CLOB;
            }
            stmt.registerOutParameter(bind.length + 1, outType);
            boolean autoCommit = conn.getAutoCommit();
            // LOB locators can't span transactions
            if (autoCommit) {
                conn.setAutoCommit(false);
            }

            Log.note("db: lobDmlFile: " + sql);

            int rowcount = stmt.executeUpdate();
            // we expect the caller specified "returning .column. into ?"
            if (blob_p) {
                BLOB blob = (BLOB)stmt.getObject(bind.length + 1);
                writeBLOB(blob, f);
            } else {
                CLOB clob = (CLOB)stmt.getObject(bind.length + 1);
                writeCLOB(clob, f);
            }
            if (autoCommit) {
                conn.commit();
                conn.setAutoCommit(true);
            }
            return rowcount;
        } catch (SQLException sqle) {
            throw new SQLExceptionWrapper(sqle, sql, bind);
        }

    }

    /** 
     * Utility function for a large string literal to a CLOB field
     * 
     * @param sql SQL insert or update, with the CLOB locator
     * returned as an OUT parameter, as the final bind variable (e.g.,
     * <code> ... returning clob_column into ?</code>) 
     * 
     * <p>
     * 
     * <b>Note:</b> Note that the SQL statement must be wrapped in a 
     * begin ... end block.  This is a kludge; issuing a bare insert/update
     * results in a "not enough bind values" error from Oracle, and 
     * trying to make the locator an IN/OUT parameter with setNull or
     * setCLOB results in weird errors in the database driver.
     * 
     * @param sqlst The sql statement with bind vars
     * @param s The string to write
     * @return the number of rows affected
     */
    public int clobDml(SqlStatement sqlst, String s) throws SQLException, IOException {
        // for some reason, Oracle doesn't like PL/SQL blocks in callable
        // statements to contain newlines.. probably a bug in the db 
        // driver.  For now, work around
        String sql = sqlst.getSql();
        Object[] bind = sqlst.getBindValues();

        try {
            // for some reason, Oracle doesn't like PL/SQL blocks in callable
            CallableStatement stmt = conn.prepareCall(stripNewLines(sql));
            bindParameters(stmt, bind);
            int outType  = java.sql.Types.CLOB;
            stmt.registerOutParameter(bind.length + 1, outType);
            boolean autoCommit = conn.getAutoCommit();
            // LOB locators can't span transactions
            if (autoCommit) {
                conn.setAutoCommit(false);
            }
            Log.note("db: clobDml: " + sql);
            Log.note("db: bind values: " + bindsToString(bind));

            int rowcount = stmt.executeUpdate();
            // we expect the caller specified "returning .column. into ?"
            CLOB clob = (CLOB)stmt.getObject(bind.length + 1);
            Writer w = clob.getCharacterOutputStream();
            w.write(s);
            w.close();
            if (autoCommit) {
                conn.commit();
                conn.setAutoCommit(true);
            }
            return rowcount;
        } catch (SQLException sqle) {
            throw new SQLExceptionWrapper(sqle, sql, bind);
        }
    }

    /** 
     * Utility function for writing a file to a BLOB in the database.
     * 
     * @param sql SQL insert or update, with the BLOB locator
     * returned as an OUT parameter, as the final bind variable (e.g.,
     * <code> ... returning blob_column into ?</code>) 
     * 
     * <p>
     * 
     * <b>Note:</b> Note that the SQL statement must be wrapped in a 
     * begin ... end block.  This is a kludge; issuing a bare insert/update
     * results in a "not enough bind values" error from Oracle, and 
     * trying to make the locator an IN/OUT parameter with setNull or
     * setBLOB results in weird errors in the database driver.
     * 
     * @param bind List of bind vals
     * @param f The file to write
     * @return the number of rows affected
     */
    public int blobDmlFile(String sql, Object[] bind, File f) throws SQLException, IOException {
	return lobDmlFile(sql, bind, f, true);
    }

    /** 
     * Utility function for writing a file to a BLOB in the database.
     * 
     * @param sql Object including SQL insert or update and bind values,
     * with the BLOB locator returned as an OUT parameter, as the final
     * bind variable (e.g., <code> ... returning blob_column into ?</code>).  
     * 
     * <p>
     * 
     * <b>Note:</b> Note that the SQL statement must be wrapped in a 
     * begin ... end block.  This is a kludge; issuing a bare insert/update
     * results in a "not enough bind values" error from Oracle, and 
     * trying to make the locator an IN/OUT parameter with setNull or
     * setBLOB results in weird errors in the database driver.
     * 
     * @param f The file to write
     * @return the number of rows affected
     */
    public int blobDmlFile(SqlStatement sql, File f) throws SQLException, IOException {
	return lobDmlFile(sql.getSql(), sql.getBindValues(), f, true);
    }

    /** 
     * Utility function for writing a file to a CLOB in the database.
     * 
     * @param sql SQL insert or update, with the CLOB locator
     * returned as an OUT parameter, as the final bind variable (e.g.,
     * <code> ... returning blob_column into ?</code>) 
     * 
     * <p>
     * 
     * <b>Note:</b> Note that the SQL statement must be wrapped in a 
     * begin ... end block.  This is a kludge; issuing a bare insert/update
     * results in a "not enough bind values" error from Oracle, and 
     * trying to make the locator an IN/OUT parameter with setNull or
     * setCLOB results in weird errors in the database driver.
     * 
     * @param bind List of bind vals
     * @param f The file to write
     * @return the number of rows affected
     */
    public int clobDmlFile(String sql, Object[] bind, File f) throws SQLException, IOException {
	return lobDmlFile(sql, bind, f, false);
    }

    /**
     * Returns a string result from a single-row, single-column SQL
     * query.
     * @param sql SQL to run
     * @param defaultValue Default if the query returns no rows
     * @return The result of the SQL query, or <code>default</code>
     * if the query returns no rows.
     */
    public String queryToString(String sql, String defaultValue) throws SQLException, ACSException {
	return queryToString(sql, null, defaultValue);
    }

    /**
     * Returns a string result from a single-row, single-column SQL
     * query.  Throws ACSException if no rows are returned.
     * @param sql SQL to run
     * @return The result of the SQL query.
     * @exception com.arsdigita.acs.ACSException 
     */
    public String queryToString(String sql) throws SQLException, ACSException {
	return queryToString(sql, null, null);
    }

    /**
     * Returns a string result from a single-row, single-column SQL
     * query.  Throws ACSException if no rows are returned.
     * @param sql SQL to run
     * @param bind List of bind values
     * @return The result of the SQL query.
     * @exception com.arsdigita.acs.ACSException 
     */
    public String queryToString(String sql, Object[] bind) throws SQLException, ACSException {
	return queryToString(sql, bind, null);
    }

    /**
     * Returns a string result from a single-row, single-column SQL
     * query.  Throws ACSException if no rows are returned.
     * @param st Object with SQL statement and bind values
     * @return The result of the SQL query.
     * @exception com.arsdigita.acs.ACSException 
     */
    public String queryToString(SqlStatement st) throws SQLException, ACSException {
	return queryToString(st.getSql(), st.getBindValues());
    }

    /**
     * Returns a string result from a single-row, single-column SQL
     * query.  Returns defaultValue if no rows are returned.
     * @param st Object with SQL statement and bind values
     * @param defaultValue Value to return if query returns no rows
     * @return The result of the SQL query (or <code>defaultValue</code> if no rows returned)
     * @exception com.arsdigita.acs.ACSException 
     */
    public String queryToString(SqlStatement st, String defaultValue) throws SQLException, ACSException {
	return queryToString(st.getSql(), st.getBindValues(), defaultValue);
    }

    /**
     * Returns a string result from a single-row, single-column SQL
     * query.  Throws ACSException if no rows are returned and no default
     * value is supplied.
     * @param sql SQL to run
     * @param bind List of bind values
     * @param defaultValue Default if the query returns no rows
     * @return The result of the SQL query, or <code>default</code>
     * if the query returns no rows.
     */
    public String queryToString(String sql, Object[] bind, String defaultValue) throws SQLException, ACSException {
	ResultSet rs;
	if (bind != null) {
	    rs = query(sql, bind);
	} else {
	    rs = query(sql);
	}
	if (! rs.next()) {
	    rs.close();
	    if (defaultValue == null) {
		throw new ACSException("Database.queryToString("+sql+", "+bindsToString(bind)+") failed to retrieve a row and no default value was specified");
	    }
	    return defaultValue;
	}
	String result = rs.getString(1);
	rs.close();
	return result;
    }

    /**
     * Returns a list of Strings resulting from a single-column SQL
     * query.  
     * @param sql SQL to run
     * @param bind List of bind values
     * @return A list of values from the SQL query.
     */
    public Vector queryToList(String sql, Object[] bind) throws SQLException {
	ResultSet rs;
	if (bind != null) {
	    rs = query(sql, bind);
	} else {
	    rs = query(sql);
	}
	Vector v = new Vector();
	while (rs.next()) {
	    String result = rs.getString(1);
	    v.addElement(result);
	}
	rs.close();
	return v;
    }

    /**
     * Returns a list of Strings resulting from a single-column SQL
     * query.  
     * @param stmt An object containing a SQL statment and a
     * list of bind values
     * @return A list of values from the SQL query.
     */
    public Vector queryToList(SqlStatement stmt) throws SQLException {
	return queryToList(stmt.getSql(), stmt.getBindValues());
    }

    /** 
     * Utility function to set bind values in a PreparedStatement.
     * @param stmt the PreparedStatement to bind
     * @param bind The list of bind values
     * @param offset Offset of position to start binding, "skipping over"
     * the preceding ones (to allow ":1 := ...").
     * @bind The list of bind values
     */
    private void bindParameters(PreparedStatement stmt, Object[] bind, 
				int offset)
	throws SQLException {
	for (int i = 0; i < bind.length; i++) {
	    if (bind[i] == null) {
		stmt.setNull(i + 1 + offset, java.sql.Types.INTEGER);
	    } else {
		stmt.setObject(i + 1 + offset, bind[i]);
	    }
	}
    }

    /** 
     * Utility function to set bind values in a PreparedStatement.
     * @param stmt the PreparedStatement to bind
     * @param bind The list of bind values
     * @bind The list of bind values
     */
    private void bindParameters(PreparedStatement stmt, Object[] bind) 
	throws SQLException {
	bindParameters(stmt, bind, 0);
    }
	
    /** 
     * Returns a CLOB from a ResultSet as a String.  
     * @param ors The result set
     * @param name The name of the CLOB column 
     * @return The contents of the CLOB column
     */
    public static String getCLOB(ResultSet ors, String name) 
    throws SQLException, IOException {
	CLOB clob = (CLOB)ors.getObject(name);
	if (clob == null) {
	    // TO DO: log error?
	    return "";
	}
	Reader is = clob.getCharacterStream();
	char buffer[] = new char[4000];
	int length = 0;
	StringBuffer sb = new StringBuffer();
	while ((length = is.read(buffer)) != -1)
	    sb.append(buffer,0,length);
	return sb.toString();
    }

    /** 
     * Returns a BLOB from a ResultSet as a byte array.
     * @param ors The result set
     * @param name The name of the BLOB column 
     * @return The contents of the BLOB column
     */
    public static byte[] getBLOB(ResultSet ors, String name)
	throws SQLException, IOException {
	BLOB blob = (BLOB)ors.getObject(name);
	if (blob == null) {
	    // TO DO: log error
	    return new byte[0];
	}
	InputStream is = blob.getBinaryStream();
	int size = (int)blob.length();
	byte buffer[] = new byte[size];
	int length = 0;
	int pos = 0;
	while ((length = is.read(buffer, pos, size-pos)) != -1) 
	    pos += length;
	is.close();
	return buffer;
    }

    /** 
     * Writes a BLOB to the database from a file.
     * @param rs The result set
     * @param name The name of the BLOB column 
     * @param f The file to fill the database BLOB with
     */
    public static void writeBLOB(ResultSet rs, String name, File f) 
	throws SQLException, IOException {
	// reads a blob from a file
	BLOB blob = (BLOB)rs.getObject(name);
	writeBLOB(blob, f);
    }

    /** 
     * Writes a BLOB to the database from a file.
     * @param blob The BLOB locator 
     * @param f The file to fill the database BLOB with
     */
    public static void writeBLOB(BLOB blob, File f) 
	throws SQLException, IOException {
	// reads a blob from a file
	FileInputStream fis = new FileInputStream(f);
	OutputStream os = blob.getBinaryOutputStream();
	byte b[] = new byte[4000];
	int ct = 0;
	while ((ct = fis.read(b,0,4000)) != -1)
	    os.write(b,0,ct);
	os.close();
	fis.close();
    }

    /** 
     * Writes a CLOB to the database from a file.
     * @param blob The CLOB locator 
     * @param f The file to fill the database CLOB with
     */
    public static void writeCLOB(CLOB clob, File f) 
	throws SQLException, IOException {	
	// reads a clob from a file
	FileInputStream fis = new FileInputStream(f);
	InputStreamReader r = new InputStreamReader(fis);
	Writer w = clob.getCharacterOutputStream();
	char c[] = new char[4000];
	int ct = 0;
	while ((ct = r.read(c,0,4000)) != -1)
	    w.write(c,0,ct);
	w.close();
	fis.close();
    }

    /** 
     * Dumps a BLOB's contents to an OutputStream.
     * @param rs The result set containing a BLOB locator
     * @param name The name of the BLOB column
     * @param os The output stream to write the BLOB contents to
     */
    public static void dumpBLOB(ResultSet rs, String name, OutputStream os)
	throws SQLException, IOException {
	BLOB blob = (BLOB)rs.getObject(name);
	if (blob == null) {
	    // TO DO: log error
	    return;
	}
	InputStream is = blob.getBinaryStream();
	byte buffer [] = new byte[4000];
	int length = 0;
	while ((length = is.read(buffer)) != -1) 
	    os.write(buffer,0,length);
	is.close();
    }

    /** 
     * Utility function returning an empty string if an input object
     * is null.  Used primarily for binding objects from a JSP tag,
     * since Object.toString has this nasty habit of putting the
     * string literal "null" in place for NULL object.  This behavior
     * wreaks havoc when trying to put a null integer into the database.
     * 
     * @param obj The object that for binding
     * @return The string representation of obj, or an empty string if null
     */
    public static String bindObj(Object obj) {
	// utility to return empty string for null objects instead
	// of literal word "NULL"
	return (obj == null) ? "" : obj.toString();
    }
	    
    public static void main(String[] args) {
	try {
	    Database db = new Database();
	    ResultSet rs = db.query("select sysdate from dual");
	    while (rs.next()) {
		System.out.println(rs.getString(1));
	    }
	    rs.close();
	    
	    Object[] bind = new Object[1];
	    bind[0] = new Integer(21);
	    rs = db.query("select first_names || ' ' || last_name from users where user_id = ?", bind);
	    while (rs.next()) {
		System.out.println(rs.getString(1));
	    }

	    rs.close();
	    db.close();
	} catch (SQLException e) {
	    System.err.println("SQLException caught: " + e.getMessage());
	}
    }

    /** 
     * Returns <code>true</code> if <code>rs</code> contains 
     * a column named <code>column</code> with a non-null value.
     */
    public static boolean contains(ResultSet rs, String column) {
	try {
	    ResultSetMetaData rsmd = rs.getMetaData();
	    for (int i = 1; i <= rsmd.getColumnCount(); i++) {
		if (column.equalsIgnoreCase(rsmd.getColumnLabel(i))) {
		    // see if the value is not null
		    return (rs.getObject(i) != null);
		}
	    }
	    return false;
	} catch(SQLException e) {
	    return false;
	}
    }

    /**
     * Strips the CR and LF characters out of a query string; useful for
     * PL/SQL calls for which Oracle complains.
     */
    private String stripNewLines(String sql) { 
	StringBuffer hackedSql = new StringBuffer(); 
	for (int i = 0; i < sql.length(); i++) {
	    char c = sql.charAt(i);
	    if (c == '\n' || c == '\r') 
		// We need the space in case the newline was the only whitespace
		hackedSql.append(' ');
	    else 
		hackedSql.append(c);
	}
	return hackedSql.toString();
    }

    static private String bindsToString(Object[] bind) {
        StringBuffer bindstr = new StringBuffer("[");
        for (int i = 0; i < bind.length; i++) {
            bindstr.append(bind[i]);
            if (i < bind.length - 1) 
                bindstr.append(" ");
        }
        bindstr.append("]");
        return bindstr.toString();
    }
}
