package com.arsdigita.acs.db;

import java.util.*;
import org.apache.turbine.util.Log;

/** 
 * Utility class for bundling a SQL statement with its bind values.
 * 
 * @author Jin Choi (jsc@photonet.arsdigita.com)
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @author David Eison (eison@arsdigita.com)
 * @version $Version$
 */
public class SqlStatement {
    private StringBuffer sql = new StringBuffer();
    private Vector bind_vars = new Vector();
    /**
     * Support additional bind values by name, to be inserted
     * after unnamed (which go in order)
     */
    private HashMap extraBindValues = new HashMap();

    /**
     * Creates a new SqlStatement with the SQL set to the input string.
     */
    public SqlStatement(String s) { 
	sql = new StringBuffer(s);
    }

    public SqlStatement() { 
    }
    
    /** 
     * Clears this statement.
     */
    public void clear() {
	bind_vars.clear();
	sql = new StringBuffer();
    }

    /** 
     * @return the SQL command string.
     */
    public String getSql() {
        bindFixup();
	return sql.toString();
    }

    /** 
     * @return True if this statement has an associated SQL command; false
     * otherwise.
     */
    public boolean isSet() {
	return sql.length() > 0;
    }

    /** 
     * Sets the SQL command string for this statement.
     * @param s The SQL command
     */
    public void setSql(String s) {
	sql = new StringBuffer(s);
    }

    /** 
     * Adds to the SQL command string for this statement.
     * Does not adjust the bind variables.
     * @param s The SQL command.
     * @return This SqlStatement (useful for chaining multiple appends on one line).
     */
    public SqlStatement append(String s) {
        sql.append(s);
        return this;
    }

    /**
     * Allows combining of SQL statements, including their bind variables.
     * Useful for building up fragments of SQL statements, such as a 
     * date (<code>"select to_date(:the_date, 'YYYY-MM-DD HH24:MI:SS')"</code> and the value
     * of <code>:the_date</code>) and then inserting them into a larger SQL statement
     * (<code>stmnt.append("SELECT ").append(dateStmnt).append(" FROM dual")</code>).
     * <P>
     * Note that bind values from the appended statement are not bound with other variables in
     * this statement.  So, extending the above example:
     * (<code>stmnt.append("SELECT ").append(dateStmnt).append(", :the_date").append(" FROM dual")</code>).
     * ":the_date" inside of dateStmnt will be bound in this statement, but the ":the_date" which was 
     * appended afterwards is not presently bound.
     * <code>stmnt.bind("the_date", someValue);</code> is necessary to bind this second "the_date".
     *
     * @param SqlStatement to append.
     * @return This SqlStatement (useful for chaning multiple appends on one line).
     */
    public SqlStatement append(SqlStatement s) {
        // We don't want to deal with "extra bind values", get 'em translated.
        s.bindFixup();

        // Intentionally bypassing the accessors, we want the underlying Vector
        bind_vars.addAll(s.bind_vars);

        // Again, skipping the accessor, just grab the stringbuffer.
        sql.append(s.sql);
        
        // NOTE that we are not bringing over the extra bind values from the appended SqlStatement...
        // We are only brining over the bind values that were actually used.  I think this 
        // will result in the most intuitive behavior and avoid potential confusion over name
        // collisions, but we may want to rethink this later.

        return this;
    }
    
    /**
     * Adds a bind value to this statement.
     * @param o The object to bind
     * @deprecated 
     */
    public void addBindValue(Object o) {
	bind_vars.addElement(o);
    }

    /**
     * Adds a bind value to this statement.
     * @param o The object to bind
     */
    public void bind(Object o) {
	bind_vars.addElement(o);
    }

    /**
     * Adds a named bind value to this statement.
     * @param name The name of the bind variable
     * @param o The object to bind
     * @deprecated
     */
    public void addBindValue(String name, Object o) { 
	extraBindValues.put (name.toLowerCase(), o);
    }

    /**
     * Adds a named bind value to this statement.
     * @param name The name of the bind variable
     * @param o The object to bind
     */
    public void bind(String name, Object o) { 
	extraBindValues.put (name.toLowerCase(), o);
    }

    /**
     * This is called by the end of a SqlTag (e.g., db_query)
     * to tell us that we're done with bind variables, and that
     * we should go put the extra (named) ones in their places.
     * Goes through the SQL code looking for bind variables (? or :var).
     */
     public void bindFixup() {
         //Log.note("BindFixup on:" + toString());
	 int ctr = 0;
	 boolean inStringLiteral = false;
	 for (int i = 0; i < sql.length(); i++) {
	     if (sql.charAt(i) == '\'') {
		 // toggle string literal
		 inStringLiteral = !inStringLiteral;
		 continue;
	     }
	     
	     if (inStringLiteral) 
		 continue;

	     if (sql.charAt(i) == '?') {
		 // make sure it's not "? := ..." 
		 int j = i + 1;
		 while (j < sql.length() 
			&& Character.isWhitespace(sql.charAt(j))) {
		     j++;
                 }
		 if (j+1 < sql.length() && !(sql.charAt(j)==':' && sql.charAt(j+1)=='='))
		     ctr++;
	     } else if (sql.charAt(i) == ':') {
		 // need to let ":=" operator slip by
		 if (i+1 < sql.length() && sql.charAt(i+1) == '=')
		     continue;

		 // find first occurrence of non alpha-numeric char
		 for (int j = i + 1; j <= sql.length(); j++) { 
                     // Fix for :varname at end of SQL string
                     char ch = ' ';
                     if (j < sql.length()) {
                         ch = sql.charAt(j);
                     }
		     if (!Character.isLetterOrDigit(ch) && ("_$#".indexOf(ch) == -1)) {
			 String var = sql.substring(i+1,j).toLowerCase();

                         //Log.note("binding var: " + var);
                         if (extraBindValues.containsKey(var)) {
                             Object value = extraBindValues.get(var);
                             // TODO: Verify functionality w/ nulls...
                             bind_vars.insertElementAt(value, ctr);
                             // Moved inside this if by DEE 10-2-00.
                             // Basically, we can only
                             ctr++;
                         }

			 // substitute the string right here, because
			 // I don't know how to make a regexp that leaves
			 // stuff in between string literals alone
			 // (might be improved later)

			 // up to and excluding the colon
			 //String before = sql.substring(0, i);
			 //String after = sql.substring(j);
			 //sql = before + "?" + after;
                         sql.delete(i, j);
                         sql.insert(i, '?');
			 break;
		     }
		 }
	     }
	 }
	 // com.oroinc.text.perl.Perl5Util re = new com.oroinc.text.perl.Perl5Util();
	 // sql = re.substitute("s/:[A-Z0-9_#\\$]+/?/gi", sql);
         //Log.note("Post-BindFixup on:" + toString());
     }

    
    /** 
     * Returns a list of bind values.
     * @return The list of bind values
     */
    public Object[] getBindValues() {
        bindFixup();
	return bind_vars.toArray();
    }

    /**
     * Performs a fixup and returns a string containg the SQL followed by the bind values.
     * For occassional debugging purposes; should never be manually combined with SQL.  Use the
     * append method to combine SqlStatements.
     */
    public String toString() {
        // Stack overflow if we use accessors due to bindFixup.
        return ("SQL:'" + sql.toString() + "'\nBind:" + bind_vars.toString() + "\nExtra bind:" + extraBindValues.toString());
    }
}
