package com.arsdigita.acs.html;

import java.util.Vector;
import com.arsdigita.acs.*;

/**
 * Class that displays a Yahoo-style context navbar for a logged-in
 * user, with "your workspace" at the root; or with the system name
 * at the root for non-logged in users.
 * <p>
 * Elements are added to the context bar in order; each element consists
 * of a path to be used as a relative URL link, and the title to be displayed
 * as a link to that path.  The last element added to the context bar is
 * the title of the current page, and a path should not be provided.
 * 
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @version $Version$
 */


public class ContextBar {
    
    private Vector paths;
    private Vector titles;
    private int type;

    /**
     * Personal context bar; "your workspace" at root
     */
    public final static int PERSONAL = 1;

    /**
     * Personal "your workspace" context bar for logged-in users;
     * points to root /index page for non-logged in users.
     */
    public final static int PERSONAL_OR_INDEX = 2;

    /**
     * Personal "your workspace" context bar for logged-in users
     * and "administration".
     */
    public final static int ADMIN = 3;

    /**
     * Group-scoped context bar for use in modules that use scoping
     */
    public final static int GROUP = 4;


    /** 
     * Creates a new context bar of the given type, for a given user.
     */
    public ContextBar(int type, UserInfo user) {
	paths = new Vector();
	titles = new Vector();
	if (type == PERSONAL || user != null) {
	    paths.addElement(Parameters.get("PvtHome", "acs", "/pvt/home"));
	    titles.addElement("Your Workspace");
	} else {
	    paths.addElement("/");
	    titles.addElement(Parameters.get("SystemName", ""));
	}

	if (type == ADMIN) { 
	    // TO DO: make this a parameter
        add("/admin", "Admin Home");
	}
    }

    /**
     * Creates a new personal context bar.
     */
    public ContextBar() { 
	this(PERSONAL_OR_INDEX, null);
    }

    /**
     * Creates a new context bar.  Takes the pageContext object and tries
     * to figure out whether we're on an admin page, or whether the user
     * is logged in or not. 
     */

    public ContextBar(javax.servlet.jsp.PageContext pc) { 
	paths = new Vector();
	titles = new Vector();

	if (((javax.servlet.http.HttpServletRequest)pc.getRequest()).getRequestURI().startsWith("/admin")) { 
	    // we're in an admin page
	    
	    this.type = ADMIN;
        add("/pvt/home", "Your Workspace");
        add("/admin", "Admin Home");

	} else {
	    
	    // try to see if we're group-scoped

	    Vector v = (Vector)pc.getRequest().getAttribute("group_context_bar_list");
	    if (v != null) {
		for (int i = 0; i < v.size(); i++) {
		    String[] pair = (String[])v.elementAt(i);
		    add(pair[0], pair[1]);
		}
		type = GROUP;
	    } else {
		{
		    this.type = PERSONAL_OR_INDEX;
		    UserInfo user = null;
		    try {
			user = Security.getVerifiedUserInfo(pc, false);
		    }
		    catch (Exception e) { 
			// just let it be null
		    } 
		    
		    if (user != null) {
			paths.addElement(Parameters.get("PvtHome", "acs", "/pvt/home"));
			titles.addElement("Your Workspace");
		    } else {
			paths.addElement("/");
			titles.addElement(Parameters.get("SystemName", ""));
		    }	    
		}
	    }
	}
    }
 
    /**
     * Adds a new element to the context bar.
     * @param path Relative URL to appear in <code>href</code>
     * @param title Title for this page
     */
    public void add(String path, String title) {
	if (path == null || path.length() == 0) {
	    path = "index";
	}

	paths.addElement(path);
	titles.addElement(title);
    }

    /** 
     * Adds a new element to the context bar with no path, for final
     * element.
     * @param title Title for this page
     */
    public void add(String title) {
	paths.addElement(null);
	titles.addElement(title);
    }

    /** 
     * Adds a new element to the context bar with no path, for final
     * element.
     * @param title Title for this page
     */
    public void add(String[][] pairs) {
	for (int i = 0; i < pairs.length - 1; i += 1)
	    add(pairs[i][0], pairs[i][1]);
	if ( Utilities.emptyString(pairs[pairs.length-1][1]) )
	    add(pairs[pairs.length-1][0]);
	else
	    add(pairs[pairs.length-1][0], pairs[pairs.length-1][1]);
    }

    /** 
     * Returns HTML represenation of this context bar.
     * @return HTML context bar
     */      
    public String getHTML() {
	StringBuffer sb = new StringBuffer();
	for (int i=0; i<paths.size()-1; i++) {
	    String path = (String)paths.elementAt(i);
	    String title = (String)titles.elementAt(i);
	    sb.append("<a href=");
	    sb.append(path);
	    sb.append(">");
	    sb.append(title);
	    sb.append("</a> : ");
	}
	sb.append((String)titles.elementAt(titles.size()-1));
	return sb.toString();
    }

    /** 
     * Returns HTML represenation of this context bar.
     * @return HTML context bar
     */      
    public String toString() {
	return getHTML();
    }
}
	    
	
    
