package com.arsdigita.acs.html;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.http.*;

/**
 * Displays a table of links that can be used to 
 * limit database query results.
 *
 * @author Luke Pond (luke@arsdigita.com)
 * @version $Id: DimensionalBar.java,v 1.2 2000/10/03 21:51:57 luke Exp $
 */
public class DimensionalBar {

    private PageContext pc;
    private String url;
    private AdSet selectedValues;
    private String joiner;

    // each dimension is modeled as an array of 3 Strings and a Vector of values.
    // each value is modeled as an array of 3 Strings.  
    private Vector dimensions = new Vector();

    public DimensionalBar(PageContext pc) throws ACSException {	
	this.pc = pc;

	// set up defaults for Url and Values properties
	// NYI: can we get the canonical name somehow from the request processor?
	// doing it this way, the ".jsp" suffix is included.
	url = ((HttpServletRequest)pc.getRequest()).getRequestURI();
	selectedValues = (AdSet)pc.getAttribute("form");
	if (selectedValues == null) {
	    throw new ACSException("Must create DimensionalBar object within an <ad:page> tag");
	}
	joiner = "and";
    }

    /**
     * Specifies a dimension along which to choose.
     * @param id the name of the URL variable
     * @param title the title of this variable, for displaying as a column header
     *              along the top of the DimensionalBar
     * @param defaultValue the value to display as selected, if a particular
     *                     value has not already been chosen by the user
     */
    public void addVariable(String id, String title, String defaultValue) {
	dimensions.addElement(new Object[] { id, title, defaultValue, new Vector() });
    }
     
    /**
     * Specifies an additional choice for one of the dimensions specified by addVariable.
     * @param id the name of the URL variable which may contain this value
     * @param value the value to assign to the URL variable when the user chooses this value
     * @param text the text to display for the value 
     * @param sqlClause the restriction to apply to a SQL query involving this variable.
     *                  This is a SQL expression that can follow "where" in a query.  The
     *                  word "where" should not be included.  The empty string is an allowable
     *                  value if no restriction should be added for this value (for example, the value
     *                  represents "All")
     */
    public void addValue(String id, String value, String text, String sqlClause) {
	for (int i = dimensions.size() - 1; i >= 0; i--) {
	    Object[] dimension = (Object[])dimensions.get(i);
	    if (id.equals((String)dimension[0])) {
		Vector values = (Vector)dimension[3];
		values.addElement(new String[] {value, text, sqlClause});
		break;
	    }
	}
    }

    /**
     * Returns an HTML fragment that displays all the choices that have been configured
     * with the addVariable and addValue methods.
     */
    public String toString() {
	StringBuffer sb = new StringBuffer();
	sb.append("<table border=0 cellspacing=0 cellpadding=3 width=100%>\n<tr>\n");

	// append dimension names as table headers
	Enumeration enum = dimensions.elements();
	while (enum.hasMoreElements()) {
	    Object[] dimension = (Object[])enum.nextElement();
	    sb.append(" <th bgcolor=\"#ECECEC\">").append(dimension[1]).append("</th>\n");
	}

	sb.append("</tr>\n<tr>\n");

	// append values inside a table cell for each dimension
        enum = dimensions.elements();
	while (enum.hasMoreElements()) {
	    Object[] dimension = (Object[])enum.nextElement();
	    sb.append(" <td align=center>[");

	    // find out what the current option value is.
	    // if it hasn't been set in selectedValues, use the default value.
	    String optionKey = (String)dimension[0];
	    String optionVal = selectedValues.get(optionKey);
	    if (optionVal == null) {
		optionVal = (String)dimension[2];
	    }

	    boolean first = true;
	    Enumeration options = ((Vector)dimension[3]).elements();
	    while (options.hasMoreElements()) {
		String[] option = (String[])options.nextElement();
		String thisOptionVal = option[0];
		if (first) {
		    first = false;
		} else {
		    sb.append(" | ");
		}

		if (optionVal.equals(thisOptionVal)) {
		    sb.append("<strong>").append(option[1]).append("</strong>");
		} else {
		    sb.append("<a href=\"").append(url).append("?");
		    sb.append(Utilities.exportURLVars(selectedValues, optionKey));
		    sb.append("&").append(Utilities.URLEncode(optionKey));
		    sb.append("=").append(Utilities.URLEncode(thisOptionVal));
		    sb.append("\">");
		    sb.append(option[1]);
		    sb.append("</a>");
		}
	    }
	    sb.append("]</td>\n");
	}
	sb.append("</tr>\n</table>\n");
	return sb.toString();
    }

    /**
     * Returns a SQL fragment for inclusion in the where clause of a query.  The fragment
     * is generated by choosing the sqlClause for the selected (or default) value for each dimension.
     */
    public String getDimensionalSql() {
	StringBuffer sb = new StringBuffer();

	boolean first = true;

	Enumeration enum = dimensions.elements();
	while (enum.hasMoreElements()) {
	    Object[] dimension = (Object[])enum.nextElement();
	
	    // find out what the current option value is.
	    // if it hasn't been set in selectedValues, use the default value.
	    String optionKey = (String)dimension[0];
	    String optionVal = selectedValues.get(optionKey);
	    if (optionVal == null) {
		optionVal = (String)dimension[2];
	    }
	
	    Enumeration options = ((Vector)dimension[3]).elements();
	    while (options.hasMoreElements()) {
		String[] option = (String[])options.nextElement();
		String thisOptionVal = option[0];
		if (optionVal.equals(thisOptionVal)) {
		    // found the sql clause for the current option value.
		    String code = option[2];
		    if (!Utilities.emptyString(code)) {
			if (first) {
			    first = false;
			} else {
			    sb.append(" ").append(joiner).append(" ");
			}
			// NYI: the tcl version allows variable subsitution in the sql clause.
			// i couldn't find any code that takes advantage of this.
			sb.append(code);
		    }
		}
	    }
	}
	return sb.toString();
    }

    /**
     * Sets the target for the links that are generated.
     * This is an optional parameter; by default the url of 
     * the current page is used, obtained from the PageContext.
     */
    public void setUrl(String url) {
	this.url = url;
    }

    /**
     * Specifies the values to be highlighted.  This is an optional
     * parameter; by default we use the form variables submitted
     * to the current page.
     */
    public void setValues(AdSet selectedValues) {
	this.selectedValues = selectedValues;
    }
}

