package com.arsdigita.acs.html;
import com.arsdigita.acs.*;
import java.util.*;
import com.oroinc.text.perl.Perl5Util;


/**
 * SidewaysBarChart.java
 * 
 * Java port of Eve Andersson's tcl proc, gr_sideways_bar_chart
 *
 * creates bar charts in plain html with 1x1 pixel gifs stretched to the 
 * right width and height.  It will work on any browser that supports tables 
 * (i.e., pretty much all of them). This package relies on a few gifs being 
 * present in /graphics/graphing-package/. 
 *
 * For a full-featured graphing package written in Java, 
 * see <a href="http://jplot.arsdigita.com">ArsDigita JPlot</a>.
 *
 * @author Luke Pond (luke@arsdigita.com)
 * @version $Id: SidewaysBarChart.java,v 1.2 2000/10/05 17:47:23 bschneid Exp $
 */

public class SidewaysBarChart {

    // customizable properties and their defaults

    private String[] legend = null;
    private String[] barColorList = new String[] { "blue", "dark-green", "purple", "red", "black", "orange", "medium_blue" };
    private boolean displayValues = false;
    private boolean displayScale = false;
    private String defaultDrilldownUrl = "";
    private boolean nonPercentValues = false;
    private int minLeftColumnWidth = 1;
    private int barHeight = 15;
    private int subcategorySpacing = 7;
    private boolean compareNonPercentsAcrossCategories = false;
    private String leftHeading = "";
    private String rightHeading = "";
    private boolean replaceNullSubcategoryWithNone = false;
    
    private Vector storage = new Vector();
    private static Hashtable hexColors = null;

    public SidewaysBarChart() {
        if (hexColors == null) {
            // first-time only init
            hexColors = new Hashtable();
            hexColors.put("red", "ff0000");
            hexColors.put("blue", "0000ff");
            hexColors.put("yellow", "ffff00");
            hexColors.put("black", "000000");
            hexColors.put("white", "ffffff");
            hexColors.put("dark-green", "009900");
            hexColors.put("aquamarine", "00ffff");
            hexColors.put("purple", "660099");
            hexColors.put("orange", "ff6600");
            hexColors.put("medium-blue", "0099ff");
            hexColors.put("magenta", "ff00ff");
            hexColors.put("muted-green", "669966");
            hexColors.put("muted-yellow", "999966");
            hexColors.put("muted-red", "996666");
            hexColors.put("muted-magenta", "996699");
            hexColors.put("muted-blue", "666699");
            hexColors.put("muted-aquamarine", "669999");
        }
    }

    /**

       Call addValue once for each bar in the chart.  The subcategory and category parameters
       determine the grouping that will be used.
       <p>
       To create a graph like this:

       <p>
       <blockquote>
       <pre>
       <b>Dogs</b>
       Samoyed    xxxxxxxxxxx 45%
       Black Lab  xxxx 20%
       Terrier    xxxxxxxxxxxxxxxxxxxx 66%
       <b>Cats</b>
       Calico     xxxxxxxx 39%
       Persian    xx 10%
       </pre>
       </blockquote>

       call addValue 5 times, as follows:
       <pre>
       SidewaysBarChart sbc = new SidewaysBarChart();
       sbc.addValue("Samoyed", "Dogs", 45);
       sbc.addValue("Black Lab", "Dogs", 20);
       sbc.addValue("Terrier", "Dogs", 66);
       sbc.addValue("Calico", "Cats", 39);
       sbc.addValue("Persian", "Cats", 10);
       </pre>

       <p> You can have multiple values per subCategory, in which case you should call
       addValue once for each value, without changing the subCategory or category parameters.
    */

    public void addValue(String subCategory, String category, int value) {
        Vector vals = null;
        for (int i = storage.size() - 1; i >= 0; i--) {
            Object[] objs = (Object[])storage.elementAt(i);
            if (subCategory.equals((String)objs[0]) && category.equals((String)objs[1])) {
                vals = (Vector)objs[2];
                break;
            }
        }
        if (vals == null) {
            Object[] objs = new Object[] { subCategory, category, new Vector() };
            storage.addElement(objs);
            vals = (Vector)objs[2];
        }
        vals.addElement(new Integer(value));
    }

    /**
     * Generates the HTML bar chart, according to the values set up by calling the addValue method.
     */
    public String toString() {
        Enumeration enum;
        Hashtable maxValues = null;
        Integer maxValueOfAllValues = null;

        if (nonPercentValues) {
            // If the values aren't percentages, I'll turn them into percentages just for the bar display.
	
            if (!compareNonPercentsAcrossCategories) {
                // The highest number in a category will correspond to a percentage of 75%, 
                // and the others will be relative to it.

                maxValues = new Hashtable();
                enum = storage.elements();
                while (enum.hasMoreElements()) {
                    Object[] objs = (Object[])enum.nextElement();
                    Vector allValsForCategory = (Vector)maxValues.get((String)objs[1]);
                    if (allValsForCategory == null) {
                        allValsForCategory = new Vector();
                        maxValues.put((String)objs[1], allValsForCategory);
                    }
                    allValsForCategory.addAll((Vector)objs[2]);
                }
                // sort each vector and replace it with its maximum value
                enum = maxValues.keys();
                while (enum.hasMoreElements()) {
                    String category = (String)enum.nextElement();
                    Object[] vals = ((Vector)maxValues.get(category)).toArray();
                    Arrays.sort(vals);
                    maxValues.put(category, vals[vals.length - 1]);
                }                                                   
            } else {
                // The highest number in all categories will correspond to a percentage of 75%, 
                // and the others will be relative to it.
                Vector valsForAllCategories = new Vector();
                enum = storage.elements();
                while (enum.hasMoreElements()) {
                    Object[] objs = (Object[])enum.nextElement();
                    valsForAllCategories.addAll((Vector)objs[2]);
                }
                Object[] vals = valsForAllCategories.toArray();
                Arrays.sort(vals);
                maxValueOfAllValues = (Integer)vals[vals.length - 1];
            }
        }

        StringBuffer sb = new StringBuffer();
        
        if (legend != null) {
            int legendCounter = 0;
            sb.append("<table border=1 cellspacing=0 cellpadding=5><tr><td>");

            for (int i = 0; i < legend.length; i++) {
                String key = legend[i];
                String color = barColorList[i % barColorList.length];
                sb.append("<img width=15 height=15 src=\"/graphics/graphing-package/");
                sb.append(color).append("-dot.gif\"> &nbsp;");
                appendFontTag(sb, "black", 3);
                sb.append(key).append("</font><br clear=all>");
            }
            sb.append("</td></tr></table><p>");
        }

        sb.append("<table border=0 cellspacing=0 cellpadding=0>");
        if (!Utilities.emptyString(leftHeading) || !Utilities.emptyString(rightHeading)) {
            sb.append("<tr><td>").append(leftHeading);
            sb.append("</td><td></td><td>").append(rightHeading);
            sb.append("</td></tr>");
            sb.append("<tr><td><img width=1 height=5 src=\"/graphics/graphing-package/white-dot.gif\"></td><td></td><td></td></tr>\n");
        }

        String prevCategory = null;
        
        enum = storage.elements();
        while (enum.hasMoreElements()) {
            Object[] objs = (Object[])enum.nextElement();
            String subcategory = (String)objs[0];
            String category = (String)objs[1];
            Vector values = (Vector)objs[2];
            // NYI: ability to provide a custom drilldownUrl for each subcategory
            
            if (!category.equals(prevCategory)) {
                prevCategory = category;
                sb.append("<tr><td><img width=").append(minLeftColumnWidth);
                sb.append(" height=10 src=\"/graphics/graphing-package/white-dot.gif\"><br clear=all>");
                appendFontTag(sb, "black", 4);
                sb.append(category).append("</font></td>");
                if (displayScale) {
                    sb.append("<td align=right><img width=10 height=15 src=\"/graphics/graphing-package/scale-left.gif\"><br clear=all><img width=1 height=3 src=\"/graphics/graphing-package/white-dot.gif\"></td><td><img width=320 height=15 src=\"/graphics/graphing-package/scale-main.gif\"><br clear=all><img width=1 height=3 src=\"/graphics/graphing-package/white-dot.gif\"></td></tr>");
                } else {
                    sb.append("<td><img width=10 height=15 src=\"/graphics/graphing-package/white-dot.gif\"></td><td> </td></tr>");
                }
            }

            sb.append("<tr><td>");
            appendFontTag(sb, "black", 3);
            sb.append(replaceNullSubcategoryWithNone && Utilities.emptyString(subcategory) ? "[none]" : subcategory);
            sb.append("</font></td><td width=10> </td><td>");

            int size = values.size();
            for (int i = 0; i < size; i++) {
                int val = ((Integer)values.elementAt(i)).intValue();
                int barLength;

                if (nonPercentValues) {
                    int max;
                    if (!compareNonPercentsAcrossCategories) {
                        max = ((Integer)maxValues.get(category)).intValue();
                    } else {
                        max = maxValueOfAllValues.intValue();
                    }
                    barLength = (max == 0 ? 0 : (75 * val / max));
                } else {
                    barLength = val;
                }

                String barColor = barColorList[i % barColorList.length];
                int imgWidth = (barLength == 0 ? 1 : 3 * barLength);

                sb.append("<img width=").append(imgWidth);
                sb.append(" height=").append(barHeight);
                sb.append(" src=\"/graphics/graphing-package/").append(barColor).append("-dot.gif\">");

                appendFontTag(sb, (String)hexColors.get(barColor), 1);
                sb.append("&nbsp;");

                if (!Utilities.emptyString(defaultDrilldownUrl)) {
                    sb.append("<a href=\"").append(defaultDrilldownUrl).append("\">");
                    sb.append(val).append("</a>");
                } else {
                    sb.append(val);
                }
                sb.append("</font><br clear=all>");
            }
            sb.append("<img width=1 height=").append(subcategorySpacing);
            sb.append("src=\"/graphics/graphing-package/white-dot.gif\"></td></tr>");
        }
        sb.append("</table>");
        return sb.toString();
    }

    private void appendFontTag(StringBuffer sb, String color, int size) {
        sb.append("<font face=arial size=").append(size);
        sb.append(" color=\"").append(color).append("\">");
    }


    public void setLegend(String[] legend) {
        this.legend = legend;
    }
    /**
       There is a default list of colors with which the values will be displayed
       if there is more than one value per subcategory, but if those
       colors don't meet your design needs, go ahead and supply your own color list.
    */
    public void setbarColorList(String[] barColorList) {
        this.barColorList = barColorList;
    }
    /**
       By default, the values (like 60%) aren't displayed after the bars, but if you
       want them to be, set this to "t".
    */
    public void setDisplayValues(boolean displayValues) {
        this.displayValues = displayValues;
    }
    /**
       By default, there's a scale displayed at the top of the chart that goes from
       0% to 100%, but if you don't want that scale there, set <code>display_scale_p</code> to "f".
       It doesn't make sense to have the scale if the values that are being charted
       are not percentages.
    */
    public void setDisplayScale(boolean displayScale) {
        this.displayScale = displayScale;
    }
    /**
       If supplied, should be a URL to which the numeric value (if being displayed) will be linked.
       This variable will undergo variable expansion, so <code>$category</code>, <code>$subcategory</code>, 
       and <code>$value</code> may be referenced.
    */
    public void setDefaultDrilldownUrl(String defaultDrilldownUrl) {
        this.defaultDrilldownUrl = defaultDrilldownUrl;
    }
    /**
       Set this to "t" if the numbers you're charting are not percentages.  Then
       this procedure will, instead of creating bars that are a fixed number times the
       value, display the values relative to each other (it will, in essence, pretend
       that the highest value within a category is 100%, and then display the other 
       values in that category relative to each other).
    */
    public void setNonPercentValues(boolean nonPercentValues) {
        this.nonPercentValues = nonPercentValues;
    }
    /**
       If you are going to stack charts, set the <code>min_left_column_width</code> to be
       the same for each of them so that they will line up.
       Otherwise, the left column, which contains the categories and subcategories, probably
       won't be the same for successive charts.
    */
    public void setMinLeftColumnWidth(int minLeftColumnWidth) {
        this.minLeftColumnWidth = minLeftColumnWidth;
    }
    /**
       By default, the height of each bar is 15 pixels.
    */
    public void setBarHeight(int barHeight) {
        this.barHeight = barHeight;
    }
    /**
       By default, the spacing between each subcategory is 7 pixels.
    */
    public void setSubcategorySpacing(int subcategorySpacing) {
        this.subcategorySpacing = subcategorySpacing;
    }
    /**
       This is only relevant if you are graphing values that are not percentages.
       Usually (as explained above for non_percent_values_p), the bars are drawn so that
       they are relative only to other bars in the same category.  This makes sense
       if one category has values like 900, 854, 942, and another totally unrelated
       category has values like 2.5, 3, 3.27.  You wouldn't want the bars in the second
       category to be drawn relative to the ones in the first category because they
       would be very small and, besides, they're not even related.  But, if the numbers
       ARE related across categories, then set <code>compare_non_percents_across_categories</code>
       to "t".
    */
    public void setCompareNonPercentsAcrossCategories(boolean compareNonPercentsAcrossCategories) {
        this.compareNonPercentsAcrossCategories = compareNonPercentsAcrossCategories;
    }
    /**
       Stuff (text/pictures/whatever) that goes above the items on the left of
       the chart (i.e. above the categories and subcategories).
    */
    public void setLeftHeading(String leftHeading) {
        this.leftHeading = leftHeading;
    }
    /**
       Stuff that goes above the items on the right of the chart (i.e. above the values).
    */
    public void setRightHeading(String rightHeading) {
        this.rightHeading = rightHeading;
    }
    /**
       Set this to "t" to have "[none]" be displayed as the subcategory if the
       subcategory is the empty string.
    */
    public void setReplaceNullSubcategoryWithNone(boolean replaceNullSubcategoryWithNone) {
        this.replaceNullSubcategoryWithNone = replaceNullSubcategoryWithNone;
    }
}
