package com.arsdigita.acs.html;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.util.*;
import javax.servlet.jsp.PageContext;
import javax.servlet.http.HttpServletRequest;
import com.oroinc.text.perl.Perl5Util;
import org.apache.turbine.util.Log;

/**
 * Displays database query results in an html table,  
 * with automatic sorting functionality.
 *
 * @author Luke Pond (luke@arsdigita.com)
 * @version $Id: Table.java,v 1.2 2000/10/03 21:51:57 luke Exp $
 */
public class Table {

    // set up by parameters to constructor
    private PageContext pc;
    private DataSource ds;

    // storage for addColumn:  each column is stored as an array of 
    // three strings and an integer.
    private Vector columns = new Vector();

    private Perl5Util re = new Perl5Util();

    private AdSet form;

    // properties configurable via property set methods
    private String orderTargetUrl;   // default set in constructor
    private String orderby;          // default set in constructor

    private String[] bandColors = new String[] { "", "#ececec" };
    private int rowsPerBand = 1;
    private String tableExtraHtml = "";
    private String headerRowExtra = "";
    private String ascOrderImg = "v";
    private String descOrderImg = "^";
    private int maxRows = 0;
    private int rowsPerPage = 0;
    private String tableBreakHtml = "<p>";
    private String missingText = "<em>No data found.</em>";


    /**
     * Default constructor to make it easier to create an anonymous subclass
     */
    public Table() {
    }

    /**
     * Public constructor
     * @param pc the pageContext for the current request
     */
    public Table(PageContext pc) throws ACSException {
	setPageContext(pc);
    }

    /**
     * Sets the JSP Page Context for the current request.  The Table object
     * requires this in order to get the orderby URL variable and the URL
     * of the current page request.
     */
    public void setPageContext(PageContext pc) throws ACSException {
	this.pc = pc;

	form = (AdSet)pc.getAttribute("form");
	if (form == null) {
	    throw new ACSException("Must create Table object within an <ad:page> tag");
	}
	
	orderby = form.get("orderby");
	if (orderby == null) {
	    orderby = "";
	}

	// NYI: can we get the canonical name somehow from the request processor?
	// doing it this way, the ".jsp" suffix is included.
	orderTargetUrl = ((HttpServletRequest)pc.getRequest()).getRequestURI();
    }

    /**
     * Adds a new column to the table.  
     * @param columnId The column name to get from the DataSource.  Also used to set orderby for sorting.
     * @param headerText The text to be wrapped in &lt;th&gt; and &lt;/th&gt; tags.
     * @param sortInfo Specifies the way sorting works for this column.  Choose from one of the SORT_*
     *        constants defined in this class.  If CUSTOM is specified, the formatOrderByClause method will be
     *        called to allow customization.
     * @param displayInfo Specifies the way column values are to be displayed.  Choose from one of the DISPLAY_*
     *        constants defined in this class.  If CUSTOM is specified, the formatCell method will be called
     *        to display values in this column.  See the docs for the formatCell method for further information.
     * @exception ACSException if the dataSource property has already been set.
     */
    public final void addColumn(String columnId, String headerText, int sortInfo, int displayInfo) 
    throws ACSException {
	if (ds != null) {
	    throw new ACSException("You may not add columns after setting the table's dataSource.");
	}
	columns.addElement(new Object[] { columnId, headerText, new Integer(sortInfo), new Integer(displayInfo) });
    }

    /**
     * Sets the dataSource property used by the table to retrieve data.
     * 
     * @param ds the data this table should display.  Most commonly
     *        obtained via the &lt;ad:db_query&gt; custom tag.  You
     *        should create the table and set up its column definitions
     *        before specifying the query, so that the table's 
     *        orderByFromSortSpec method can be used to limit the query.
     */
    public void setDataSource(DataSource ds) {
	this.ds = ds;
    }

    public String toString() {
	try {
	    return getHtml();
	} catch(Exception e) {
	    Log.error("Exception caught in Table.toString: " + e.getMessage());
	    return "";
	}
    }

    /**
     * Iterates through the DataSource to generate an html table.
     */
    public final String getHtml() throws ACSException, DataSourceException {

	if (ds == null) {
	    throw new ACSException("You must set the dataSource property");
	}
	if (pc == null) {
	    throw new ACSException("You must set the pageContext property");
	}

	int count=0;
	int bandCount=0;
	int pageCount=0;
	int bandColor=0;
	int nBands = (bandColors == null ? 0 : bandColors.length);

	// get the current ordering information
	String orderbykey = "::not_sorted::";
	String order = "asc";
	
	if (re.match("/^([^*,]+)([*]?)/", orderby)) {
	    orderbykey = re.group(1);
	    if (re.group(2).equals("*")) {
		order = "desc";
	    }
	}

	String export = Utilities.exportURLVars(form, "orderby");
	if (!Utilities.emptyString(export)) {
	    export += "&";
	}
	String sortUrl = orderTargetUrl + "?" + export + "orderby=";
	StringBuffer sb = new StringBuffer();
		
	// generate the table header 
	if (Utilities.emptyString(tableExtraHtml)) {
	    sb.append("<table>\n");
	} else {
	    sb.append("<table ").append(tableExtraHtml).append(">\n");
	}
	if (Utilities.emptyString(headerRowExtra)) {
	    sb.append("<tr>\n");
	} else {
	    sb.append("<tr ").append(headerRowExtra).append(">\n");
	}

	Enumeration enum = columns.elements();
	while (enum.hasMoreElements()) {
	    Object[] col = (Object[])enum.nextElement();
	    String columnId = (String)col[0];
	    String headerText = (String)col[1];
	    Integer sortInfo = (Integer)col[2];
	    if (sortInfo.intValue() == SORT_NO_SORT) {
		sb.append(" <th>").append(headerText).append("</th>\n");
	    } else {
		String orderImg;
		if (columnId.equals(orderbykey)) {
		    orderImg = (order.equals("asc") ? ascOrderImg : descOrderImg);
		} else {
		    orderImg = "";
		}
		String newOrderby = Utilities.URLEncode(newSortSpec(columnId, orderby));
		sb.append(" <th><a href=\"").append(sortUrl).append(newOrderby);
		sb.append("\">\n").append(headerText).append("</a>&nbsp;");
		sb.append(orderImg).append("</th>\n");
	    }
	}
	String header = sb.toString();
	sb = new StringBuffer();

	while (ds.next()) {

	    if (maxRows != 0 && count >= maxRows) {
		// NYI: need to close the ResultSet somehow
		break;
	    }

	    // append the header if we're starting a new page
	    if (count == 0) {
		sb.append(header);
	    } else if (pageCount == 0) {
		sb.append("</table>\n").append(tableBreakHtml).append("\n").append(header);
	    }

	    // generate the row band color
	    if (nBands != 0) {
		if (bandCount >= rowsPerBand) {
		    bandCount = 0;
		    bandColor = (bandColor + 1) % nBands;
		}
		String color = bandColors[bandColor];	       
		if (Utilities.emptyString(color)) {
		    sb.append("<tr>\n");
		} else {
		    sb.append("<tr bgcolor=\"" + color + "\">\n");
		}
	    } else {
		sb.append("<tr>\n");
	    }

	    // display each column for this row
	    enum = columns.elements();
	    while (enum.hasMoreElements()) {
		Object[] col = (Object[])enum.nextElement();
		String columnId = (String)col[0];
		Integer displayInfo = (Integer)col[3];
		privateFormatCell(sb, columnId, displayInfo, ds);
	    }

	    count++;
	    bandCount++;

	    // handle page breaking
	    pageCount++;
	    
	    if (rowsPerPage != 0 && pageCount >= rowsPerPage) {
		bandColor = 0;
		bandCount = 0;
		pageCount = 0;
	    }

	    sb.append("</tr>");
	}

	if (count > 0) {
	    sb.append("</table>\n");
	} else {
	    sb.append(missingText);
	}
	return sb.toString();
    }

    /**
     * Generates an html fragment for a table cell.  Behavior depends on the value 
     * of the column's displayInfo parameter, as passed to the addColumn method.  
     */
    private final void privateFormatCell(StringBuffer sb, String columnId, Integer displayInfo, DataSource ds) 
    throws DataSourceException {
	switch (displayInfo.intValue()) {
	case DISPLAY_DEFAULT:
	    sb.append(" <td>").append(ds.get(columnId)).append("</td>\n");
	    break;
	case DISPLAY_LEFT:
	    sb.append(" <td align=left>").append(ds.get(columnId)).append("</td>\n");
	    break;
	case DISPLAY_RIGHT:
	    sb.append(" <td align=right>").append(ds.get(columnId)).append("</td>\n");
	    break;
	case DISPLAY_CENTER:
	    sb.append(" <td align=center>").append(ds.get(columnId)).append("</td>\n");
	    break;
	case DISPLAY_TRUE_OR_FALSE:
	    sb.append(" <td align=center>");
	    sb.append(ds.get(columnId).equals("f") ? "No": "Yes");
	    sb.append("</td>\n");
	    break;
	case DISPLAY_ZERO_OR_ONE:
	    sb.append(" <td align=center>");
	    sb.append(ds.get(columnId).equals("0") ? "No": "Yes");
	    sb.append("</td>\n");
	    break;
	case DISPLAY_CUSTOM:
	    formatCell(sb, columnId, ds);
	    break;
	}
	return;
    }

    /**
     * For columns whose displayInfo parameter is set to CUSTOM, this method is called so that
     * subclasses can generate a display different from what is possible with the default displays.
     * @param sb Generate output by calling append on this StringBuffer.  
     * @param columnId the column in the DataSource we want to display
     * @param ds the DataSource we are in the process of displaying
     */
    public void formatCell(StringBuffer sb, String columnId, DataSource ds)
    throws DataSourceException {
	// intended for subclasses to override
    }


    // BEGIN SORTING-RELATED METHODS

    /**
     * Makes a new orderby string, sorting by columnId.
     *
     * If the key is followed by "*", that indicates the ordering should
     * be reversed from the default ordering for that key.
     *
     * Old sort keys are retained, so the sort appears to be a little more stable.
     * That is, suppose two things are sorted into an order, and their values for a
     * different column are the same.  If that different column is used as the primary
     * sort key to reorder, the things which have the same value for the newly-sorted
     * column will remain in the same relative order.
     */
    public String newSortSpec(String columnId, String orderby) {
	if (Utilities.emptyString(orderby)) {
	    return columnId;
	} 
	orderby += ",";

	if (re.match("/^" + columnId + "([*]?),/", orderby)) {
	    // if this was already the first key, then reverse order
	    if (re.group(1).equals("*")) {
		orderby = columnId + "," + re.postMatch();
	    } else {
		orderby = columnId + "*," + re.postMatch();
	    }
	} else {
	    orderby = re.substitute("s/," + columnId + "[*]?,/,/", orderby);
	    orderby = columnId + "," + orderby;
	}       
	return re.substitute("s/,$//", orderby);
    }

    /**
     * Translates the "orderby" URL parameter into a SQL ORDER BY clause
     * that may be inserted into a query.  If a column has SORT_CUSTOM
     * specified for its sortInfo parameter, the formatSortClause method
     * will be called to generate the custom sort clause.
     * @exception ACSException if pageContext has not been set
     */
    public String orderByFromSortSpec() throws ACSException {
	if (pc == null) {
	    throw new ACSException("You must set the pageContext property");
	}
	StringBuffer sb = new StringBuffer();
	Enumeration e = re.split("/,/", orderby).elements();
	while (e.hasMoreElements()) {
	    String sortKeySpec = (String)e.nextElement();
	    if (re.match("/^([A-Za-z_0-9]+)(\\*?)$/", sortKeySpec)) {
		String sortKey = re.group(1);
		String order = re.group(2).equals("*") ? "desc" : "asc";

		Enumeration eCol = columns.elements();
		while (eCol.hasMoreElements()) {
		    Object[] col = (Object[])eCol.nextElement();
		    if (sortKey.equals((String)col[0])) {
			sb.append( sb.length() == 0 ? "order by " : ", ");
			
			switch (((Integer)col[2]).intValue()) {
			case SORT_DEFAULT:
			    sb.append(sortKey).append(" ").append(order);
			    break;
			case SORT_CUSTOM:
			    formatSortClause(sb, sortKey, order);
			    break;
			}
			break;
		    }
		}
	    }
	}
	return sb.toString();
    }    

    /**
     * Called from the orderByFromSortSpec method for columns whose sortInfo parameter
     * has been set to SORT_CUSTOM.  
     * @param sb Contains the ORDER BY clause that orderByFromSortSpec is generating.
     *           You should append your custom sort expression to this StringBuffer.
     * @param columnId The column for which to provide custom sort information.
     * @param order Either "desc" or "asc", according to the current state of the 
     *        orderby URL parameter.  
     */
    public void formatSortClause(StringBuffer sb, String columnId, String order) {
	// intended for subclasses to override
    }

    // BEGIN OPTIONAL PROPERTY SETTER METHODS
    /**
     * The url linked by the table header tags, meant to sort the table on a different column.  
     * Defaults to the current request's URL.
     */
    public void setOrderTargetUrl(String orderTargetUrl) {
	this.orderTargetUrl = orderTargetUrl;
    }

    /**
     * Each row in the table will be displayed with alternating colors from this array.
     * If a String in the array is empty, no bgcolor attribute will be given for the row,
     * so it will be displayed with the page's background color.  By default we band with
     * the background color (typically white) and gray.  If you wish to disable banding
     * entirely, set bandColors to null.
     */
    public void setBandColors(String[] bandColors) {
	this.bandColors = bandColors;
    }

    /**
     * How many rows to display within a single color band.  Set to 1 by default.
     */
    public void setRowsPerBand(Integer rowsPerBand) {
	this.rowsPerBand = rowsPerBand.intValue();
    }

    /**
     * This is the variable that specifies the last sort column that was clicked.
     * By default, we look for a form variable named "orderby".
     */
    public void setOrderby(String orderby) {
	this.orderby = orderby;
    }

    /**
     * This string may contain any attributes to add to the &lt;table&gt; tag.
     */
    public void setTableExtraHtml(String tableExtraHtml) {
	this.tableExtraHtml = tableExtraHtml;
    }

    /**
     * This string may contain any attributes to add to the &lt;th&gt; tag.
     */
    public void setHeaderRowExtra(String headerRowExtra) {
	this.headerRowExtra = headerRowExtra;
    }

    /**
     * This string may contain an icon to display next to the table header 
     * which is currently sorted in ascending order.  By default we use
     * the string "v".
     */
    public void setAscOrderImg(String ascOrderImg) {
	this.ascOrderImg = ascOrderImg;
    }

    /**
     * This string may contain an icon to display next to the table header 
     * which is currently sorted in descending order.  By default we use
     * the string "^".
     */
    public void setDescOrderImg(String descOrderImg) {
	this.descOrderImg = descOrderImg;
    }

    /**
     * Specifies the maximum number of rows to display in the table.
     * Set to 0 by default, which means "display everything".
     */
    public void setMaxRows(Integer maxRows) {
	this.maxRows = maxRows.intValue();
    }

    /**
     * Breaks up the display into multiple &lt;table&gt; tags.
     * Useful if displaying a large amount of data because the
     * browser can render the first page without waiting for the
     * rest to download.  Set to 0 by default, which disables
     * the table breaking mechanism.
     */
    public void setRowsPerPage(Integer rowsPerPage) {
	this.rowsPerPage = rowsPerPage.intValue();
    }

    /**
     * Specifies the html to use between &lt;/table&gt; and the next
     * &lt;table&gt;, when breaking the display into multiple tables
     * using the rowsPerPage property.  Set to &lt;p&gt; by default.
     */
    public void setTableBreakHtml(String tableBreakHtml) {
	this.tableBreakHtml = tableBreakHtml;
    }

    /**
     * Message to display when the DataSource is empty.
     */
    public void setMissingText(String missingText) {
	this.missingText = missingText;
    }

    // Per-column display options
    public final static int DISPLAY_DEFAULT=0;
    public final static int DISPLAY_LEFT=1;
    public final static int DISPLAY_RIGHT=2;
    public final static int DISPLAY_CENTER=3;
    public final static int DISPLAY_TRUE_OR_FALSE=4;
    public final static int DISPLAY_ZERO_OR_ONE=5;
    public final static int DISPLAY_CUSTOM=6;

    // Per-column sort options
    public final static int SORT_DEFAULT=10;
    public final static int SORT_NO_SORT=11;
    public final static int SORT_CUSTOM=12;


    // NYI: we don't have the "columns" parameter which was intended 
    // to allow customization of columns to display. 

    // NYI: not currently supporting the hack for general permissions to allow
    // additional "fake" rows to be passed in.

    // NYI: "audit mode"

    // NYI: pre_row_code property


    public static void main(String[] args) throws Exception {
	Table t = new Table(null);
	String columnId = args[0];
	String sortSpec = args[1];
	System.out.println(t.newSortSpec(columnId, sortSpec));
    }
}

