package com.arsdigita.acs.tags;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.io.*;

/**
 * Tag to pass control from a business-logic JSP (which does database
 * queries, access filtering, etc.) to a presentation-logic template JSP.
 *
 * <p>
 *
 * The following requirements must be met for the template tag to work:
 * 
 * <ul>
 *
 * <li> All template JSPs must end with a .tpl extension, which must
 * be mapped to the JSP handler servlet; this is so that we can bypass
 * the request processor servlet for templates, but not for regular
 * JSPs.
 * 
 * <li> The presentation-layer JSP files must reside in the /templates
 * directory, relative to the context root; this directory parallels
 * the normal-JSP hierarchy, so that use_template in /path/to/page.jsp
 * will forward to the template /templates/path/to/page.tpl. 
 *
 * <li> If you get an error message from a template that "response has
 * already been committed," try taking out the ad:header tags in the
 * template.
 *
 */

public class UseTemplateTag
    extends TagSupport
    implements Tag
{
    /** 
     * do we forward or include the template?  
     * (some pages require templates to be included rather than forwarded)
     */
    private String method = "forward";

    public int doStartTag() throws JspException {
	return SKIP_BODY;
    }
    
    public int doEndTag() throws JspException {
	try {
	    // first figure out what page we're looking at
	    HttpServletRequest req = (HttpServletRequest)pageContext.getRequest();
	    String request = req.getRequestURI();
	    // strip off the JSP and replace it with "tpl"
	    String template = request.substring(0,request.lastIndexOf("."));
	    template += ".tpl";

	    // strip off servlet context, if any
	    String contextPath = req.getContextPath();
	    template = template.substring(contextPath.length());
	    if (contextPath.endsWith("/")) 
		contextPath = contextPath.substring(0, contextPath.length()-1);
	    if (!template.startsWith("/")) 
		template = "/" + template;

	    String fullPath = contextPath + "/templates" + template;
	    // System.out.println("including template " + fullPath);
	    if (method.equals("forward")) {
		pageContext.forward(fullPath);
	    } else {
		pageContext.include(fullPath);
	    }
	} catch (ServletException se) {
		throw new JspTagException("UseTemplateTag: caught ServletException: " + se.getMessage());
	}
	catch (IOException se) {
	    throw new JspTagException("UseTemplateTag: caught IOException: " + se.getMessage());
	}
	return EVAL_PAGE;
    }

    public void setMethod(String meth) throws JspTagException { 
	if (method.equals("forward") || method.equals("include")) { 
	    method = meth;
	} else {
	    throw new JspTagException("UseTemplateTag: method must be \"forward\" or \"include\"");
	}
    }
}
