package org.apache.turbine.services.localization;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Jyve",
 *    "Apache Jyve", "Jyve Project", "Apache Jyve Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

/** This class has been added 17.01.2000 by [jm],mediaphil digital media

	Jonas Maurus <a href="mailto:jm@mediaphil.de">jm@mediaphil.de</a>
	Please contact me if you have any questions, which ARE NOT ANSWERED
	by the Java Localization/Internationalization Documentation.
*/
import java.util.*;
import org.apache.turbine.services.*;
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.TurbineResources;

/** 
 * This class is the single point of access to all localization resources.
 * It caches different ResourceBundles for different Locales.
 * <p>
 * Usage example:<br>
 * LocalizationService ls = (LocalizationService)TurbineServices.getInstance()
 *                                   .getService(LocalizationService.SERVICE_NAME);
 * <p>
 * Then call one of four methods to retrieve a ResourceBundle:
 * <br>
 * - getBundle("MyBundleName")<br>
 * - getBundle("MyBundleName", httpAcceptLanguageHeader)<br>
 * - getBundle("MyBundleName", RunData)<br>
 * - getBundle("MyBundleName", Locale)<br>
 *
 * @author Jonas Maurus, mediaphil digital media <a href="mailto:jm@mediaphil.de">jm@mediaphil.de</a>
 * @author Jon S. Stevens <a href="mailto:jon@clearink.com">jon@clearink.com</a>
 * @author Frank Y. Kim <a href="mailto:frank.kim@clearink.com">frank.kim@clearink.com</a>
 */
public class TurbineLocalizationService extends TurbineBaseService 
        implements LocalizationService
{    
    /** The ResourceBundles in this service. 
        Key=bundle name
        Value=Hashtable containing ResourceBundles keyed by Locale. */
    private static Hashtable bundles = null;
    /** The name of the default bundle to use. */
    private static String defaultBundle = null;
    
    public TurbineLocalizationService()
    {
        bundles = new Hashtable();
        defaultBundle = TurbineResources.getString("locale.default.bundle");
    }

    /**
     * This method returns a ResourceBundle given the bundle name "DEFAULT" and
     * the default Locale information supplied in TurbineProperties.
     */
    public ResourceBundle getBundle()
    {
        return getBundle( defaultBundle );
    }
    
    /**
     * This method returns a ResourceBundle given the bundle name and
     * the default Locale information supplied in TurbineProperties.
     */
    public ResourceBundle getBundle(String bundleName)
    {
        String language = TurbineResources.getString("locale.default.language", "en").trim();
        String country = TurbineResources.getString("locale.default.country", "US").trim();
        
        return getBundle( bundleName, new Locale(language, country) );
    }
    
    /**
     * This method returns a ResourceBundle given the bundle name and
     * the Locale information supplied in the HTTP "Accept-Language" header.
     */
    public ResourceBundle getBundle(String bundleName, String languageHeader)
    {
        return getBundle( bundleName, LocaleDetector.getLocale(languageHeader) );
    }
    
    /**
     * This is a convenience which method returns a ResourceBundle given 
     * the bundle name and the Locale information supplied in the 
     * HTTP "Accept-Language" header which is stored in RunData.
     */
    public ResourceBundle getBundle(RunData data)
    {
        Locale locale = LocaleDetector.getLocale(data);
        if (locale == null)
            return getBundle();
        return getBundle( defaultBundle, locale );
    }

    /**
     * This is a convenience which method returns a ResourceBundle given 
     * the bundle name and the Locale information supplied in the 
     * HTTP "Accept-Language" header which is stored in RunData.
     */
    public ResourceBundle getBundle(String bundleName, RunData data)
    {
        Locale locale = LocaleDetector.getLocale(data);
        if (locale == null)
            return getBundle(bundleName);
        return getBundle( bundleName, locale );
    }
    
    /**
     * This method returns a ResourceBundle for the given bundle name and
     * the given Locale.
     */
    public ResourceBundle getBundle(String bundleName, Locale locale)
    {
        bundleName = bundleName.trim();
        
        if ( bundles.containsKey(bundleName) )
        {
            Hashtable locales = (Hashtable)bundles.get(bundleName);
            
            if ( locales.containsKey(locale) )
            {
                return (ResourceBundle)locales.get(locale);
            }
            else
            {
                // try to create a ResourceBundle for this Locale
                ResourceBundle rb = ResourceBundle.getBundle(bundleName, locale);
                
                // cache the ResourceBundle in memory
                locales.put( locale, rb );
                
                // can't call getLocale() cuz that is jdk2
                // this needs to be changed back cuz the above
                // approach caches extra Locale and Bundle objects
                //locales.put( rb.getLocale(), rb );
                
                return rb;
            }
        }
        else
        {
            // try to create a ResourceBundle for this Locale
            ResourceBundle rb = ResourceBundle.getBundle(bundleName, locale);
                
            // cache the ResourceBundle in memory
            Hashtable ht = new Hashtable();
            ht.put( locale, rb );
            
            // can't call getLocale() cuz that is jdk2
            // this needs to be changed back cuz the above
            // approach caches extra Locale and Bundle objects
            //ht.put( rb.getLocale(), rb );
            
            bundles.put( bundleName, ht );
                
            return rb;
        }
    }
    
    /**
     * This method sets the name of the defaultBundle.
     */
    public void setBundle(String defaultBundle)
    {
        this.defaultBundle = defaultBundle;
    }

}