package org.apache.turbine.tools;

import java.sql.*;
import java.io.*;
import java.util.*;
import org.apache.turbine.util.db.map.*;

/** 
 * A utility class that builds a DatabaseMap from metadata of a JDBC
 * database.  You provide the database driver, url, user and password
 * and the class provide an insance of DatabaseMap.<p>
 * Please note that database metadata is an optional feature and therefore
 * some databases does not implement it.  To use this class you will 
 * need a database and driver that supports metadata.
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 */
public class JDBCSchemaToDatabaseMap
{
    private String dbUrl = "";
    private String dbDriver = "";
    private String dbUser = "";
    private String dbPassword = "";

    /**
     * Get the database url
     */
    public String getDbUrl ()
    { 
        return dbUrl;
    }
    
    /**
     * Get the database driver
     */
    public String getDbDriver ()
    {
        return dbDriver;
    }
    
    /**
     * Get the database user
     */
    public String getDbUser ()
    {
        return dbUser;
    }
    
    /**
     * Get the databae password
     */
    public String getDbPassword ()
    {
        return dbPassword;
    }
    
    /**
     * Set the database url
     */
    public void setDbUrl (String v)
    {
        dbUrl = v;
    }
    
    /**
     * Set the database driver
     */
    public void setDbDriver (String v)
    {
        dbDriver = v;
    }

    /**
     * Set the database user.  By default the user is an empty string
     */
    public void setDbUser (String v)
    {
        dbUser = v;
    }
    
    /**
     * Set the database password.  By default the password is an empty string
     */
    public void setDbPassword (String v)
    {
        dbPassword = v;
    }
    
    
    /**
     * Generates a database map from a JDBC 
     */
    public DatabaseMap generateMap(String mapName) throws Exception
    {
        //Connect to the interbase database
        //Load the Interbase Driver
        Class.forName(getDbDriver());
        //Attemt to connect to a database
        Connection con = DriverManager.getConnection(getDbUrl(),getDbUser(),getDbPassword());
        //Get the database Metadata
        DatabaseMetaData dbMetaData = con.getMetaData();
        
        
        //The database map
        DatabaseMap dbMap = new DatabaseMap(mapName);
        Vector tableList = getTableNames(dbMetaData);

        for (int i = 0;i < tableList.size(); i++)
        {
            //Add Table
            String curTable = (String) tableList.elementAt(i);
            dbMap.addTable(curTable);

            //Add Columns
            TableMap tblMap = dbMap.getTable(curTable);
        
            Vector columns = getColumns(dbMetaData,curTable);
            Vector primKeys = getPrimaryKeys(dbMetaData,curTable);
            Vector forgnKeys = getForeignKeys(dbMetaData,curTable);

            for (int j = 0;j < columns.size();j++)
            {
                Vector v = (Vector) columns.elementAt(j);
                String name = (String) v.elementAt(0);
                Integer type = (Integer) v.elementAt(1);
            
                if (type.intValue() == Types.BIGINT ||
                    type.intValue() == Types.INTEGER ||
                    type.intValue() == Types.SMALLINT ||
                    type.intValue() == Types.TINYINT)
                    {
                        tblMap.addColumn(name, new Integer(0));       
                    }
            
                else if (type.intValue() == Types.CHAR ||
                         type.intValue() == Types.LONGVARCHAR ||
                         type.intValue() == Types.VARCHAR)
                         {
                            tblMap.addColumn(name, "");
                         }
                        
                else if (type.intValue() == Types.DATE ||
                         type.intValue() == Types.TIME ||
                         type.intValue() == Types.TIMESTAMP)
                         {
                            tblMap.addColumn(name, new java.util.Date());
                         }
                else
                {
                    tblMap.addColumn(name, "");       
                }        
                
            }

            //Set the primary Keys
            for (int k = 0;k < primKeys.size();k++)
            {
                String curPrimaryKey = (String) primKeys.elementAt(k);
                ColumnMap clMap = tblMap.getColumn(curPrimaryKey);
                clMap.setPrimaryKey(true);
            }
    
            //Set the foreign Keys
            for (int l = 0;l < forgnKeys.size();l++)
            {
                Vector v = (Vector)forgnKeys.elementAt(l);
                ColumnMap clMap = tblMap.getColumn((String)v.elementAt(1));
                if (clMap != null)
                {
                    clMap.setForeignKey((String)v.elementAt(0),(String)v.elementAt(1));
                }
            }
        }
        return dbMap;
    }   

    
    /**
     * Get all the table names in the current database that is not system tables.
     */ 
    public Vector getTableNames(DatabaseMetaData dbMeta) throws SQLException
    {
        System.out.println ("Getting tables");
        
        String[] types = {"TABLE"};
        
        ResultSet tableNames = dbMeta.getTables("",null,"%",types);
        Vector tables = new Vector();
        while (tableNames.next())
        {
            String name = tableNames.getString(3);
            String type = tableNames.getString(4);
            tables.addElement(name);
        }
        return tables;
    }


    /**
     * Retrieves all the column names and types for a given table from JDBC metadata.
     * It returns a vector of vectors.  Each element of the returned vector is a
     * vector with element 0 a String object for the column name and element 1 a 
     * Integer object for the column type.  You can use the Types class to decode the
     * column type.
     */
    public Vector getColumns(DatabaseMetaData dbMeta,String tableName) throws SQLException
    {
        ResultSet columnSet = dbMeta.getColumns("",null,tableName,null);
        Vector columns = new Vector();
        while (columnSet.next())
        {
            String name = columnSet.getString(4);
            Integer sqlType = new Integer(columnSet.getString(5));
            Vector v = new Vector();
            v.addElement (name);
            v.addElement (sqlType);
            columns.addElement (v);
        }
        return columns;
    }

    /**
     * Retrieves a list of primary key columns for a given table.
     */
    public Vector getPrimaryKeys(DatabaseMetaData dbMeta,String tableName) throws SQLException
    {
        ResultSet primaryKeys = dbMeta.getPrimaryKeys("",null,tableName);
        Vector keys = new Vector();
        while (primaryKeys.next())
        {
            keys.addElement(primaryKeys.getString(4));
        }
        return keys;
    }


    /**
     * Retrieves all the foreign key and table names for a given table from JDBC metadata.
     * It returns a vector of vectors.  Each element of the returned vector is a
     * vector with element 0 a String object for the table name and element 1 a 
     * Integer object for the column name.
     */
    public Vector getForeignKeys(DatabaseMetaData dbMeta,String tableName) throws SQLException
    {
        ResultSet foreignKeys = dbMeta.getImportedKeys("",null,tableName);
        Vector keys = new Vector();
        
        if (foreignKeys != null)
        {
            while (foreignKeys.next())
            {
                String table = foreignKeys.getString(3);
                String column = foreignKeys.getString(4);
                Vector v = new Vector();
                v.addElement (table);
                v.addElement (column);
                keys.addElement (v);
            }
        }
            
        return keys;
    }

    /** 
     *For testig only
     */
    static public void main (String[] args) throws Exception
    {
        JDBCSchemaToDatabaseMap gen = new JDBCSchemaToDatabaseMap();
        gen.setDbUrl ("");
        gen.setDbDriver ("");
        gen.setDbPassword ("");
        gen.setDbUser ("");
        
        BufferedReader br = new BufferedReader (new InputStreamReader (System.in));
        
        DatabaseMap dbMap = gen.generateMap ("MapName");
        TableMap[] tables = dbMap.getTables();
        for (int i=0; i<tables.length; i++)
        {
            TableMap table = tables[i];
            ColumnMap[] columns = table.getColumns();
            for (int j=0; j<columns.length; j++)
            {
                ColumnMap c = columns[j];
                System.out.print ("   "+c.getColumnName()+"   "+c.getType().getClass()+"   ");
                if (c.isPrimaryKey()) System.out.print ("*");
                if (c.isForeignKey()) System.out.print ("^");
                System.out.println();
            }
                
            System.out.println (table.getName());
            br.readLine();
        }
        
        
    }
    
}