package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.io.*;
import java.util.*;

import org.apache.turbine.util.db.map.*;

/**
    This class will read in a MySQL schema and attempt to create
    a populated DatabaseMap from that schema. The usefulness of this
    class is then to take that DatabaseMap and output your MapBuilder
    classes. Right now, the only example of outputting the MapBuilder
    classes is in the <a href="http://scarab.tigris.org"/>Scarab</a> Project
    distribution. You also need to write your MySQL schema in a format that
    this parser can read. For an example of this format, please look at the 
    MySQL schema for the Scarab Project. Essentially, this format is a format
    that is produced by the ERStudio commercial ER design software for an
    Oracle Schema and then converted to be MySQL compliant with a few
    global find/replaces in the file. It should be pretty easy (ie: it should 
    work, but isn't tested) to also use this same class for an ERStudio 
    generated Oracle .sql file as well because of the way that I coded things.
    <p>
    The reason for this lack of abstraction is because it is generally very project
    specific. If someone would like to develop something that is less 
    project specific, that would be appreciated. Hopefully though, with this
    class, you should be able to get an understanding of what it takes to 
    generate a parser to convert a .sql file into an in memory DatabaseMap
    representation.
    <p>
    Example usage is to simply pass in the name of the DatabaseMap you want
    to create and the FileReader path to the file. As a result, you should
    get a fully populated DatabaseMap.

    @author Jon S. Stevens <a href="mailto:jon@latchkey.com">jon@latchkey.com</a>
    @version $Id: MySQLSchemaToDatabaseMap.java,v 1.1 2000/08/30 21:08:43 bschneid Exp $
*/
public class MySQLSchemaToDatabaseMap
{
    /**
        Pass in a FileReader which contains the path to 
        the .sql file. This method will return the DatabaseMap
        fully populated. Please see the class documentation for
        more details. Also, as always, read the source code.
        It is pretty well documented.
    */
    public static DatabaseMap generateMap(String mapName, FileReader fr)
        throws Exception
    {
        /** name of the table */
        String tableName = null;
        /** our database map */
        DatabaseMap dbMap = new DatabaseMap(mapName);
        /** a buffered reader to loop over things line by line */
        BufferedReader br = new BufferedReader(fr);
        /** the current line */
        String line = null;
        /** are we in a create statement? */
        boolean inCreate = false;
        /** the data in the create statement */
        Vector columndata = new Vector();
        
        /** loop over every line in the file */
        while ((line = br.readLine()) != null)
        {
            // ignore crap we don't need
            if (line == null || line.length() == 0 || line.startsWith("#") 
            || line.startsWith("//")|| line.startsWith(")") || line.startsWith("--") )
                continue;
            
            // found end of table creation
            // now parse data into a TableMap/ColumnMap
            if (line.indexOf(";") != -1 )
            {
                parseColumnData(dbMap, tableName, columndata);
                columndata = new Vector();
                inCreate = false;
                continue;
            }
            
            // found beginning of CREATE TABLE
            if (line.startsWith ("CREATE TABLE "))
            {
                inCreate = true;
                tableName = getTableName(line);
                continue;
            }
            
            // we are in CREATE TABLE
            if (inCreate)
            {
                columndata.addElement ( line );
            }
        }
        return dbMap;
    }
    /**
        parses the vector of column data into the DatabaseMap
    */
    private static void parseColumnData(DatabaseMap dbMap, String tableName, Vector columndata)
    {
        dbMap.addTable(tableName);
        TableMap tmap = dbMap.getTable(tableName);
        // go over each column line
        String[] fkeyLines = null;
        // loop over all the data in the create statement
        for (Enumeration e = columndata.elements() ; e.hasMoreElements() ;)
        {
            // grab a line
            String line = (String)e.nextElement();
            if (line == null)
                continue;
            else
                line = line.toUpperCase(); // make it uppercase
                
            // make sure the line doesn't start with any of these (ie: column creation)
            // order of the if tree is important
            if (line.indexOf("PRIMARY KEY") == -1 && line.indexOf("FOREIGN KEY") == -1 && 
                line.indexOf("REFERENCES ") == -1)
            {
                StringTokenizer st = new StringTokenizer(line," ");
                String columnName = st.nextToken();
                String columnType = st.nextToken();
                // strip comma off INTEGER,
                if (columnType.endsWith(","))
                    columnType = columnType.substring(0,columnType.length()-1);
                // strip (3) off INTEGER(3)
                if (columnType.endsWith(")"))
                    columnType = columnType.substring(0,columnType.indexOf("("));

                //SAX2MapParser.columnMaker(tmap, columnName.trim(), columnType.trim());
            }
            // the primary key lines
            else if (line.indexOf("PRIMARY KEY") != -1)
            {
                // grab the column names and get the columnmap and mark it as primary
                String[] keys = parsePrimaryKey(line);
                for (int i=0; i<keys.length;i++ )
                {
                    ColumnMap cmap = tmap.getColumn(keys[i]);
                    if (cmap == null )
                        continue;
                    cmap.setPrimaryKey(true);
                }
            }
            // the foreign key lines
            else if (line.indexOf("FOREIGN KEY") != -1)
            {
                // grab the column names into a String[] for later 
                // processing in the References else if below.
                fkeyLines = parsePrimaryKey(line);
            }
            // the references lines
            else if (line.indexOf("REFERENCES ") != -1 && fkeyLines != null && fkeyLines.length > 0)
            {
                // go over the foreign keys and then make the references associations
                for (int i=0; i<fkeyLines.length;i++ )
                {
                    ColumnMap cmap = tmap.getColumn(fkeyLines[i]);
                    if (cmap == null )
                        continue;
                    String tblName = parseReferencesGetTableName(line);
                    cmap.setForeignKey(tblName,fkeyLines[i]);
                }
                fkeyLines = null;
            }
        }
    }
    /**
        Input: PRIMARY KEY (FOO,BAR),
        <br>
        Output: FOO    BAR
    */
    static final String[] parsePrimaryKey(String line)
    {
        // get FOO,BAR
        String foo = getForeignKeyColumnName(line);
        StringTokenizer st = new StringTokenizer(foo,",");
        String[] tokens = new String[st.countTokens()];
        int counter = 0;
        // get FOO and then BAR into a String[]
        while (st.hasMoreTokens())
        {
            tokens[counter] = st.nextToken();
            counter++;
        }
        return tokens;
    }
    /**
        Input: REFERENCES TABLE_NAME(COLUMN_NAME),
        <br>
        Output: TABLE_NAME    COLUMN_NAME
    */
    static final String parseReferencesGetTableName(String refline)
    {
        // skip over any pre-white space to the REFERENCES text...then skip over that text...
        return refline.substring (refline.indexOf("REFERENCES") + "REFERENCES ".length(), refline.indexOf("("));
    }
    /**
        Input: FOREIGN KEY (FOO,BAR)
        <br>
        Output: FOO,BAR
    */
    static final String getForeignKeyColumnName(String fkeyline)
    {
        return fkeyline.substring (fkeyline.indexOf("(")+1,fkeyline.indexOf(")"));
    }
    /**
        parses out the table name from the CREATE TABLE statement
    */
    static final String getTableName(String line)
    {
        String tmp = null;
        StringTokenizer st = new StringTokenizer(line," ");
        while (st.hasMoreTokens())
        {
            String token = st.nextToken();
            if (! token.equalsIgnoreCase("TABLE"))
            {
                // case for a CREATE TABLE FOO( and remove the (
                if (token.endsWith("("))
                {
                    tmp = token.substring(0,token.length()-1);
                }
                else
                {
                    tmp = token;
                }
            }
        }
        return tmp;
    }
}
