package org.apache.turbine.util;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */


// Java Core Classes
import java.sql.*;
import java.util.*;
import java.io.IOException;

// Java Servlet Classes
import javax.servlet.*;
import javax.servlet.http.*;

// Turbine Utility Classes
import org.apache.turbine.util.db.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.user.User;
import org.apache.turbine.services.resources.TurbineResources;

// ECS Classes
import org.apache.ecs.*;
import org.apache.ecs.html.*;

//support for ServletResponse.getWriter
import java.io.PrintWriter;

/**
    RunData is an object that is passed within Turbine. This provides 
    the threading mechanism for the entire system because 
    multiple requests can potentially come in at the same time.
    Thus, there is only one RunData object for each request 
    that is being serviced.
    <P>
    This object caches a lot of frequently used data such as 
    a database connection, the queryTable and other important 
    information.

    @author Jon S. Stevens <a href="mailto:jon@clearink.com">jon@clearink.com</a>
    @version $Id: RunData.java,v 1.1 2000/08/30 21:08:48 bschneid Exp $        
*/
public class RunData
{
    /** Servlet request interfaces */
    public HttpServletRequest req = null;
    /** Servlet response interfaces */
    public HttpServletResponse res = null;
    /** Servlet Session information */
    public HttpSession session = null;
    
    /**
      Servlet context information.  Note that this is from the "Turbine" 
      Servlet context.
    */
    public ServletContext servletContext = null;
    
    /** The servlet configuration */
    private ServletConfig config = null;

    /** JNDI Contexts */
    public Hashtable jndiContexts = null;

    /** the HTTP content type to return */
    public String contentType = "text/html";
    /** the HTTP charset */
    public String charSet = null;
    /** the HTTP status code to return */
    public int statusCode = HttpServletResponse.SC_OK;
    /** if this is set, also set the status code to 302 */
    public String redirectURI = null;
    
    /** this contains a reference to the GET/POST data */
    public ParameterParser parameters = null;
    
    /** cached screen name to execute for this request */
    private String screen = null;
    /** cached action name to execute for this request */
    private String action = null;

    /** the User object */
    public User user = null;
    /** the AccessControl object */
    public AccessControlList acl = null;

    /** This creates a ECS Document */
    private Document page = null;
    /** determines if there is information in the document or not */
    private boolean pageSet = false;
    /** cache the output stream because it can be 
    used in many different places */
    private PrintWriter out = null;
    /** determines if there is information in the outputstream or not */
    private boolean outSet = false;
    
    /** This is what will build the <title></title> of the document */
    private String title = null;
    /** this is the layout that the page will use to render the screen */
    private String layout = null;
    /** this is where output messages from actions should go */
    private StringElement message = null;
    /** this is a dedicated message class where output messages from actions should go */
    private FormMessages messages = null;
    /** this is a vector to hold critical system errors */
    private Vector errors = new Vector();
   
    public String stackTrace = null;
    public Throwable stackTraceException = null;

    /** Holds ServerData (basic properties) about this RunData object */
    private ServerData serverData = null;

    /** Information used by a Template system (such as WebMacro/Freemarker). **/
    private TemplateInfo templateInfo = null;

    /**
        Put things here and they will be shown on the
        default Error screen. This is great for debugging 
        variable values when an exception is thrown.
    */
    public Hashtable varDebug = new Hashtable();
    
    /**
    @see getRemoteAddr
    */
    private String remoteAddr = null;

    /**
    @see getRemoteHost
    */
    private String remoteHost = null;

    /**
    @see getUserAgent
    */
    private String userAgent = null;

    /** 
        the AccessControl object 
    */
    public AccessControlList getACL()
    {
        return acl;
    }
    /** 
        the AccessControl object 
    */
    public void setACL(AccessControlList acl)
    {
        this.acl = acl;
    }
    /**
        Gets the page
    */
    public Document getPage()
    {
        pageSet = true;
        if ( this.page == null )
            this.page = new Document();
        return this.page;
    }

    /**
        Gets the parameters.
    */
    public ParameterParser getParameters()
    {
        return parameters; 
    }
    
    /**
        Gets the PrintWriter. First time calling this 
        will set the PrintWriter via res.getWriter().
    */
    public PrintWriter getOut()
        throws IOException
    {
        // check to see if null first
        if ( this.out == null )
            setOut(res.getWriter());
        pageSet = false;
        outSet = true;
        return this.out;
    }
    /**
        Sets the PrintWriter
    */
    public void setOut(PrintWriter out)
    {
        this.out = out;
    }
    
    /**
        Sets the parameters.
    */
    public void setParameters(ParameterParser params)
    {
        this.parameters = params;
    }
    
    /**
        Check to see if page is set
    */
    public boolean isPageSet()
    {
        return pageSet;
    }
    
    /**
        Check to see if out is set
    */
    public boolean isOutSet()
    {
        return outSet;
    }
    
    /**
        Adds the string to message.  If message has prior messages from other
        Actions or Screens, this method can be used to chain them.
    */
    public void addMessage(String msg)
    {
        addMessage(new StringElement(msg));
    }        
    /**
        Adds the ECS Element to message.  If message has prior messages from other
        Actions or Screens, this method can be used to chain them.
    */    
    public void addMessage(Element msg)
    {
        if (msg != null)
        {
            if ( message != null )
            {
                message.addElement(msg);
            }
            else
            {
                message = new StringElement(msg);
            }
        }
    }
    /**
        Get the ACTION= value. It returns an empty string
        if null so that it is easy to do conditionals on it based 
        on the equalsIgnoreCase() method.
        
        @returns empty string if null
    */
    public String getAction()
    {
        if ( ! hasAction() )
            return "";
        return this.action;
    }
    /**
        If the Layout has not been defined by the Screen
        then set the layout to be "DefaultLayout". The Screen 
        object can also override this method to provide intelligent 
        determination of the Layout to execute. You can also 
        define that logic here as well if you want it to apply 
        on a global scale. For example, if you wanted to allow 
        someone to define Layout "preferences" where they could 
        dynamicially change the Layout for the entire site.
    */
    public String getLayout()
    {
        if ( this.layout == null )
            return TurbineResources.getString("layout.default", "DefaultLayout");
        return this.layout;
    }
    /**
        The results of an Action or other message to be displayed 
        should be placed here.
    */
    public String getMessage()
    {
        if (this.message == null) return null;
        return this.message.toString();
    }
    /**
        Returns a FormMessages object where all the messages to the user
        should be stored.
    */
    public FormMessages getMessages()
    {
        if (this.messages == null) return null;
        return this.messages;
    }
    /**
        Returns an array of system errors
    */
    public SystemError[] getSystemErrors()
    {
        // SystemError[] result = new SystemError[errors.size()];
        // errors.copyInto(result);
        SystemError[] result = toSystemErrorArray(errors);
        return result;
    }
    
    /**
     * utility to copy Vector of known InternetAddress objects into an array.
     * This method is here to simulate a method available in jdk1.2 Vector
     */
    private SystemError[] toSystemErrorArray(Vector v)
    {
        int size = v.size();
        SystemError[] se = new SystemError[size];
        for (int i=0; i<size; i++)
        {
            se[i] = (SystemError)v.elementAt(i);
        }
        return se;
    }

    
    /**
        The results of an Action or other message to be displayed 
        should be placed here.
    */
    public StringElement getMessageAsHTML()
    {
        return this.message;
    }
    /**
        The screen to execute.
    */
    public String getScreen()
    {
        if ( ! hasScreen() )
            return "";
        return this.screen;
    }
    /** gets JNDI Contexts */
    public Hashtable getJNDIContexts()
    {
        if ( jndiContexts == null )
            jndiContexts = new Hashtable();            
        return jndiContexts;
    }  
    /** sets JNDI Contexts */
    public void setJNDIContexts(Hashtable contexts)
    {
        this.jndiContexts = contexts;
    }  
    /** gets the cached script name that is stored in the ServerData object */
    public String getScriptName()
    {
        return getServerData().getScriptName();
    }
    /** sets the cached script name that is stored in the ServerData object */
    public void setScriptName(String sn)
    {
        getServerData().setScriptName(sn);
    }
    /** gets the cached serverName that is stored in the ServerData object */
    public String getServerName()
    {
        return getServerData().getServerName();
    }
    /** sets the cached serverName that is stored in the ServerData object */
    public void setServerName(String sn)
    {
        getServerData().setServerName(sn);
    }
    /** gets the cached serverPort that is stored in the ServerData object */
    public int getServerPort()
    {
        return getServerData().getServerPort();
    }
    /** sets the cached serverPort that is stored in the ServerData object */
    public void setServerPort(int port)
    {
        getServerData().setServerPort(port);
    }
    /** gets the cached serverScheme that is stored in the ServerData object */
    public String getServerScheme()
    {
        return getServerData().getServerScheme();
    }
    /** sets the cached serverScheme that is stored in the ServerData object */
    public void setServerScheme(String ss)
    {
        getServerData().setServerScheme(ss);
    }
    /** gets the Servlet Session information */
    public HttpSession getSession()
    {
        return session;
    }
    /** sets the Servlet Session information */
    public void setSession(HttpSession sess)
    {
        this.session = sess;
    }
    /**
        get the charset. if it has not already been defined with setCharSet(), 
        then attempt to get it from the TurbineResources.properties file under 
        the property name of "locale.default.charset" 
    */
    public String getCharSet()
    {
        if (this.charSet == null)
            return TurbineResources.getString("locale.default.charset", null);
        return this.charSet;
    }
    /** set the charset */
    public void setCharSet(String lcharset)
    {
        this.charSet = lcharset;	
    }
    /**
        gets the HTTP content type to return
    */
    public String getContentType()
    {
        if ( this.contentType == null )
            return "";
        String tmpCharset = getCharSet();
        if (tmpCharset == null)
        {
            return contentType;
        }
        else
        {
            return contentType + "; charset=" + tmpCharset;
        }
    }
    /** sets the HTTP content type to return */
    public void setContentType(String ct)
    {
        this.contentType = ct;
    }
    /** gets the HTTP status code to return */
    public int getStatusCode()
    {
        return statusCode;
    }
    /** sets the HTTP status code to return */
    public void setStatusCode(int sc)
    {
        this.statusCode = sc;
    }
    /**
        get the redirect URI. 
        if this is set, also make sure to set the status code to 302 
        @returns empty string if null
    */
    public String getRedirectURI()
    {
        if ( this.redirectURI == null )
            return "";
        return redirectURI;
    }
    /** 
        sets the redirect uri. 
        if this is set, also make sure to set the status code to 302     
    */
    public void setRedirectURI(String ruri)
    {
        this.redirectURI = ruri;
    }    
    /**
        The title of the page.
    */
    public String getTitle()
    {
        if ( this.title == null )
            return "";
        return this.title;
    }
    /**
        Return the user object.
    */
    public User getUser()
    {
        return this.user;
    }
    /**
        Set the user object
    */
    public void setUser(User user)
    {
        this.user = user;
    }
    /**
        Whether or not an action has been defined
    */
    public boolean hasAction()
    {
        if ( this.action == null || this.action.length() == 0 
            || this.action.equalsIgnoreCase("null") )
            return false;
        return true;
    }
    /**
        Whether or not a message has been defined.
    */
    public boolean hasMessage()
    {
        if ( this.message == null || this.message.toString().length() == 0 )
            return false;
        return true;
    }
    /**
        Whether or not a screen has been defined.
    */
    public boolean hasScreen()
    {
        if ( this.screen == null || this.screen.length() == 0 )
            return false;
        return true;
    }
    /**
        Set the action for the request.
    */
    public void setAction (String action )
    {
        this.action = action;
    }
    /**
        set the layout for the request.
    */
    public void setLayout ( String lay )
    {
        this.layout = lay;
    }
    /**
        set the message for the request.    
    */
    public void setMessage (String msg )
    {
        this.message = new StringElement(msg);
    }
    /**
        set the FormMessages object for the request.    
    */
    public void setMessages (FormMessages msgs)
    {
        this.messages = msgs;
    }
    /**
        add critical system error.
    */
    public void setSystemError (SystemError err)
    {
        this.errors.addElement( err );
    }
    /**
        set the message for the request.    
    */
    public void setMessage (Element msg )
    {
        this.message = new StringElement(msg);
    }
    /**
     *  unset the message for the request
     *  @author Bernie Hoeneisen <a href="mailto:bhoeneis@ee.ethz.ch">bhoeneis@ee.ethz.ch</a>
     */
    public void unsetMessage ( )
    {
        this.message = null;
    }
    /**
        set the screen for the request
    */
    public void setScreen (String screen )
    {
        this.screen = screen;
    }
    /**
        set the title for the request
    */
    public void setTitle (String title )
    {
        this.title = title;
    }
    /**
       See if a User object exists in this session.
    */
    public boolean userExists()
    {
        user = getUserFromSession();
        if ( user == null )
            return false;
        
        return true;
    }
    /**
       Attempts to get the User object from the session. If 
       it does not exist, it returns null.
    */
    public User getUserFromSession()
    {
        return getUserFromSession(session);
    }
    /**
       Attempts to get the User object from the session. If 
       it does not exist, it returns null.
    */
    public static User getUserFromSession(HttpSession session)
    {
        try
        {
            return (User) session.getValue(User.SESSION_KEY);
        }
        catch ( ClassCastException e )
        {
            return null;
        }
    }
    /**
       Allows one to invalidate the user in the session.
    */
    public boolean removeUserFromSession()
    {
        return removeUserFromSession(session);
    }
    /**
       Allows one to invalidate the user in the session.
    */
    public static boolean removeUserFromSession(HttpSession session)
    {
        try
        {
            session.removeValue(User.SESSION_KEY);
        }
        catch ( Exception e )
        {
            return false;
        }
        return true;
    }
    /**
       Pulls a User object from the session and increments the 
       access counter and sets the last access date for the object.
    */
    public void populate()
    {
        user = getUserFromSession();
        
        if ( user != null )
        {
            user.setLastAccessDate();
            user.incrementAccessCounter();
            user.incrementAccessCounterForSession();
        }
    }
    /**
       Saves a User object into the session.
    */
    public void save()
    {
        session.putValue(User.SESSION_KEY, (Object) user );
    }
    /**
       gets the HttpServletRequest object
    */
    public HttpServletRequest getRequest()
    {
        return this.req;
    }
    /**
        gets the HttpServletResponse object
    */
    public HttpServletResponse getResponse()
    {
        return this.res;
    }
    /**
       sets the HttpServletRequest object
    */
    public void setRequest(HttpServletRequest req)
    {
        this.req = req;
    }
    /**
       sets the HttpServletResponse object
    */
    public void setResponse(HttpServletResponse res)
    {
        this.res = res;
    }
    
    /**
        Set the Servlet Configuration used during servlet init.
    */    
    public void setServletConfig (ServletConfig config)
    {
        this.config = config;
        if (config == null)
        {
            this.servletContext = null;
        }
        else
        {    
            this.servletContext = config.getServletContext();
        }
    }

    /**
        Get the Servlet Configuration used during servlet init.
    */    
    public ServletConfig getServletConfig()
    {
        return this.config;
    }

    /**
        Get the Servlet Configuration used during servlet init.
    */    
    public ServletContext getServletContext()
    {
        return this.servletContext;
    }

    /**
        Get the ServerData used by this RunData
    */
    public ServerData getServerData() 
    {
        return this.serverData;
    }
    
    /**
        Set the ServerData used by this RunData
    */
    public void setServerData( ServerData serverData ) 
    {
        this.serverData = serverData;
    }

    /**
     * Return the TemplateInfo object. Create a
     * new one if needed.
     * @return TemplateInfo
     */
    public TemplateInfo getTemplateInfo()
    {
        if ( templateInfo == null )
        {
            templateInfo = new TemplateInfo( this );
            return templateInfo;
        }
        else
        {
            return templateInfo;
        }
    }

    /**
    Returns the Internet Protocol (IP) address of the client that sent the
    request
    */
    public String getRemoteAddr() 
    {
        if ( this.remoteAddr == null ) 
        {
            this.remoteAddr = this.getRequest().getRemoteAddr();
        }
        
        return this.remoteAddr;
    }

    /**
    Returns the fully qualified name of the client that sent the request.    
    */
    public String getRemoteHost() 
    {
        if ( this.remoteHost == null ) 
        {
            this.remoteHost = this.getRequest().getRemoteHost();
        }
        
        return this.remoteHost;
    }

    /**
    Get the user agent for this RunData
    */
    public String getUserAgent() 
    {
        if ( this.userAgent == null ) 
        {
            this.userAgent = this.getRequest().getHeader( "User-agent" );
        }
         
        return this.userAgent;
    }
}
