package org.apache.turbine.util.db;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.io.*;

// Turbine Classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.pool.DBBroker;
import org.apache.turbine.util.db.map.*;
import java.math.BigDecimal;

/**
 * This is a utility class that is used for retrieving different
 * types of values from a hashtable based on a simple name string.
 * This class is meant to minimize the amount of casting that needs
 * to be done when working with Hashtables.
 *
 * NOTE: other methods will be added as needed and as time permits.
 *
 * @author Frank Y. Kim
 * @author John D. McNally
 * @author Brett McLaughlin
 */
public class Criteria extends Hashtable // extends Vector
{

    public static final String EQUAL = "=";
    public static final String NOT_EQUAL = "<>";
    public static final String ALT_NOT_EQUAL = "!=";
    public static final String GREATER_THAN = ">";
    public static final String LESS_THAN = "<";
    public static final String GREATER_EQUAL = ">=";
    public static final String LESS_EQUAL = "<=";
    public static final String LIKE = " LIKE ";
//    public static final String STRING_EQUAL = "='";
    public static final String CUSTOM = "CUSTOM";
//    public static final String FOREIGN_KEY = "fk";
    public static final String DISTINCT = "DISTINCT ";
    public static final String IN = " IN ";
    public static final String NOT_IN = " NOT IN ";
    public static final String ALL = "ALL ";

    private boolean ignoreCase = false;
    private boolean singleRecord = false;
    private boolean cascade = false;
    private StringStackBuffer selectModifiers = new StringStackBuffer();
    private StringStackBuffer selectColumns = new StringStackBuffer();
    private StringStackBuffer orderByColumns = new StringStackBuffer();
    /** the name of the database */
    private String dbName = DBBroker.DEFAULT;
    /** the name of the MapBuilder */
    private String dbMap = DBBroker.DEFAULT;
    /** to limit the number of rows to return.  -1 means return all rows. */
    private int limit = -1;
    /** to start the results at a row other than the first one */
    private int offset = 0;

//    public static final String CUSTOM = "true";

    public Criteria()
    {
        super(10);
    }
    public Criteria(int initialCapacity)
    {
        super(initialCapacity);
    }
    public Criteria(String dbName)
    {
        super(10);
        this.dbName = dbName;
    }
    public Criteria(String dbName, int initialCapacity)
    {
        super(initialCapacity);
        this.dbName = dbName;
    }

    /**
    * Does this Criteria Object contain the specified key?
    * @param table the name of the table
    * @param column the name of the column
    */
    public boolean containsKey( String table, String column )
    {
        return containsKey(table + "." + column);
    }

    /**
     * Convenience method to return value as a boolean
     *
     * @param name - String name of key
     * @return boolean - value of object at key
     */
    public boolean getBoolean(String name)
    {
        return ((Boolean)getCObject(name).getValue()).booleanValue();
    }

    /**
     * Convenience method to return value as a boolean
     *
     * @param table - String name of key
     * @param column - String name of key
     * @return boolean - value of object at key
     */
    public boolean getBoolean(String table, String column)
    {
        return getBoolean( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Returns true if any of the tables in the criteria contain an Object
     * ciolumn.
     */
    public boolean containsObjectColumn() throws Exception
    {
        return containsObjectColumn(dbMap);
    }

    /**
     * Returns true if any of the tables in the criteria contain an Object
     * ciolumn.
     */
    public boolean containsObjectColumn(String databaseMapName) throws Exception
    {
        DatabaseMap map = DBBroker.getInstance().getDatabaseMap(databaseMapName);
        StringStackBuffer tables = new StringStackBuffer();
        for (Enumeration e = super.elements(); e.hasMoreElements(); )
        {
            CriteriaObject co = (CriteriaObject)e.nextElement();
            String tableName = co.getTable();
            if (!tables.contains(tableName))
            {
                if (map.getTable(tableName).containsObjectColumn())
                  return true;
                tables.add(tableName);
            }
        }
        return false;
    }

    /**
     * Method to return criteria related to columns in a table
     *
     * @param table - String name of key
     * @param column - String name of key
     * @return CriteriaObject - value of object at key
     */
    private CriteriaObject getCObject(String name)
    {
        return (CriteriaObject)super.get(name);
    }

    /**
     * Method to return criteria related to column in a table
     *
     * @param table - String name of key
     * @param column - String name of key
     * @return CriteriaObject - value of object at key
     */
    private CriteriaObject getCObject(String table, String column)
    {
        return getCObject( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * method to return a String table name
     *
     * @param name - String name of key
     * @return String - value of object at key
     */
     public String getColumnName(String name)
    {
        return getCObject(name).column;
    }


    /**
     * Method to return a comparison String
     *
     * @param key - String name of key
     * @return String - value of object at key
     */
    public String getComparison(String key)
    {
        return getCObject(key).comparison;
    }
    /**
     * Method to return a comparison String
     *
     * @param name - String name of key
     * @return String - value of object at key
     */
    public String getComparison(String table, String column)
    {
        return getComparison( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * Convenience method to return a Date
     *
     * @param name - String name of key
     * @return java.util.Date - value of object at key
     */
    public java.util.Date getDate(String name)
    {
        return (java.util.Date)getCObject(name).getValue();
    }
    /**
     * Convenience method to return a Date
     *
     * @param name - String name of key
     * @return java.util.Date - value of object at key
     */
    public java.util.Date getDate(String table, String column)
    {
        return getDate( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
    * Get the database name for this criteria object
    * By default, this is DBBroker.DEFAULT
    */
    public String getDbName()
    {
        if ( this.dbName == null )
            return DBBroker.DEFAULT;
        return dbName;
    }
    /**
    * Get the database name for this criteria object
    * By default, this is DBBroker.DEFAULT
    */
    public void setDbName(String name)
    {
        this.dbName = name;
    }
    /**
        gets the DatabaseMap name. By default, this is DBBroker.DEFAULT
    */
    public String getMapName()
    {
        if ( this.dbMap == null )
            return DBBroker.DEFAULT;
        return dbMap;
    }
    /**
        sets the DatabaseMap name.
    */
    public void setMapName(String map)
    {
        this.dbMap = map;
    }
    /**
     * Convenience method to return a double
     *
     * @param name - String name of key
     * @return double - value of object at key
     */
    public double getDouble(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Double((String)obj).doubleValue();
        return ((Double) obj).doubleValue();
    }
    /**
     * Convenience method to return a double
     *
     * @param name - String name of key
     * @return double - value of object at key
     */
    public double getDouble(String table, String column)
    {
        return getDouble( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * Convenience method to return a float
     *
     * @param name - String name of key
     * @return float - value of object at key
     */
    public float getFloat(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Float((String)obj).floatValue();
        return ((Float) obj).floatValue();
    }
    /**
     * Convenience method to return a float
     *
     * @param name - String name of key
     * @return float - value of object at key
     */
    public float getFloat(String table, String column)
    {
        return getFloat( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * Convenience method to return a Integer
     *
     * @param name - String name of key
     * @return Integer - value of object at key
     */
    public Integer getInteger(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Integer((String)obj);
        return ((Integer)obj);
    }
    /**
     * Convenience method to return a Integer
     *
     * @param name - String name of key
     * @return Integer - value of object at key
     */
    public Integer getInteger(String table, String column)
    {
        return getInteger( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * Convenience method to return a int
     *
     * @param name - String name of key
     * @return int - value of object at key
     */
    public int getInt(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Integer((String)obj).intValue();
        return ((Integer)obj).intValue();
    }
    /**
     * Convenience method to return a int
     *
     * @param name - String name of key
     * @return int - value of object at key
     */
    public int getInt(String table, String column)
    {
        return getInt( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    
    /**
     * Convenience method to return a BigDecimal
     *
     * @param name - String name of key
     * @return BigDecimal - value of object at key
     */
    public BigDecimal getBigDecimal(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new BigDecimal((String)obj);
        return (BigDecimal)obj;
    }
    
    /**
     * Convenience method to return a long
     *
     * @param name - String name of key
     * @return long - value of object at key
     */
    public long getLong(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Long((String)obj).longValue();
        return ((Long) obj).longValue();
    }
    /**
     * Convenience method to return a long
     *
     * @param name - String name of key
     * @return long - value of object at key
     */
    public long getLong(String table, String column)
    {
        return getLong( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * Convenience method to return a String
     *
     * @param name - String name of key
     * @return String - value of object at key
     */
    public String getString(String name)
    {
        return (String) getCObject(name).getValue();
    }
    /**
     * Convenience method to return a String
     *
     * @param name - String name of key
     * @return String - value of object at key
     */
    public String getString(String table, String column)
    {
        return getString( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * method to return a String table name
     *
     * @param name - String name of key
     * @return String - value of object at key
     */
     public String getTableName(String name)
    {
        return getCObject(name).getTable();
    }

    /**
     * Convenience method to return a vector
     *
     * @param name - String name of key
     * @return Vector - value of object at key
     */
    public Vector getVector(String name)
    {
        return (Vector) getCObject(name).getValue();
    }
    /**
     * Convenience method to return a vector
     *
     * @param name - String name of key
     * @return Vector - value of object at key
     */
    public Vector getVector(String table, String column)
    {
        return getVector( new StringBuffer(table).append('.').append(column)
            .toString() );
    }
    /**
     * method to return the value that was added to Criteria
     *
     * @param name - String name of key
     * @return Object - value of object at key
     */
    public Object getValue( String name )
    {
        return getCObject(name).getValue();
    }
    /**
     * method to return the value that was added to Criteria
     *
     * @param table - String name of key
     * @param column - String name of key
     * @return Object - value of object at key
     */
    public Object getValue( String table, String column )
    {
        return getValue( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
    * Overrides Hashtable get, so that the value placed in the CriteriaObject is
    * returned instead of the CriteriaObject
    */
    public Object get( Object key )
    {
            return getValue( (String)key );
    }

    public Object put( Object key, Object value )
    {
        if ( value instanceof CriteriaObject )
        {
            return super.put(key, value);
        }
        return super.put( key, new CriteriaObject((String)key, value, EQUAL) );
    }

    /**
        This method is a slightly faster alternative to using the method put().
        It also allows code that looks like this:
        <p>
        <code>
        Criteria crit = new Criteria().add ( &quot;name&quot;, &quot;value&quot; );
        </code>
    */
    public Criteria add ( String key, Object value )
    {
        add(key, value, EQUAL);
        return this;
    }
    /**
        This method is a slightly faster alternative to using the method put().
        It also allows code that looks like this:
        <p>
        <code>
        Criteria crit = new Criteria().add ( &quot;name&quot;, &quot;value&quot; );
        </code>
    */
    public Criteria add ( String key, Object value, String comparison )
    {
        super.put( key, new CriteriaObject(key, value, comparison) );
        return this;
    }
    /**
        This method is a slightly faster alternative to using the method put().
        It also allows code that looks like this:
        <p>
        <code>
        Criteria crit = new Criteria().add ( &quot;name&quot;, &quot;value&quot; );
        </code>
    */
    public Criteria add ( String table, String column, Object value )
    {
        add(table, column, value, EQUAL);
        return this;
    }
    /**
        This method is a slightly faster alternative to using the method put().
        It also allows code that looks like this:
        <p>
        <code>
        Criteria crit = new Criteria().add ( &quot;name&quot;, &quot;value&quot; );
        </code>
    */
    public Criteria add ( String table, String column, Object value, String comparison )
    {
        super.put( new StringBuffer(table).append('.').append(column).toString(),
            new CriteriaObject(table, column, value, comparison) );
        return this;
    }
    /**
     * Convenience method to add a boolean to Criteria
     */
    public Criteria add ( String key, boolean value )
    {
        add( key, new Boolean(value) );
        return this;
    }
    /**
     * Convenience method to add a boolean to Criteria
     */
    public Criteria add ( String key, boolean value, String comparison )
    {
        add( key, new Boolean(value), comparison );
        return this;
    }
    /**
     * Convenience method to add an int to Criteria
     */
    public Criteria add ( String key, int value )
    {
        add( key, new Integer(value) );
        return this;
    }
    /**
     * Convenience method to add an int to Criteria
     */
    public Criteria add ( String key, int value, String comparison )
    {
        add( key, new Integer(value), comparison );
        return this;
    }
    /*
     * Convenience method to add a long to Criteria
     *
     * @param key - String value to use as key
     * @param value - long to add
     * @return Criteria - modified Criteria object
     */
    public Criteria add(String key, long value)
    {
        add(key, new Long(value));
        return this;
    }
    /*
     * Convenience method to add a long to Criteria
     *
     * @param key - String value to use as key
     * @param value - long to add
     * @return Criteria - modified Criteria object
     */
    public Criteria add(String key, long value, String comparison)
    {
        add(key, new Long(value), comparison);
        return this;
    }
    /**
     * Convenience method to add a float to Criteria
     *
     * @param key - String value to use as key
     * @param value - float to add
     * @return Criteria - modified Criteria object
     */
    public Criteria add(String key, float value)
    {
        add(key, new Float(value));
        return this;
    }
    /**
     * Convenience method to add a float to Criteria
     *
     * @param key - String value to use as key
     * @param value - float to add
     * @return Criteria - modified Criteria object
     */
    public Criteria add(String key, float value, String comparison)
    {
        add(key, new Float(value), comparison);
        return this;
    }
    /**
     * Convenience method to add a double to Criteria
     *
     * @param key - String value to use as key
     * @param value - double to add
     * @return Criteria - modified Criteria object
     */
    public Criteria add(String key, double value)
    {
        add(key, new Double(value));
        return this;
    }
    /**
     * Convenience method to add a double to Criteria
     *
     * @param key - String value to use as key
     * @param value - double to add
     * @return Criteria - modified Criteria object
     */
    public Criteria add(String key, double value, String comparison)
    {
        add(key, new Double(value), comparison);
        return this;
    }
    /**
     * This method lets you put a Date object into the criteria object
     * by specifying the year, month, and date.
     *
     */
    public Criteria addTime( String key, int year, int month, int date )
    {
        add( key, new GregorianCalendar(year, month, date) );
        return this;
    }
    /**
     * This method lets you put a Date object into the criteria object
     * by specifying the year, month, and date.
     */
    public Criteria addTime( String key, int year, int month, int date, String comparison)
    {
        add( key, new GregorianCalendar(year, month, date), comparison );
        return this;
    }
    /**
        This is the way that you should add a join of two tables.
        For example:
        <p>
        AND PROJECT.PROJECT_ID=FOO.PROJECT_ID
        <p>
        left = PROJECT.PROJECT_ID
        right = FOO.PROJECT_ID
    */
    public Criteria addJoin( String left, String right)
    {
        add(left, (Object)(left + "=" + right), Criteria.CUSTOM);
        return this;
    }
    /**
        Adds an 'IN' clause with the criteria supplied as an Object array.
        For example:
        <p>
        FOO.NAME IN ('FOO', 'BAR', 'ZOW')
        <p>
        where 'values' contains three objects that evaluate to the respective
        strings above when .toString() is called
    */
    public Criteria addIn(String key, Object[] values)
    {
        add(key, (Object)values, Criteria.IN);
        return this;
    }
    /**
       Adds an 'IN' clause with the criteria supplied as an int array.
       For example:
       <p>
       FOO.ID IN ('2', '3', '7')
       <p>
       where 'values' contains those three integers.
    */
    public Criteria addIn(String key, int[] values)
    {
        add(key, (Object)values, Criteria.IN);
        return this;
    }
    /**
        Adds an 'IN' clause with the criteria supplied as a Vector.
        For example:
        <p>
        FOO.NAME IN ('FOO', 'BAR', 'ZOW')
        <p>
        where 'values' contains three objects that evaluate to the respective
        strings above when .toString() is called
    */
    public Criteria addIn(String key, Vector values)
    {
        add(key, (Object)values, Criteria.IN);
        return this;
    }
    /**
        Adds a 'NOT IN' clause with the criteria supplied as an Object array.
        For example:
        <p>
        FOO.NAME NOT IN ('FOO', 'BAR', 'ZOW')
        <p>
        where 'values' contains three objects that evaluate to the respective
        strings above when .toString() is called
    */
    public Criteria addNotIn(String key, Object[] values)
    {
        add(key, (Object)values, Criteria.NOT_IN);
        return this;
    }
    /**
       Adds a 'NOT IN' clause with the criteria supplied as an int array.
       For example:
       <p>
       FOO.ID NOT IN ('2', '3', '7')
       <p>
       where 'values' contains those three integers.
    */
    public Criteria addNotIn(String key, int[] values)
    {
        add(key, (Object)values, Criteria.NOT_IN);
        return this;
    }
    /**
        Adds an 'IN' clause with the criteria supplied as a Vector.
        For example:
        <p>
        FOO.NAME IN ('FOO', 'BAR', 'ZOW')
        <p>
        where 'values' contains three objects that evaluate to the respective
        strings above when .toString() is called
    */
    public Criteria addNotIn(String key, Vector values)
    {
        add(key, (Object)values, Criteria.NOT_IN);
        return this;
    }
    /**
        Adds "ALL " to the SQL statement
    */
    public void setAll()
    {
        selectModifiers.add(ALL);
    }
    /**
        Adds "DISTINCT " to the SQL statement
    */
    public void setDistinct()
    {
        selectModifiers.add(DISTINCT);
    }
    /**
        Sets ignore case
    */
    public Criteria setIgnoreCase(boolean b)
    {
        ignoreCase = b;
        return this;
    }
    /**
        Is ignore case on or off?
    */
    public boolean isIgnoreCase()
    {
        return ignoreCase;
    }
    /**
        Set single Record?
    */
    public Criteria setSingleRecord(boolean b)
    {
        singleRecord = b;
        return this;
    }
    /**
        Is single Record?
    */
    public boolean isSingleRecord()
    {
        return singleRecord;
    }
    /**
        set Cascade?
    */
    public Criteria setCascade(boolean b)
    {
        cascade = b;
        return this;
    }
    /**
        is Cascade?
    */
    public boolean isCascade()
    {
        return cascade;
    }
    /**
        set limit
    */
    public Criteria setLimit(int limit)
    {
        this.limit = limit;
        return this;
    }
    /**
        get Limit
    */
    public int getLimit()
    {
        return limit;
    }
    /**
        set offset
    */
    public Criteria setOffset(int offset)
    {
        this.offset = offset;
        return this;
    }
    /**
        get offset
    */
    public int getOffset()
    {
        return offset;
    }
    /**
        add select column
    */
    public Criteria addSelectColumn( String name )
    {
        selectColumns.add( name );
        return this;
    }
    /**
        get select columns
    */
    public StringStackBuffer getSelectColumns()
    {
        return selectColumns;
    }
    /**
        get select modifiers
    */
    public StringStackBuffer getSelectModifiers()
    {
        return selectModifiers;
    }
    /**
        add order by column name
    */
    public Criteria addOrderByColumn( String name )
    {
        orderByColumns.add( name );
        return this;
    }
    /**
        get order by columns
    */
    public StringStackBuffer getOrderByColumns()
    {
        return orderByColumns;
    }
    /**
        remove an object from the criteria
    */
    public Object remove(String key)
    {
        Object foo = super.remove(key);
        if (foo instanceof CriteriaObject)
            return ((CriteriaObject)foo).getValue();
        return foo;
    }
    /**
        build a string representation of the Criteria
    */
    public String toString()
    {
        StringBuffer sb = new StringBuffer("Criteria:: ");
        Enumeration e = keys();
        while (e.hasMoreElements() )
        {
            String key = (String)e.nextElement();
            sb.append(key).append("<=>").append(getValue(key).toString()).append(":  ");
        }
        return sb.toString();
    }
    /**
        This is a private inner class that describes an object in the criteria.
    */
    private class CriteriaObject
    {
        /** value of the CO */
        private Object value = null;
        /** comparison value */
        private String comparison = "";
        /** table name */
        private String table = "";
        /** column name */
        private String column = "";

        /**
            Constructor
        */
        CriteriaObject( String table, String column, Object val, String comp )
        {
            this.table = table;
            this.column = column;
            value = val;
            comparison = comp;
        }
        /**
            Constructor
        */
        CriteriaObject( String tableColumn, Object val, String comp )
        {
            int dot = tableColumn.indexOf('.');
            if ( dot == -1 )
            {
                column = tableColumn;
            }
            else
            {
                table = tableColumn.substring(0, dot);
                column = tableColumn.substring(dot+1);
            }
            value = val;
            comparison = comp;
        }
        /**
            Constructor
        */
        CriteriaObject( String table, String column, Object val )
        {
            this.table = table;
            this.column = column;
            value = val;
            comparison = EQUAL;
        }
        /**
            Constructor
        */
        CriteriaObject( String tableColumn, Object val )
        {
            int dot = tableColumn.indexOf('.');
            if ( dot == -1 )
            {
                column = tableColumn;
            }
            else
            {
                table = tableColumn.substring(0, dot);
                column = tableColumn.substring(dot+1);
            }
            value = val;
            comparison = EQUAL;
        }
        /**
            get the column name
        */
        public String getColumn()
        {
            return this.column;
        }
        /**
            get the table name
        */
        public String getTable()
        {
            return this.table;
        }
        /**
            get the comparison
        */
        public String getComparison()
        {
            return this.comparison;
        }
        /**
            get the value
        */
        public Object getValue()
        {
            return this.value;
        }

    }

}
