package org.apache.turbine.util.db.map;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.util.*;

/**
 * TableMap is used to model a table in a database.
 *
 * @author John D. McNally
 */
public class TableMap
{
    /** key generation through auto-increment */
    public static final String AUTOINCREMENT = "autoincrement";
    /** key generation through sequences */
    public static final String SEQUENCE = "sequence";
    /** key generation through the IDBroker table */
    public static final String IDBROKERTABLE = "idbroker_table";
    /** keys are given by the  application */
    public static final String NONE = "none";
    
    /** the columns in the table */
    private Hashtable columns;
    /** the database this table belongs to */
    private DatabaseMap dbMap;
    /** the name of the table */
    private String tableName;
    /** the prefix on the table name */
    private String prefix;
    /** the primary key generation method */
    private String primaryKeyMethod = NONE;

    /** 
        Object to store information that is needed if the primaryKeyMethod
        is set to SEQUENCE   
    */
    private Object sequenceInfo = null;
    
    
    /**
        Constructor
        @params tableName the name of the table
        @params numberOfColumns the number of columns in the table
        @params containingDB a DatabaseMap that this table belongs to
    */
    public TableMap(String tableName, int numberOfColumns, DatabaseMap containingDB )
    {
        this.tableName = tableName;
        dbMap = containingDB;
        columns = new Hashtable( (int)(1.25*numberOfColumns) + 1 );
    }
    /**
        Constructor
        @params tableName the name of the table
        @params containingDB a DatabaseMap that this table belongs to
    */
    public TableMap(String tableName, DatabaseMap containingDB )
    {
        this.tableName = tableName;
        dbMap = containingDB;
        columns = new Hashtable( 20 );
    }
    /**
        Constructor
        @params tableName the name of the table
        @params prefix the prefix for the table name (ie: SCARAB for SCARAB_PROJECT)
        @params containingDB a DatabaseMap that this table belongs to
    */
    public TableMap(String tableName, String prefix, DatabaseMap containingDB )
    {
        this.tableName = tableName;
        this.prefix = prefix;
        dbMap = containingDB;
        columns = new Hashtable( 20 );
    }
    /**
        does this table contain the specified column?
    */
    public boolean containsColumn( ColumnMap column )
    {
        return containsColumn( column.getColumnName() );
    }
    /**
        does this table contain the specified column?
    */
    public boolean containsColumn( String name )
    {
        if ( name.indexOf('.') > 0 )
        {
            name = name.substring( name.indexOf('.')+1 );
        }
        return columns.containsKey(name);
    }
    
    /**
        get the DatabaseMap containing this TableMap
    */
    public DatabaseMap getDatabaseMap()
    {
        return dbMap;
    }
    
    /**
     * Returns true if this tableMap contains a column with object data.
     * If the type of the column is not a string, a number or a date, it is
     * assumed that it is objectdata.
     */
    public boolean containsObjectColumn()
    {
        Enumeration e = columns.elements();
        while (e.hasMoreElements())
        {
            Object theType = ((ColumnMap)e.nextElement()).getType();
            if (! ( theType instanceof String || 
                    theType instanceof Number ||
                    theType instanceof java.util.Date ) )
            {
                return true;
            }
        }
        return false;
    }
    
    /**
        get the name of the Table
    */
    public String getName()
    {
        return tableName;
    }
    /**
        table prefix name
    */
    public String getPrefix()
    {
        return this.prefix;
    }
    /**
        table prefix name
        @params prefix the prefix for the table name (ie: SCARAB for SCARAB_PROJECT)
    */
    public void setPrefix(String prefix)
    {
        this.prefix = prefix;
    }
    
    /**
        get the method used to generate primary keys for this table
    */
    public String getPrimaryKeyMethod()
    {
        return primaryKeyMethod;
    }
    
    /**
        get the information  used to generate a primary key
        using a sequence
    */
    public Object getSequenceInfo()
    {
        return sequenceInfo;
    }
    

    /**
        get a ColumnMap[] of the columns in this table
    */
    public ColumnMap[] getColumns()
    {
        ColumnMap[] tableColumns = new ColumnMap[columns.size()];
        Enumeration e = columns.elements();
        int i = 0;
        while (e.hasMoreElements())
        {
            tableColumns[i++] = (ColumnMap) e.nextElement();
        }
        return tableColumns;
    }
    /**
        get a ColumnMap for the named table
    */
    public ColumnMap getColumn( String name )
    {
        try
        {
            return (ColumnMap) columns.get( name );
        }
        catch (Exception e)
        {
        }
        return null;
    }
    /**
        add a pre-created column to this table
        it will replace any existing column
    */
    public void addColumn ( ColumnMap cmap )
    {
        columns.put ( cmap.getColumnName(), cmap );
    }
    /**
        add a column to this table of a certain type
    */
    public void addColumn( String columnName, Object type )
    {
        addColumn( columnName, type, false, null, null, 0 );
    }
    /**
        add a column to this table of a certain type and size
    */
    public void addColumn( String columnName, Object type, int size )
    {
        addColumn( columnName, type, false, null, null, size );
    }
    /**
        add a primary key column to this Table
    */
    public void addPrimaryKey( String columnName, Object type )
    {
        addColumn( columnName, type, true, null, null, 0 );
    }
    /**
        add a primary key column to this Table
    */
    public void addPrimaryKey( String columnName, Object type, int size )
    {
        addColumn( columnName, type, true, null, null, size );
    }
    /**
        add a foreign key column to the table
    */
    public void addForeignKey( String columnName, Object type, String fkTable, String fkColumn )
    {
        addColumn( columnName, type, false, fkTable, fkColumn, 0 );
    }
    /**
        add a foreign key column to the table
    */
    public void addForeignKey( String columnName, Object type, String fkTable, String fkColumn, int size )
    {
        addColumn( columnName, type, false, fkTable, fkColumn, size );
    }
    /**
        add a foreign key column to the table
    */
    public void addForeignPrimaryKey( String columnName, Object type, String fkTable, String fkColumn )
    {
        addColumn( columnName, type, true, fkTable, fkColumn, 0 );
    }
    /**
        add a foreign key column to the table
    */
    public void addForeignPrimaryKey( String columnName, Object type, String fkTable, String fkColumn, int size )
    {
        addColumn( columnName, type, true, fkTable, fkColumn, size );
    }
    /**
        add a column to the table
    */
    private void addColumn( String name, Object type, boolean pk, String fkTable, String fkColumn, int size )
    {
        // if the tablename is prefixed with the name of the column, remove it
        // ie: SCARAB_PROJECT.PROJECT_ID  remove the SCARAB_PROJECT.
        if (name.indexOf('.') > 0 && name.indexOf(getName()) != -1)
        {
            name = name.substring(getName().length()+1);
        }
        if ( fkTable != null && fkTable.length() > 0
            && fkColumn !=null && fkColumn.length() > 0 )
        {
            if (fkColumn.indexOf('.') > 0 && fkColumn.indexOf(fkTable) != -1)
            {
                fkColumn = fkColumn.substring(fkTable.length()+1);
            }
        }
        ColumnMap col = new ColumnMap(name, this);
        col.setType(type);
        col.setPrimaryKey(pk);
        col.setForeignKey(fkTable, fkColumn);
        col.setSize(size);
        columns.put( name, col );
    }


    /** 
        sets the method used to generate a key for this table.  Valid values
        are AUTOINCREMENT, SEQUENCE, IDBROKERTABLE, NONE.
    */
    public void setPrimaryKeyMethod(String method)
    {
        this.primaryKeyMethod = method;
    }

    /** 
        sets the sequence information needed to generate a key and
        also sets the primaryKeyMethod to SEQUENCE   
    */
    public void setSequenceInfo(Object sequenceInfo)
    {
        this.sequenceInfo = sequenceInfo;
        setPrimaryKeyMethod(SEQUENCE);
    }

    //------Utility methods for doing intelligent lookup of table names
    
    /**
        tell me if i have PREFIX in my string
    */
    private final boolean hasPrefix ( String data )
    {
        return ( data.indexOf(getPrefix()) != -1 );
    }
    /**
        removes the PREFIX
    */
    private final String removePrefix ( String data )
    {
        return data.substring(getPrefix().length());
    }
    /**
        removes the PREFIX
        removes the underscores and makes firstlettercaps
        
        SCARAB_FOO_BAR becomes FooBar
    */
    public final String removeUnderScores (String data)
    {
        String tmp = null;
        StringBuffer out = new StringBuffer();
        if (hasPrefix(data))
        {
            tmp = removePrefix(data);
        }
        else
        {
            tmp = data;
        }
            
        StringTokenizer st = new StringTokenizer(tmp, "_");
        while (st.hasMoreTokens())
        {
            String element = (String) st.nextElement();
            out.append ( firstLetterCaps(element)); 
        }
        return out.toString();
    }
    /**
        makes the first letter caps and the rest lowercase
    */
    private final String firstLetterCaps ( String data )
    {
        String firstLetter = data.substring(0,1).toUpperCase();
        String restLetters = data.substring(1).toLowerCase();
        return firstLetter + restLetters;
    }
}   