package org.apache.turbine.util.db.pool;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.Hashtable;

// Turbine Utility Classes
import org.apache.turbine.util.db.map.DatabaseMap;
import org.apache.turbine.util.db.map.MapBuilder;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * This class is the single point of access to connection pools.<p>
 *
 * To retrieve a Connection object from this singleton do either:<p>
 *
 * 1) Get a connection using default values specified in a
 * properties file.<br>
 * DBConnection db = DBBroker.getInstance().getConnection();<br>
 * Connection con = db.getConnection();
 * <p>
 * 2) Get a connection using the given parameters.<br>
 * DBConnection db = DBBroker.getInstance().getConnection(driver, url, username, password)<br>
 * Connection con = db.getConnection();
 * <p>
 * Call DBBroker.getInstance().releaseConnection(db) when you are done. 
 *
 * @author Frank Y. Kim 
 * @author Brett McLaughlin
 * @author Greg Ritter
 */
public class DBBroker
{
    /** The single instance of this class. */
    private static DBBroker instance = null;

    /** The various connection pools this broker contains. */
    /** Keyed by database url. */
    private static Hashtable pools = null;

    /** The DatabaseMap which contains the tables in this DB */
    private static Hashtable dbMaps = null;
    
    /** name of the default map builder and map */
    public static final String DEFAULT = "default";
    
    /**
     * This ctor is private to force clients to use
     * getInstance() to access this class.
     */
    private DBBroker() 
    {
        pools = new Hashtable();
        dbMaps = new Hashtable();
    }

    /**
     * This method returns a connection using the default values
     * specified in a properties file.
     */
    public DBConnection getConnection() throws Exception
    {
        return getConnection( TurbineResources.getString( "database." + DEFAULT + ".driver", "" )
                            , TurbineResources.getString( "database." + DEFAULT + ".url", "" )
                            , TurbineResources.getString( "database." + DEFAULT + ".username", "" )
                            , TurbineResources.getString( "database." + DEFAULT + ".password", "" ) );
    }

    /**
     * This method returns a connection using the values
     * specified in a properties file. The format of the values is this:
     *
     * database.[name].driver
     * database.[name].url
     * database.[name].username
     * database.[name].password
     */
    public DBConnection getConnection(String name) throws Exception
    {
        return getConnection( TurbineResources.getString( "database." + name + ".driver", "" )
                            , TurbineResources.getString( "database." + name + ".url", "" )
                            , TurbineResources.getString( "database." + name + ".username", "" )
                            , TurbineResources.getString( "database." + name + ".password", "" ) );
    }

    /**
     * This method returns a connection using the given parameters.
     */
     public DBConnection getConnection(String driver,
                                       String url,
                                       String username,
                                       String password) throws Exception
    {
        ConnectionPool pool;
        url = url.trim();
        
        if ( pools.containsKey(url) )
        {
            pool = (ConnectionPool) pools.get(url);
        }
        else
        {
            pool = new ConnectionPool();
            pools.put( url, pool );
        }
        return pool.getConnection(driver, url, username, password);
    }

    /**
     * Returns the default database map information.
     */
    public DatabaseMap getDatabaseMap()
        throws Exception
    {
        return getDatabaseMap(DEFAULT);
    }
    
    /**
     * Returns the database map information. Name relates to the name
     * of the connection pool to associate with the map
     */
    public DatabaseMap getDatabaseMap(String name) throws Exception
    {
        if ( name == null )
            throw new Exception ("DatabaseMap name was null!");

        if ( dbMaps.containsKey(name) )
        {
            return (DatabaseMap)dbMaps.get(name);
        }
        else
        {
            DatabaseMap dbMap = new DatabaseMap(name);
            dbMaps.put(name, dbMap);
            return dbMap;
        }
    }

    /**
     * Adds a new database map.  
     *
     * @deprecated A database map is added automatically by getDatabaseMap.
     */
    public void addDatabaseMap(String name)
    {
        dbMaps.put(name, new DatabaseMap(name) );
    }
    
    /**
        put a DatabaseMap into the cache of maps
        it is keyed off the name in the DatabaseMap.
        
        @deprecated A database map is added automatically by getDatabaseMap.
    */
    public void addDatabaseMap(DatabaseMap map)
    {
        dbMaps.put (map.getName(), map);
    }
    /**
     * Returns the database specific class.
     */
    public DB getDB() throws Exception
    {
        return getDB(DEFAULT);
    }
    /**
     * Returns the database specific class.
     */
    public DB getDB(String name) throws Exception
    {
        String url = TurbineResources.getString( "database." + name + ".url", "" ).trim();
        ConnectionPool cp = (ConnectionPool)pools.get(url);
        if ( cp == null)
        {
            DBConnection dbcon = getConnection(name);
            releaseConnection(dbcon);
            cp = (ConnectionPool)pools.get(url);
        }
        return cp.getDB();
	}

    /**
     * The method through which this class is accessed.
     *
     * @return The single instance of this class.
     */
    public static DBBroker getInstance()
    {
        if (instance == null)
        {
            synchronized (DBBroker.class)
            {
                if (instance == null)
                {
                    instance = new DBBroker();
                }
            }
        }
        return instance;
    }
    /**
        release a connection back to the database pool
    */
    public void releaseConnection(DBConnection connection) throws Exception
    {
        if ( connection != null )
        {
            String url = connection.getUrl();
            if ( url != null )
            {
                if (pools.containsKey( url ))
                {
                    ConnectionPool pool = (ConnectionPool) pools.get( url );
                    pool.releaseConnection( connection );
                }
                // else if there is no pool for this connection
                // don't do anything
            }
        }
    }    
}