package org.apache.turbine.util.mail;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.net.*;
import java.util.*;

// Java Mail API
import javax.mail.*;
import javax.mail.internet.*;

import org.apache.turbine.util.Log;

// adapted for ACS by Bill Schneider (bschneid@arsdigita.com)
import com.arsdigita.acs.Parameters;

/**
 * The base class for all email messages.  This class sets
 * the sender's email & name, receiver's email & name, subject,
 * and the sent date.  Subclasses are responsible for setting
 * the message body.
 *
 * @author Jon S. Stevens
 * @author Frank Y. Kim
 * @author Brett McLaughlin
 * @author Greg Ritter
 * @author Regis Koenig
 */
public abstract class Email
{
    /** Constants used to Email classes. */
    public static final String SENDER_EMAIL = "sender.email";
    public static final String SENDER_NAME = "sender.name";
    public static final String RECEIVER_EMAIL = "receiver.email";
    public static final String RECEIVER_NAME = "receiver.name";
    public static final String EMAIL_SUBJECT = "email.subject";
    public static final String EMAIL_BODY = "email.body";
    
    public static final String MAIL_SERVER = "mail.server";
    public static final String MAIL_HOST = "mail.host";
    public static final String MAIL_TRANSPORT_PROTOCOL = "mail.transport.protocol";
    public static final String SMTP = "SMTP";
    public static final String TEXT_HTML = "text/html";
    public static final String TEXT_PLAIN = "text/plain";
    public static final String ATTACHMENTS = "attachments";
    public static final String FILE_SERVER = "file.server";
    
    /** The email message to send.
     */
    Message message;

    /* Lists of related email adresses */
    private Vector toList;
    private Vector ccList;
    private Vector bccList;
    private Vector replyList;

    private Session getMailSession()
    {
        Properties properties = System.getProperties();
        properties.put ( MAIL_TRANSPORT_PROTOCOL, SMTP );
        properties.put ( MAIL_HOST, Parameters.get("MailHost", null, "localhost"));
        return Session.getDefaultInstance(properties, null);
    }
    
    /** Initializes the mail.
     * <p> This is the first method that should be called by
     * a sublasse in its constructor;
     */
    protected void init()
        throws MessagingException
    {
    
        // create the message
        message = new MimeMessage( getMailSession() );
    
        toList = new Vector();
        ccList = new Vector();
        bccList = new Vector();
        replyList = new Vector();

        // set the sent date
        setSentDate( new Date() );
    }

    /** Set the FROM field of the email
     */
    public Email setFrom(String email, String name )
        throws MessagingException
    {

        try          
        {
            if( (name == null) || (name.trim().equals("")) )
            {
                name = email;
            }
            message.setFrom(new InternetAddress( email, name ) );
        }
        catch( Exception e )
        {
            throw new MessagingException("cannot set from", e);
        }
        return this;
    }


    /** Add a recipient TO to the email
     */
    public Email addTo(String email, String name )
        throws MessagingException
    {

        try
        {
            if( (name == null) || (name.trim().equals("")) )
            {
                name = email;
            }
            toList.addElement( new InternetAddress( email, name ) );
        }
        catch( Exception e )
        {
            throw new MessagingException("cannot add to", e);
        }
        return this;
    }

    /** Add a recipient CC to the email
     */
    public Email addCc(String email, String name )
        throws MessagingException
    {

        try
        {
            if( (name == null) || (name.trim().equals("")) )
            {
                name = email;
            }
            ccList.addElement( new InternetAddress( email, name ) );
        }
        catch( Exception e )
        {
            throw new MessagingException("cannot add cc", e);
        }

        return this;
    }

    /** Add a blind CC repcipient to the email
     */
    public Email addBcc(String email, String name )
        throws MessagingException
    {

        try
        {
            if( (name == null) || (name.trim().equals("")) )
            {
                name = email;
            }
            bccList.addElement( new InternetAddress( email, name ) );
        }
        catch( Exception e )
        {
            throw new MessagingException("cannot add bcc", e);
        }

        return this;
    }

    /** Add a reply to address to the email
     */
    public Email addReplyTo(String email, String name )
        throws MessagingException
    {

        try
        {
            if( (name == null) || (name.trim().equals("")) )
            {
                name = email;
            }
            replyList.addElement( new InternetAddress( email, name ) );
        }
        catch( Exception e )
        {
            throw new MessagingException("cannot add replyTo", e);
        }
        return this;
    }

    /** Set the email subject
     */
    public Email setSubject( String subject )
        throws MessagingException
    {
        message.setSubject(subject);
        return this;
    }

    /** Set the sent date field
     */
    public Email setSentDate( Date date )
        throws MessagingException
    {
        message.setSentDate( date );
        return this;
    }

    /** Define the content of the mail
     * <p>should be overidden by the sublcasses
     */
    public abstract Email setMsg(String msg)
        throws MessagingException;
    
    /**
     * Does the work of actually sending the email.
     *
     * @exception MessagingException if an error
     */
    public void send()
        throws MessagingException
    {
        InternetAddress[] foo = new InternetAddress[0];
        message.setRecipients(Message.RecipientType.TO,
                              toInternetAddressArray(toList));
        message.setRecipients(Message.RecipientType.CC,
                              toInternetAddressArray(ccList));
        message.setRecipients(Message.RecipientType.BCC,
                              toInternetAddressArray(bccList));
        message.setReplyTo(toInternetAddressArray(replyList));

	Thread t = new SendMessageThread(message);
	t.start();
    }
    
    /**
     * utility to copy Vector of known InternetAddress objects into an array.
     */
    private InternetAddress[] toInternetAddressArray(Vector v)
    {
        int size = v.size();
        InternetAddress[] ia = new InternetAddress[size];
        for (int i=0; i<size; i++)
        {
            ia[i] = (InternetAddress)v.elementAt(i);
        }
        return ia;
    }

    /**
     * Helper class for sending a message in the background.
     */
    private class SendMessageThread extends Thread { 
	private Message m;

	SendMessageThread(Message m) {
	    this.m = m;
	}

	public void run() {
	    try { 
		Address[] alist = m.getAllRecipients();
		for (int i = 0; i < alist.length; i++) { 
		    Log.note("Sending e-mail to: " + alist[i]);
		}
		Transport.send(m);
	    } catch (MessagingException e) { 
		Log.error("Error sending e-mail: " + e.getMessage());
	    }
	}
    }
}
