package org.apache.turbine.util.mail;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.net.*;
import java.util.*;

// Java Mail API
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

// Turbine Utility classes
import org.apache.turbine.util.db.Criteria;
import org.apache.turbine.services.resources.TurbineResources;
//import org.apache.turbine.util.mail.*;

//ECS Classes
import org.apache.ecs.*;
import org.apache.ecs.html.*;

/** An HTML multipart email
 * <p>This class is used to send HTML formatted email.
 * A text message can also be set for HTML unaware email
 * clients, such as text-based email clients.
 * <p>This class also inherit from MultiPartEmail, so it
 * is easy to add attachents to the email.
 * <p> To send an email in HTML, one should create a HtmlEmail,
 * then use the setFrom, addTo, etc. methods. The HTML content
 * can be set with the setHtmlMsg method. The alternate text
 * content can beset with setTextMsg.
 *
 * @author Regis Koenig
 */
public class HtmlEmail extends MultiPartEmail
{

    protected MimeMultipart htmlContent;
    protected MimeBodyPart msgText;
    protected MimeBodyPart msgHtml;

    /** Basic constructor
     */
    public HtmlEmail()
        throws MessagingException
    {
        this.init();
    }

    /** Initialize the email structure
     */
    protected void init()
        throws MessagingException
    {
        super.init();
    
        MimeMultipart msg = new MimeMultipart(); // the message in text & html form
        msg.setSubType("alternative");
        main.setContent(msg);

        /* The text part */ 
        msgText = new MimeBodyPart();
        msg.addBodyPart(msgText);

        /* The html part */
        htmlContent = new MimeMultipart();
        htmlContent.setSubType("related");
        MimeBodyPart bphtml = new MimeBodyPart();
        msg.addBodyPart(bphtml);
        bphtml.setContent(htmlContent);
    
        msgHtml = new MimeBodyPart();
        htmlContent.addBodyPart(msgHtml);
    }

    /** Set the alternate text content
     */
    public HtmlEmail setTextMsg(String text)
        throws MessagingException
    { 
        msgText.setText(text);
        return this;
    }

    /** Set the HTML content
     */
    public HtmlEmail setHtmlMsg(String html)
        throws MessagingException
    {
        msgHtml.setContent( html, TEXT_HTML);
        return this;
    }

    public HtmlEmail setHtmlMsg(org.apache.ecs.Document doc)
        throws MessagingException
    {
        return setHtmlMsg(doc.toString());
    }

    /** Set the message
     * <p>This method overrides the MultiPartEmail set Msg
     * method in order to send HTML message instead of full text
     * message in the mail body. The message is formatted in HTML
     * for the HTML part of the message, it is let as is in the
     * alternate text part.
     */
    public Email setMsg(String msg)
        throws MessagingException
    {
        setTextMsg(msg);
        setHtmlMsg(new ElementContainer(new Html(new Body().addElement(new PRE(msg)))).toString());
        return this;
    }

    /** Embeds an URL in the htnl
     * <p>This method allows to embed a file located by an URL
     * into the mail body. It allows, for instance, to add
     * inline images to the email. Inline files may be referenced
     * with a <code>cid:xxxxxx</code> URL, where xxxxxx is the Content-ID
     * returned by the embed function.
     * <p>Exemple of use: <br><code><pre>
     * HtmlEmail he = new HtmlEmail();
     * he.setHtmlMsg("&lt;html&gt;&lt;img src=cid:"+embed("file:/my/image.gif","image.gif")+"&gt;&lt;/html&gt;");
     * // code to set the others email fields (not shown)
     *
     * </pre></code>
     * @param url the URL of the file
     * @param name the name which will be set in the filename header field
     * @return the Content-ID of the file.
     */
    public String embed(URL url, String name)
        throws MessagingException
    {
        MimeBodyPart mbp = new MimeBodyPart();
        htmlContent.addBodyPart( mbp );

        mbp.setDataHandler ( new DataHandler( new URLDataSource(url) ) );
        mbp.setFileName( name );
        mbp.setDisposition("inline");
        String cid = org.apache.turbine.util.GenerateUniqueId.getIdentifier();
        mbp.addHeader("Content-ID", cid);
        return mbp.getContentID();
    }
    
}
