package org.apache.turbine.util.mail;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.net.*;
import java.util.*;

// Java Mail API
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

// Turbine Utility classes
import org.apache.turbine.util.db.Criteria;
import org.apache.turbine.services.resources.TurbineResources;
//import org.apache.turbine.util.mail.*;

/** A multipart email
 * <p>This class is used to send multi-part internet email
 * like messages with attachments.
 *
 * <p>To create a multi-part email, call the default constructor
 * and then you can call setMsg() to set the message and
 * call the different attach() methods.
 *
 * @author Jon S. Stevens
 * @author Frank Y. Kim
 * @author Brett McLaughlin
 * @author Regis Koenig
 */
public class MultiPartEmail extends Email
{

    /** Body portion of the email. */
    protected MimeMultipart emailBody;

    /** The message container */ 
    protected MimeBodyPart main;

    /** The file server if it exists */ 
    private String fileServer = null;

    /** Initialize the multipart email
     */
    public MultiPartEmail()
        throws MessagingException
    {
        this.init();
    }
    
    /** Constructor used to initialize attributes.
     * <p> This method uses the criteria object to set the different
     *   fields of the e-mail. The expected fields of the Criteria are:
     * <li>SENDER_EMAIL
     * <li>RECEIVER_EMAIL
     * <li>EMAIL_SUBJECT
     * <li>EMAIL_BODY
     * <li>ATTACHMENTS - a Vector of EmailAttachment
     * <li>FILE_SERVER - where the files are located. If not given, they are
     *   assumed to be local.
     *
     * @deprecated  since Criteria is deprecated in mail API
     */
    public MultiPartEmail( Criteria criteria )
        throws MessagingException
    {
        this.init();
        this.initCriteria(criteria);
    }

    /** Initialize the multipart email
     */
    protected void init()
        throws MessagingException
    {
        super.init();

        fileServer = null;
    
        /* The body of the mail */
        emailBody = new MimeMultipart();
        //emailBody.setSubType("attachement");
        message.setContent( emailBody );

        /* The main message content */
        main = new MimeBodyPart();
        emailBody.addBodyPart(main);
    }


    /** Uses the criteria to set the fields
     * <p> This methos uses the criteria object to set the different
     * fields of the e-mail. The expected fields of the Criteria are:
     * <li>SENDER_EMAIL
     * <li>RECEIVER_EMAIL
     * <li>EMAIL_SUBJECT
     * <li>EMAIL_BODY
     * <li>ATTACHMENTS - a Vector of EmailAttachment
     * <li>FILE_SERVER - where the files are located. If not given, they are
     *   assumed to be local.
     *
     * @deprecated since the Criteria is deprecated
     */
    protected void initCriteria( Criteria criteria )
        throws MessagingException
    {
        super.initCriteria(criteria);

        if( criteria.containsKey( EMAIL_BODY ) )
            setMsg( criteria.getString( EMAIL_BODY ) );
        else
            setMsg("NO MESSAGE");
    
        Vector attachments;
    
        if( criteria.containsKey(ATTACHMENTS ) )
            attachments = (Vector)criteria.get( ATTACHMENTS );
        else
            attachments = new Vector();
    
        if( criteria.containsKey(FILE_SERVER) )
            fileServer = criteria.getString(FILE_SERVER);
    
        for (int i=0; i<attachments.size(); i++)
        {
            EmailAttachment attachment = (EmailAttachment)attachments.elementAt(i);
            attach( attachment );
        }
    }

    /** Set the message of the email
     */
    public Email setMsg(String msg)
        throws MessagingException
    {
        main.setText(msg);
        return this;
    }

    /** attach an EmailAttachement
     */
    public MultiPartEmail attach( EmailAttachment attachment )
        throws MessagingException
    {
        URL url = attachment.getURL();
        if( url == null )
        {
            try
            {
                String file = attachment.getPath();
                url = new URL( "file", fileServer, file );
            }
            catch( Exception e)
            {
                throw new MessagingException("Cannot find file", e);
            }
        }
        
        return attach(url, attachment.getName(),
                      attachment.getDescription(), attachment.getDisposition() );
    }

    /** Attach a file located by its url.
     * <p> The disposition of the file is set to mixed
     */
    public MultiPartEmail attach( URL url, String name, String description)
        throws MessagingException
    {
        return attach( url, name, description, EmailAttachment.ATTACHMENT);
    }

    /** Attach a file located by its url
     * @param url the URL of the file (may be any valid URL)
     * @param name the name field for the attachment
     * @param description a description for the attachement
     * @param disposition either mixed or inline
     *
     */
    public MultiPartEmail attach( URL url, String name, String description, String disposition)
        throws MessagingException
    {

        MimeBodyPart mbp = new MimeBodyPart();
        emailBody.addBodyPart( mbp );
    
        mbp.setDisposition( disposition );
        mbp.setFileName ( name );
        mbp.setDescription ( description );
        mbp.setDataHandler ( new DataHandler( new URLDataSource(url) ) );

        return this;

    }
}
