package org.apache.turbine.util.upload;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
*    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.io.*;
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.*;
import org.apache.turbine.services.upload.*;
import org.apache.turbine.services.uniqueid.*;

/**
 * <p> This class represents a file that was received by Turbine using <code>multipart/form-data</code> POST request.
 * 
 * <p> After retrieving an instace of this class from the {@link org.apache.turbine.util.ParameterParser ParameterParser} 
 * (see {@link org.apache.turbine.util.ParameterParser#getFileItem(String) ParameterParser.getFileItem(String)} and 
 * {@link org.apache.turbine.util.ParameterParser#getFileItems(String) ParameterParser.getFileItems(String)})
 * you can use it to acces the data that was sent by the browser. You may either request all contents of file at once using
 * {@link #get()} or reuqest an {@link java.io.InputStream InputStream} with {@link #getStream()} and process the file without 
 * attempting to load it into the memory, which may come handy with large files.
 *
 * <p>
 *
 * @since 2.0-beta, added on 16 Aug 2000
 * @version $Id: FileItem.java,v 1.1 2000/08/30 21:08:56 bschneid Exp $
 * @author Rafal Krzewski, <a href="mailto:Rafal.Krzewski@e-point.pl">Rafal.Krzewski@e-point.pl</a>
 */
public class FileItem
{
    /** the maximal size of request that will have it's elements stored in memory */
    public static final int DEFAULT_UPLOAD_SIZE_THRESHOLD = 10240;

    /** the original filename in the user's filesystem. */
    protected String fileName;

    /** the content type passed by the browser or <code>null</code> if not defined. */
    protected String contentType;

    /** cached contents of the file. */
    protected byte[] content;

    /** temporary storage location. */
    protected File storeLocation;

    /** temporary storage for in-memory files */
    protected ByteArrayOutputStream byteStream;

    /** 
    * <p> Constructs a new <code>FileItem</code>.
    *
    * <p> Use {@link #newInstance(String,String,int)} to instantiate <code>FileItems</code>      
    *
    * @param fileaName the original filename in the user's filesystem.
    * @param the content type passed by the browser or <code>null</code> if not defined.
    */
    protected FileItem( String fileName, String contentType )
    {
        this.fileName = fileName;
        this.contentType = contentType;
    }

    /**
    * <p> Returns the original filename in the user's filesystem.
    * @return the original filename in the user's filesystem.
    */
    public String getFileName()
    {
        return fileName;
    }

    /**
    * <p> Returns the content type passed by the browser or <code>null</code> if not defined.
    *
    * @return the content type passed by the browser or <code>null</code> if not defined.
    */
    public String getContentType()
    {
        return contentType;
    }

    /**
    * <p> Provides a hint if the file contents will be read from memory.
    *
    * @return <code>true</code> if the file contents will be read from memory.
    */
    public boolean inMemory()
    {
        return (content != null || byteStream != null);        
    }

    /**
    * <p> Returns the size of the file.
    *
    * @return the size of the file.
    */
    public long getSize()
    {
        if(storeLocation != null)
        {
            return storeLocation.length();
        }
        else if(byteStream != null)
        {
            return byteStream.size();
        }
        else
        {
            return content.length;
        }
    }

    /**
    * <p> Returns the contents of the file as an array of bytes.
    *
    * <p> If the contents of the file were not yet cached int the memory, they will be
    * loaded from the disk storage and chached.<br>
    *
    * @return the contents of the file as an array of bytes.
    */
    public byte[] get()
    {
        if(content == null)
        {
            if(storeLocation != null)
            {
                content = new byte[(int)getSize()];
                try
                {
                    FileInputStream fis = new FileInputStream(storeLocation);
                    fis.read(content);
                } 
                catch (Exception e)
                {
                    content = null;
                }
            }
            else
            {
                content = byteStream.toByteArray();
                byteStream = null;
            }
        }
        return content;
    }

    /**
    * <p> Returns the contents of the file as a String, using default encoding.
    *
    * <p> This method uses {@link #get()} to retireve the contents of the file.<br>
    * 
    * @return The contents of the file.
    */
    public String getString()
    {
        return new String(get());
    }

    /**
    * <p> Returns the contents of the file as a String, using specified encoding.
    *
    * <p> This method uses {@link #get()} to retireve the contents of the file.<br>
    * 
    * @param encoding The encoding to use.
    * @return The contents of the file.
    */
    public String getString( String encoding ) throws UnsupportedEncodingException
    {
        return new String(get(), encoding);
    }

    /**
    * <p> Returns an {@link java.io.InputStream InputStream} that can be used to retrieve
    * the contents of the file.
    *
    * @return an {@link java.io.InputStream InputStream} that can be used to retrieve
    * the contents of the file.
    */
    public InputStream getStream() throws Exception
    {
        if(content == null)
        {
            if(storeLocation != null)
            {
                try
                {
                    return new FileInputStream(storeLocation);
                } 
                catch( FileNotFoundException e )
                {
                    throw new Exception("FileItem: stored item was lost");
                }
            }
            else
            {
                content = byteStream.toByteArray();
                byteStream = null;
            }
        }
        return new ByteArrayInputStream(content);
    }           

    /**
    * <p> Returns the {@link java.io.File} objects for the FileItems's data temporary location
    * on the disk. 
    * <p> Note that for <code>FileItems</code> that have their data stored in memory this method
    * will return <code>null</code>.
    * <p> When handling large files, you can use {@link java.io.File.renameTo(String)} to move
    * the file to new location without copying the data, if the source and destination locations
    * reside within the same logical volume.
    */
    public File getStoreLocation()
    {
        return storeLocation;
    }

    /**
    * <p> Removes the file contents from the temporary storage.
    */
    protected void finalize()
    {
        if(storeLocation != null && storeLocation.exists())
        {
            storeLocation.delete();
        }
    }

    /**
    * <p> Returns an {@link java.io.OutputStream OutputStream} that can be used for storing
    * the contensts of the file
    * @return an {@link java.io.OutputStream OutputStream} that can be used for storing
    * the contensts of the file
    */
    public OutputStream getOutputStream() throws IOException
    {
        if(storeLocation == null)
        {
            return byteStream;
        }
        else
        {
            return new FileOutputStream(storeLocation);
        }
    }

    /**
    * <p> Instantiates a FileItem. 
    *
    * <p> It uses <code>requestSize</code> to decide what temporary
    * storage approach should the new item take. The largest request that will have it's 
    * items cached in memory can be configured in <code>TurbineResources.properties</code>
    * in the entry named <code>file.upload.size.threshold</code>
    *
    * @param name the original filename in the user's filesystem.
    * @param contentType the content type passed by the browser or <code>null</code> if not defined.
    * @param requestSize the total size of the POST request this item belongs to.
    */
    public static FileItem newInstance(String path, String name, String contentType, int requestSize)
    {
        FileItem item = new FileItem(name, contentType);
        if(requestSize > TurbineUpload.getSizeThreshold())
        {
            String instanceName = TurbineUniqueId.getInstanceId();
            String fileName = TurbineUniqueId.getUniqueId();
            fileName = instanceName + "_upload_" + fileName + ".tmp";
            fileName = path + "/" + fileName;
            item.storeLocation = new File(fileName);
        }
        else
        {
            item.byteStream = new ByteArrayOutputStream();
        }
        return item;
    }
}

