package org.apache.turbine.util.webmacro;
    
/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// ECS Classes
import org.apache.ecs.*;
import org.apache.ecs.html.*;

// Turbine classes
import org.apache.turbine.util.*;

/**
 * WebMacro context tool that will set various attributes of the HTML page.
 *
 * It is automatically placed in the WebMacro context as '$page'.
 *
 * Here's an example of some uses:
 *
 * $page.setBgColor("#ffffff");
 * $page.setBgColor("white");
 * $page.setBackground("/images/standardbg.jpeg");
 * $page.setTitle("This is the title!");
 * $page.setKeywords("turbine, cool, servlet framework");
 * $page.setStyleSheet("/style.css");
 *
 * @author <a href="mailto:sean@somacity.com">Sean Legassick</a>
 */

public class WebMacroPageAttributes
{
    private RunData data = null;

    /** construct a new instance with the given RunData object */
    public WebMacroPageAttributes(RunData data)
    {
        this.data = data;
    }

    /** sets the TITLE of the page */
    public WebMacroPageAttributes setTitle(String title)
    {
        data.setTitle(title);
        return this;
    }

    /** retrieves the TITLE of the page */
    public String getTitle()
    {
        return data.getTitle();
    }
    
    /** adds a LINK to a CSS styleshet to the HEAD of the page */
    public WebMacroPageAttributes setStyleSheet(String url)
    {
        data.getPage().getHead().addElement(new Link()
                .setRel("stylesheet").setType("text/css").setHref(url));
        return this;
    }
    
    /** set a keywords META tag in the HEAD of the page */
    public WebMacroPageAttributes setKeywords(String keywords)
    {
       data.getPage().getHead().addElement(
               new Meta().setName("keywords").setContent(keywords));
       return this;
    }

    /** add a description META tag to the HEAD of the page */
    public WebMacroPageAttributes setDescription(String description)
    {
        data.getPage().getHead().addElement(
                new Meta().setName("description").setContent(description));
        return this;
    }
    
    /** set the background image for the BODY tag */
    public WebMacroPageAttributes setBackground(String url)
    {
        data.getPage().getBody().setBackground(url);
        return this;
    }

    /** set the background color for the BODY tag.
        you can use either color names or color values
        (e.g. "white" or "#ffffff" or "ffffff") */
    public WebMacroPageAttributes setBgColor(String color)
    {
        String hexColor = HtmlColor.getColor(color);
        if (hexColor == null)
        {
            hexColor = color;
        }
        data.getPage().getBody().setBgColor(hexColor);
        return this;
    }

    /** set the text color for the BODY tag.
        you can use either color names or color values
        (e.g. "white" or "#ffffff" or "ffffff") */
    public WebMacroPageAttributes setTextColor(String color)
    {
        String hexColor = HtmlColor.getColor(color);
        if (hexColor == null)
        {
            hexColor = color;
        }
        data.getPage().getBody().setText(hexColor);
        return this;
    }

    /** set the link color for the BODY tag. 
        you can use either color names or color values
        (e.g. "white" or "#ffffff" or "ffffff") */
    public WebMacroPageAttributes setLinkColor(String color)
    {
        String hexColor = HtmlColor.getColor(color);
        if (hexColor == null)
        {
            hexColor = color;
        }
        data.getPage().getBody().setLink(hexColor);
        return this;
    }

    /** set the visited link color for the BODY tag */
    public WebMacroPageAttributes setVlinkColor(String color)
    {
        String hexColor = HtmlColor.getColor(color);
        if (hexColor == null)
        {
            hexColor = color;
        }
        data.getPage().getBody().setVlink(hexColor);
        return this;
    }
    
    /** adds an attribute to the BODY tag */
    public WebMacroPageAttributes addAttribute(String name, String value)
    {
        data.getPage().getBody().addAttribute(name, value);  
        return this;
    }
        
    /** a dummy toString method that returns an empty string */
    public String toString()
    {
        return "";
    }
}
