-- calendar-core.sql
-- Core Calendar Data Model for new on-line calendar (ArsDigita Calendar)
-- 
-- Based on design by Joseph Croswell, Jorge Lopez, Orton Huang
-- 
-- acroswel@MIT.EDU, jlopez@MIT.EDU, kenshin@MIT.EDU

-- Rewritten by W. Scott Meeks (smeeks@arsdigita.com), June 2000
-- 
-- This file defines the core data model for calendar items independent 
-- of a particular view of calendars and items.

--- cal_categories table
create sequence cal_category_id_sequence start with 1 ;

create table cal_categories (
	category_id	integer primary key,
	-- if scope=public, this is a calendar category the whole system
        -- if scope=group, this is a calendar category for a particular group
	-- if scope=user, this is a calendar category for a user   
 	scope           varchar(20) not null,
	group_id	references user_groups,
	user_id		references users,
	category	varchar(100) not null,
	enabled_p	char(1) default 't' check(enabled_p in ('t','f')),
	constraint cal_categories_scope_ck check ((scope='group' and group_id is not null) or
						  (scope='user' and user_id is not null) or
						  (scope='public')),
	constraint cal_categories_scope_un unique(scope, category, group_id, user_id)
);

create sequence cal_item_id_sequence start with 1;

-- Table of base item information
-- Table of individual events or appointments.
create table cal_items (
        item_id			integer 
				constraint cal_itms_item_id_pk
				primary key,
	-- For subcommunities
--	site_spec_id	integer
--			constraint cal_items_site_spec_id_fk
--			references ad_site_specifications,
	-- short title to include in calendar view
        title           	varchar(700) not null,
        description     	varchar(4000),
	-- is the item description written in html
        html_p      		char(1) default 'f' 		
				constraint cal_itms_html_p_ck
				check(html_p in ('t','f')), 
	-- a url which points to a page related to the item
	-- this could be either additional detail or a substitution
	-- for the link on the calendar view, e.g. drill-down from absences
	-- calendar
        related_url     	varchar(4000),  
	related_url_text	varchar(4000),
	-- should related_url be used as the link on the calendar view
	related_url_p 		char(1) default 'f'
				constraint cal_itms_related_url_p_ck
				check(related_url_p in ('t','f')),
	-- reference to events_venues_id_sequence table in the event package
	-- which holds exact address information
	-- This may be superseded by venue party in calendar_item_map
--        venue_id        
--			constraint references events_venues,
	-- signifies event has been approved in cases when approval is needed
        approved_p      	char(1)
				constraint cal_itms_approved_p_ck
				check(approved_p in ('t','f')),
	approved_date		date,
        creation_user   	not null 
				constraint cal_itms_creation_user_fk
				references users(user_id),
	creation_date		date default sysdate,
	-- If an item was added by another package, e.g. reservations,
	-- may want to disallow editing by users. 
	-- An item can still be deleted by a user with write permission.
	editable_p		char(1)
				constraint cal_itms_editable_p_ck
				check(editable_p in ('t','f'))
);

create sequence cal_instance_id_sequence start with 1;

-- For repeating events, this table holds start and end time, title, 
-- description, and related_url for specific instances.  If the latter
-- three items are null, the value in the item is used.  This table also
-- provides an instance_id which can be used to attach instance specific
-- information (e.g. a file reference).
create table cal_item_instances (
	item_id 		integer
				constraint cal_item_inst_item_id_fk
				references cal_items,
	instance_id		integer
				constraint cal_item_inst_instance_id_pk
				primary key,
	-- item start and end dates.  if an item doesn't have an exact time
	-- (a birthday or holiday) then end_date should be null
        start_date     		date not null,
        end_date       		date,
	-- short title to include in calendar view
        title           	varchar(700),
        description     	varchar(4000),
	-- is the item description written in html
        html_p      		char(1) default 'f' 		
				constraint cal_item_inst_html_p_ck
				check(html_p in ('t','f')), 
	-- a url which points to a page related to the item
	-- this could be either additional detail or a substitution
	-- for the link on the calendar view, e.g. drill-down from absences
	-- calendar
        related_url     	varchar(4000),  
	related_url_text	varchar(4000),
	-- should related_url be used as the link on the calendar view
	related_url_p 		char(1) default 'f'
				constraint cal_itm_inst_related_url_p_ck
				check(related_url_p in ('t','f')),
        last_modifying_user   	not null 
				constraint cal_itm_inst_last_mod_user_fk
				references users(user_id),
	last_modified		date default sysdate not null
);
-- This index makes a significant difference in times for the 
-- month and week queries.
create index cal_item_inst_start_date_idx on cal_item_instances(start_date);

-- For repeating events, this table stores how the item repeats.
-- Repeating options are modeled on the Palm DateBook.
create table cal_repeat_items (
	item_id 		integer
				constraint cal_rpt_itms_item_id_pk
				primary key
				constraint cal_rpt_itms_item_id_fk
				references cal_items,
	-- Indicates what the type of interval between repeating 
	-- items is.  'day', 'week', and 'year' are straightforward.
	-- 'month_by_date' indicates a monthly interval where
	-- the item repeats on a specific date, e.g. the 26th.  'month_by_day'
	-- indicates a monthly interval based on the day of the week and the 
	-- week of the month, e.g. 2nd Tuesday of the month.  'last_of_month' 
	-- is a special indicating to repeat on the last day of the week 
	-- of the month, e.g. last Monday of the month.
	interval_type		varchar(20) default 'week'
				constraint cal_rpt_itms_interval_type_ck
				check(interval_type in ('day', 'week', 'month_by_date', 'month_by_day', 'last_of_month', 'year')),
	-- Indicates how many of the given intervals between repeats.
	every_nth_interval	integer default 1 not null,
	-- If repeating on a weekly basis (interval_type = 'week')
	-- indicates which days of the week the item repeats on.
	-- This is represented as a space separated list of numbers
	-- corresponding to days of the week, where 0 corresponds to
	-- Sunday, 1 to Monday, and so on.  Null indicates no days are set.  
	-- So for example, '1' indicates repeat on Mondays, '3 5' indicates
	-- repeat on Wednesday and Friday.
	days_of_week		varchar(20),
	-- Indicates when this item should stop repeating.  Null indicates
	-- repeat indefinitely.
	repeat_until		date,
	-- Indefinitely repeating items are partially prepopulated.  This
	-- column indicates up to what date this item has been repeated.  This
	-- allows further instances to be added if the user attempts to view
	-- a date beyond db_populated_until.  If 'until' is not null, then this
	-- column will always be the same date as 'until'.
	db_populated_until	date
);

-- Give events optional categories
-- (i.e. birthday, anniversary, etc)
-- This allows an event to have multiple categories
create table cal_category_map (
        instance_id     integer
			constraint cal_cat_map_instance_id_fk
			references cal_item_instances not null,
        category_id	constraint cal_cat_map_category_id_fk 
			references cal_categories
);

-- A list of groups that have calendars
-- This is needed for cases where a group calendar has been newly created.
-- It has no items, but we want it to show up in various lists.
create table cal_groups (
       group_id		integer 
			constraint cal_groups_group_id_pk
			references user_groups 
			not null 
			primary key
);

-- a list of group types that have calendars
create table cal_group_types (
       group_type       varchar(20)
                        constraint cal_groups_types_pk
			primary key
                        constraint cal_groups_types_group_type_fk
			references user_group_types(group_type)
);

-- Maps events to specific user's or group's calendars.  
-- If both user_id and group_id are set, then it indicates that
-- item_id is a user's item which should appear in the group's calendar.
-- If both are null, then the item is a 'public' item?
create table cal_item_map (
        user_id         integer 
			constraint cal_itm_map_user_id_fk
			references users,
        group_id        integer
			constraint cal_itm_map_group_id_fk
			references cal_groups,
        item_id		integer
			constraint cal_itm_map_item_id_fk
			references cal_items not null,
	-- Can't be primary key because user_id or group_id can be null
	constraint cal_itm_map_un
	unique(user_id, group_id, item_id)
);
create index cal_item_map_ids_idx on cal_item_map(item_id, user_id, group_id);

-- Maps between calendar items and arbitrary rows in other tables.
-- This is useful, e.g. for mapping to file attachments
create table cal_db_row_map (
	item_id		integer
			constraint cal_db_row_map_item_id_fk
			references cal_items not null,
	on_what_id	integer not null,
	on_which_table	varchar(30) not null
);

create table cal_instance_db_row_map (
	instance_id	integer
			constraint cal_inst_db_row_map_inst_id_fk
			references cal_item_instances not null,
	on_what_id	integer not null,
	on_which_table	varchar(30) not null
);

-- index for fast instance_id lookups. We often hit this mapping table
-- to find the instance_id
create unique index cal_instance_table_id on cal_instance_db_row_map(on_which_table, on_what_id, instance_id);


-- This view shows which item_ids a user
-- may write.
-- Permission is granted to administrators of
-- a group, owners of an item, and the
-- general public if there are no owners 
-- or groups.
create or replace view cal_user_write_privilege
as select distinct ci.item_id, ug.user_id
   from user_group_map ug, cal_item_map ci
   where ug.group_id = ci.group_id
   and ug.role = 'administrator'
   union
   select distinct ci.item_id, ci.user_id
   from cal_item_map ci
   where ci.user_id IS NOT null
   union
   select distinct ci.item_id, u.user_id
   from cal_item_map ci, users u
   where ci.user_id is null
         and ci.group_id is null;

-- This view shows which item_ids a user
-- may read.
-- Permission is granted to members of
-- a group, owners of an item, and the
-- general public if there are no owners
-- or groups.
create or replace view cal_user_read_privilege
as select distinct ci.item_id, ug.user_id
   from user_group_map ug, cal_item_map ci
   where ug.group_id = ci.group_id
   union
   select distinct ci.item_id, ci.user_id
   from cal_item_map ci
   where ci.user_id IS NOT null
   union
   select distinct ci.item_id, u.user_id
   from cal_item_map ci, users u
   where ci.user_id is null
         and ci.group_id is null;

-- Given an item_id and a date, inserts any needed instances
-- I think what this needs to support indefinite events is
-- to take another argument of the minimum # of repeats
-- So for indefinite items, this will be called with an up_to_date
-- one year (or some default) out and a default number of items.
-- For definite items, number of items will be 0.
-- The arguments are then set appropriate in the tcl code.
create or replace procedure cal_insert_instances 
	(v_item_id IN integer, 
	 v_last_modifying_user IN integer, 
	 up_to_date IN date,
	 num_items IN integer)
AS
  v_cal_repeat_item cal_repeat_items%ROWTYPE;
  v_type cal_repeat_items.interval_type%TYPE;
  v_n_intervals cal_repeat_items.every_nth_interval%TYPE;
  v_days_of_week cal_repeat_items.days_of_week%TYPE;
  -- Last date that has been populated.
  v_last_date_done date;
  -- Last date to be populated.
  v_stop_date date;
  v_start_date date;
  v_end_date date;
  -- Difference between start and end date.
  v_diff number;
  -- Current date to be inserted into DB.
  v_current_date date;
  -- Last day of month.
  v_last_day date;
  -- Calculated date of next weekly recurrence.
  v_week_date date;
  -- Count of instances added, to limit indefinite repeats.
  v_count integer;
  -- Length of days_of_week string
  v_length integer;
  -- Index within days_of_week
  v_index integer;
  -- Day number extracted from days_of_week
  v_day_num integer;
BEGIN
  select * into v_cal_repeat_item 
  from cal_repeat_items
  where item_id = v_item_id;

  -- Indefinitely repeating items only populated until passed in date.
  if v_cal_repeat_item.repeat_until is null
  then
    v_stop_date := up_to_date;
  else
    v_stop_date := v_cal_repeat_item.repeat_until;
  end if;

  select max(start_date), max(end_date) into v_start_date, v_end_date
  from cal_item_instances
  where item_id = v_item_id
  group by item_id;

  v_current_date := v_start_date;
  v_diff := v_end_date - v_start_date;

  if v_cal_repeat_item.db_populated_until IS NULL
  then
    v_last_date_done := v_start_date;
  else
    v_last_date_done := v_cal_repeat_item.db_populated_until;
  end if;

  v_n_intervals := v_cal_repeat_item.every_nth_interval;
  v_type := v_cal_repeat_item.interval_type;
  
  -- Week has to be handled specially.
  -- Start with the beginning of the week containing the start date.
  if v_type = 'week'
    then
      v_current_date := NEXT_DAY(v_current_date - 7, 'SUNDAY');
  end if;

  -- Check v_count to prevent runaway in case of error
  v_count := 0;
  while  v_count < 100000 and (trunc(v_last_date_done) <= trunc(v_stop_date) or v_count < num_items)
    loop
      v_count := v_count + 1;
      if v_type = 'day'
	then
	  v_current_date := v_current_date + v_n_intervals;
      elsif v_type = 'month_by_date'
	then
	  v_current_date := ADD_MONTHS(v_current_date, v_n_intervals);
      elsif v_type = 'month_by_day'
	then
	  -- Find last day of month before correct month
	  v_last_day := ADD_MONTHS(LAST_DAY(v_current_date), v_n_intervals - 1);
	  -- Find correct week and go to correct day of week
	  v_current_date := NEXT_DAY(v_last_day + (7 * (to_char(v_current_date, 'W') - 1)), to_char(v_current_date, 'DAY'));
      elsif v_type = 'last_of_month'
	then
	  -- Find last day of correct month
	  v_last_day := LAST_DAY(ADD_MONTHS(v_current_date, v_n_intervals));
	  -- Back up one week and find correct day of week
	  v_current_date := NEXT_DAY(v_last_day - 7, to_char(v_current_date, 'DAY'));
      elsif v_type = 'year'
	then
	  v_current_date := ADD_MONTHS(v_current_date, 12 * v_n_intervals);
      end if;

      -- Check to make sure we're not going past
      -- Trunc because dates aren't integral
      exit when trunc(v_current_date) > trunc(v_stop_date);

      if v_type = 'week'
	then
	  -- loop over days_of_week extracting each day number
	  -- add day number and insert
	  v_days_of_week := v_cal_repeat_item.days_of_week;
	  v_index := 1;
	  v_length := LENGTH(v_days_of_week);
	  while v_index <= v_length
	    loop
	      v_day_num := SUBSTR(v_days_of_week, v_index, 1);
	      v_week_date := v_current_date + v_day_num;
	      if trunc(v_week_date) > trunc(v_start_date) and 
	         trunc(v_week_date) <= trunc(v_stop_date)
		then
	          insert into cal_item_instances
      	          (item_id, instance_id, start_date, end_date, last_modifying_user)
      	          values
      	          (v_item_id, cal_instance_id_sequence.nextval, v_week_date, v_week_date + v_diff, v_last_modifying_user);
	      end if;
	      v_index := v_index + 2;
	  end loop;
	  -- Now move to next week with repeats.
	  v_current_date := v_current_date + 7 * v_n_intervals;
	  v_last_date_done := v_current_date - 1;
      else
      	insert into cal_item_instances
      	(item_id, instance_id, start_date, end_date, last_modifying_user)
      	values
      	(v_item_id, cal_instance_id_sequence.nextval, v_current_date, v_current_date + v_diff, v_last_modifying_user);
	v_last_date_done := v_current_date;
      end if;

  end loop;

  update cal_repeat_items
  set db_populated_until = up_to_date
  where item_id = v_item_id;
END;
.
/
show errors

-- takes as input the id of the calendar instance, plus the desired separator (<br>, comma, whatever)
create or replace function combine_category_names (v_instance_id IN integer, v_separator IN varchar)
return varchar
IS
TYPE cur_type IS REF CURSOR;
c cur_type;
my_query VARCHAR2(4000);
v_category_name VARCHAR2(100);
v_return_string VARCHAR2(4000);
BEGIN
	my_query := 'select distinct category
                                from cal_category_map ccm, cal_categories cc
                                where ccm.instance_id = :v_instance_id
                                and ccm.category_id = cc.category_id
	  	 	        order by category';
	OPEN c FOR my_query USING v_instance_id;
	FETCH c INTO v_category_name;
	v_return_string := v_category_name;
	LOOP
		FETCH c INTO v_category_name;
		EXIT when c%NOTFOUND;
		v_return_string := v_category_name || v_separator || v_return_string;
	END LOOP;
	return v_return_string;
END;
/
show errors
