-- calendar-view.sql
--
-- Data model for implementing a particular view on calendar items.
--
-- This data model supports displaying lists of items, and a calendar
-- formatted by day, week, month or year


-- For displaying specific blocks of time, e.g. 24 hours, 7 days, 31 days
-- it's useful to return a set number of rows from a generic table.  The
-- worst case needed by calendar is 42 to display a calendar that spans
-- six complete weeks, so we can just generate a table of that fixed size.
-- Otherwise we'd probably want a function that could generate an arbitrary
-- number of rows.

-- Helper table for returning a fixed number of rows.  char(1) should minimize
-- memory used.
create table cal_generic_rows (
	a_row	char(1)
);

-- Populate the helper table if it needs it.
create or replace procedure insert_cal_generic_rows (num_rows IN integer)
AS
  v_count integer;  
BEGIN
  select count(*) into v_count from cal_generic_rows;

  while v_count < num_rows
    loop
      insert into cal_generic_rows values (NULL);
      v_count := v_count + 1;
  end loop;
END;
.
/
show errors

execute insert_cal_generic_rows(42);

-- Three common views 
create or replace view cal_hours as
  select (rownum - 1) as hour
  from cal_generic_rows
  where rownum <= 24;

create or replace view cal_week_days as
  select (rownum - 1) as day_num
  from cal_generic_rows
  where rownum <= 7;

create or replace view cal_month_days as
  select (rownum - 1) as day_num
  from cal_generic_rows
  where rownum <= 42;

-- maps alerts on events to specific user's or group's calendars.  Each 
-- row in this table has either user_id or group_id null
-- Email alerts are supported using General Alerts.
create table cal_alerts (
        user_id		integer 
			constraint cal_alerts_user_id_fk
			references users,
        group_id	integer 
			constraint cal_alerts_group_id_fk
			references cal_groups,
        item_id		integer 
			constraint cal_alerts_item_id_fk
			references cal_items not null,
	-- Floating point number of days before item to alert
        advance_time	number default 0 not null,
        type		varchar(10) 
			constraint cal_alerts_type_ck
			check(type in ('email', 'instant', 'palm', 'online'))
			not null
	-- For email alerts
--	alert_id	integer
--			constraint cal_alerts_alert_id_fk
--			references general_alerts
);

-- an individual user's preferences for his personal calendar
create table cal_user_prefs (
        user_id			constraint cal_u_prfs_user_id_pk
				references users primary key,
	-- default view on login. Set to last view selected.
        default_view            varchar(10)
				constraint cal_u_prfs_default_view_ck
				check(default_view in ('list', 'day', 'week', 'month', 'quarter', 'year', 'custom')),
	list_new_p		char(1) default 't' 
				constraint cal_u_prfs_list_new_p_ck
				check(list_new_p in ('t', 'f')) not null,
	list_upcoming_p		char(1) default 't' 
				constraint cal_u_prfs_list_upcmng_p_ck
				check(list_upcoming_p in ('t', 'f')) not null,
	-- Relative order to display lists, lower = higher on page
	new_order		integer default 0 not null,
	upcoming_order		integer default 1 not null,
	-- Age in days of new item before it's no longer new
	new_items_age		integer default 3 not null,
	-- Number of upcoming items to present in list view
	upcoming_items_length	integer default 10 not null,
	-- Number days of upcoming items to present in list view
	upcoming_items_days	integer default 1 not null,
	-- Default length for items in minutes
	-- 0 or null indicates no time by default
	default_duration	integer default 60,
	-- Controls the span of hours displayed in day and week views
	-- It is specfied as HH24HH24, e.g. 0717 represents 7am-5pm and 
	-- 0023 represents 24 hours
	hours_span		varchar(4),
	compress_day_view_p	char(1)
				constraint cal_u_prfs_cmprs_dy_vw_p_ck
				check(compress_day_view_p in ('t', 'f')),
        size_type		varchar(10) default 'percentage' 
				constraint cal_u_prfs_size_type_ck
				check(size_type in ('percentage', 'pixels'))
				not null,
        size_value		integer default '90' not null,
        -- Automatically show lists of users or groups with viewing 
	-- or editing permissions on individual's calendar.
        viewers_p		char(1) default 'f' 
				constraint cal_u_prfs_viewers_p_ck
				check(viewers_p in ('t', 'f')) not null,
        editors_p		char(1) default 'f' 
				constraint cal_u_prfs_editors_p_ck
				check(editors_p in ('t', 'f')) not null,
	-- Which day to start weeks on, usually either Sunday or Monday
	first_day_of_week	varchar(8) default 'Sunday',
	-- Default value for update_type when user is added to group
       update_type		varchar(13) default 'automatically' 
				constraint cal_u_prfs_update_type_ck
				check(update_type in ('email', 'automatically', 'ask online')) not null
);

--insert old users into cal_user_prefs
insert into cal_user_prefs (user_id)
select user_id from users u where not exists
 (select 1 from cal_user_prefs c where c.user_id = u.user_id);

-- an individual's peferences about how his personal calendar is
-- updated with group events
-- In the current UI, on an individual calendar, there are links to all group
-- calendars that exist for groups the user is a member of.  If update_type is 
-- automatically, then events are placed on calendar.  If 'ask online' then
-- events are not on calendar, but user can click a link to add them.
-- If 'none' then the group doesn't appear on the front page, but can be
-- added back on a user prefs page.
create table cal_user_update_prefs (
	user_id		integer 
			constraint cal_usr_updt_prfs_user_id_fk
			references users not null,
	group_id		integer 
			constraint cal_usr_updt_prfs_group_id_fk
			references cal_groups not null,
	-- Users can be added to groups with multiple roles.
	-- We don't care for purposes of update_type, but need to keep
	-- track for removal triggers.
	role		varchar(200),
	update_type	varchar(13) 
			constraint cal_usr_updt_prfs_updt_typ_ck
			check(update_type in ('none', 'automatically', 'ask online', 'email')) not null,
	constraint cal_usr_updt_prfs_pk
	primary key(user_id, group_id, role)
);

-- This table sets up filters between a user's calendar
-- and various group calendars.
-- If the user is a member of the group, then this is used
-- to do simple filtering where only items of the matching category are
-- displayed. If the user is not a member of the group, then only 'public'
-- items of the particular category will be visible.
create table cal_user_item_filter (
	user_id		integer 
			constraint cal_usr_itm_fltr_user_id_fk
			references users not null,
	group_id	integer 
			constraint cal_usr_itm_fltr_group_id_fk
			references cal_groups,
        category_id	constraint cal_usr_itm_fltr_cat_id_fk
			references cal_categories
);	

-- When a user is added to a group with a calendar, this trigger
-- adds a row for the default value of showing the group's events on the
-- individual's calendar
create or replace trigger cal_groups_insert
before insert on user_group_map
for each row
declare
  num integer;
  v_update_type cal_user_prefs.update_type%TYPE;
begin
  select count(*) into num from cal_groups 
	where group_id = :new.group_id
	  and not exists (select * from cal_user_update_prefs
			  where user_id = :new.user_id
			    and group_id = :new.group_id
			    and role = :new.role);
  if num > 0 then
    select update_type into v_update_type 
	from cal_user_prefs
	where user_id = :new.user_id;
    insert into cal_user_update_prefs (user_id, group_id, role, update_type)
    values (:new.user_id, :new.group_id, :new.role, v_update_type);
  end if;
end;
/
show errors

-- When a calendar is added for a group, this trigger
-- adds a row for the default value of showing the group's events on the
-- individual's calendar
create or replace trigger cal_groups_add
after insert on cal_groups
for each row
begin
  insert into cal_user_update_prefs 
	select ugm.user_id, ugm.group_id, ugm.role,
	nvl(p.update_type, 'automatically')
	from user_group_map ugm, cal_user_prefs p
	where ugm.group_id = :new.group_id
	  and ugm.user_id = p.user_id(+)
	  and not exists (select * from cal_user_update_prefs
			  where user_id = ugm.user_id
			    and group_id = ugm.group_id
			    and role = ugm.role);
end;
/
show errors

-- Do we need a cal_groups_delete trigger?

-- When a user is removed for a group, this trigger removes the
-- corresponding row in cal_user_update_prefs
create or replace trigger cal_groups_remove
before delete on user_group_map
for each row
begin
  delete from cal_user_update_prefs 
  where user_id = :old.user_id
    and group_id = :old.group_id
    and role = :old.role;
end;
/
show errors

-- When a user is added, this trigger adds a row in
-- cal_user_prefs
create or replace trigger cal_user_add
after insert on users
for each row
begin
  insert into cal_user_prefs
	(user_id)
	values
	(:new.user_id);
end;
/
show errors

-- When a user is removed, this trigger removes the 
-- corresponding row in cal_user_prefs
create or replace trigger cal_user_remove
before delete on users
for each row
begin
  delete from cal_user_prefs
  where user_id = :old.user_id;
end;
/
show errors

